/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package cat.lib.cript;

import cat.lib.errors.ErrorEx;
import cat.lib.math.RandomEx;
import cat.lib.utils.NumberToString;
import cat.lib.utils.StringToNumber;
import cat.lib.utils.StringUtils;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 *
 * @author Miquel
 * 
 * versio 2
 */
public class CriptUtils {

    private static final int DEFAULT_SHA1_DIGEST_LENGTH = 20;

    
    //   ______                  _
    //  |  ____)                (_)
    //  | |___ _   _ _ __   ____ _  ___  _ __  ___    _ __   ___ _ __      _ _
    //  |  ___) | | | `_  \/ ___) |/ _ \| `_  \ __)  | `_ \ / _ \ `__)   / _` |
    //  | |   | |_| | | | | (___| | (_) | | | |__ \  | |_) |  __/ |     | (_| |
    //  |_|    \__,_|_| |_|\____)_|\___/|_| |_|___/  |  __/ \___|_|      \__,_|
    //                                               | |
    //                                               |_|
    //                                                                   _
    //                         _       _                               _
    //                        (_)     | |                             | |
    //    ___ _ __   ____ _ __ _ _ __ | |_  _ _ _ __     ____  _ _  __| | ___ _ __   ___ ___
    //   / _ \ `_  \/ ___) `__) | `_ \| __) _` | `__)   / ___) _` |/ _` |/ _ \ `_  \/ _ \ __)
    //  |  __/ | | | (___| |  | | |_) | |_ (_| | |     | (___ (_| | (_| |  __/ | | |  __/__ \
    //   \___|_| |_|\____)_|  |_|  __/ \__)__,_|_|      \____)__,_|\__,_|\___|_| |_|\___|___/
    //                          | |
    //                          |_|

    public static String shaCript (String text, String password, int SHA1_DIGEST_LENGTH, ErrorEx error) {

        StringBuilder res = new StringBuilder();

        // Valida paàmetres
        if (StringUtils.isNotEmpty (text)) {
            if (StringUtils.isNotEmpty (password)) {

                try {
                    MessageDigest sha = MessageDigest.getInstance("SHA-1");

                    // Els paràmetres són les cadenes de text que cal encriptar i s'incorporen
                    // a la classe MessageDigest amb el métode "upDate()".
                    byte[] textTable = new byte[text.length()];
                    textTable = text.getBytes();
                    sha.update (textTable);

                    byte[] passwordTable = new byte[password.length()];
                    passwordTable = password.getBytes();

                    // Encripta les cadenes obtingudes amb els paràmetres i les encripta amb la clau
                    byte[] cadenaHash = sha.digest (passwordTable);

                    // Al encriptat les cadenes, s'obté un array de bytes que cal convertir-los a una cadena
                    // de text amb els corresponents valors hexadecimals.
                    int num = 0;
                    String hex = null;
                    for (int cont1=0; cont1<SHA1_DIGEST_LENGTH; cont1++) {

                       num = (int) cadenaHash[cont1];
                       if (num<0) { num+=256; } // Evita els valors negatius

                       // Passa el número a hexadecimal
                       hex = Integer.toHexString (num);
                       if (hex.length()<2) { res.append("0"); }

                       res.append (hex.toUpperCase());
                    }
                }
                catch (NoSuchAlgorithmException ex) { error.setError ("CRIPT_ERROR", "DESCRIPTION: No se ha encontrado el algoritmo de encriptación."); }
            }
            else { error.setError("CRIPT_ERROR", "DESCRIPTION: No s'han indicat la clau d'encriptació"); }
        }
        else { error.setError("CRIPT_ERROR", "DESCRIPTION: No s'han indicat el text a encriptar"); }

        return res.toString();
    }

    public static String shaCript (String text, String password, int SHA1_DIGEST_LENGTH) { return shaCript (text, password, SHA1_DIGEST_LENGTH, new ErrorEx()); }
    public static String shaCript (String text, String password) { return shaCript (text, password, DEFAULT_SHA1_DIGEST_LENGTH, new ErrorEx()); }

    public static String md5Cript (String text, ErrorEx error) {

        StringBuffer res = null;

        if (StringUtils.isNotEmpty(text)) {
            try {
                MessageDigest md = MessageDigest.getInstance("MD5");

                byte[] b = md.digest (text.getBytes());
                int size = b.length;

                res = new StringBuffer (size);

                for (int cont1=0; cont1<size; cont1++) {

                    int value = b[cont1] & 255;
                    if (value<16) { res.append ("0"); }
                    res.append (Integer.toHexString(value));
                }
            }
            catch (NoSuchAlgorithmException e) { error.setError ("CRIPT_ERROR", "DESCRIPTION: No se ha encontrado el algoritmo de encriptación.");}
        }

        if (res==null) { return null; }
        return res.toString();
    }

    public static String md5Cript (String text) { return md5Cript (text, new ErrorEx()); }


    /**
     * Encripta una cadena de text utilitzant el llibre de claus "book"
     *
     * @param text Cadena de text a encriptar
     * @param book LLibre de claus a utilitzar
     * @param error
     * @return
     */
    public static String bookCript (String text, String book, ErrorEx error) {

        String res = null;

        if (!StringUtils.isEmpty(text) && !StringUtils.isEmpty(book)) {

            StringBuilder buffer = new StringBuilder (text.length());

            for (int cont1=0; cont1<text.length(); cont1++) {

                char caract = text.charAt(cont1);
                int startPos = RandomEx.random (book.length()-1);
                int trobat = -1;
                int pos = startPos;
                while (trobat==-1 && error.getError()==null) {
                    if (book.charAt(pos)==caract) { trobat = pos; }
                    pos++;
                    if (pos>=book.length()) { pos = 0; }
                    if (pos==startPos) { error.setError ("CRIPT_ERROR", "DESCRIPTION: No se ha podido encriptar la clave indicada."); }
                }

                String codi = NumberToString.toString(trobat);
                while (codi.length()<3) { codi = "0"+codi; }
                buffer.append (codi);
            }
            res = buffer.toString();
        }

        return res;
    }

    /**
     * Encripta una cadena de text utilitzant el llibre de claus per defecte
     * 
     * @param text Cadena de text a encriptar
     * @param error 
     * @return
     */
    public static String bookCript (String text, ErrorEx error) {

        String clau = "ì ñ'eren çync zambors, què ànáwéñ a la gùërrâ, y el més xíkuets de tòts pôrtawä un zâm de rôsetês; ho pòbre xïqùet, kue t'häs fët màl a lá vúmwa ï te l'hâs trençada sóta el pêù í l'aixädór: î töca êl joü qûè ël joc ès fos. 13051796408268457253597092389174580"+
                      "Ì Ñ'EREN ÇYNC ZAMBORS, QUÈ ÀNÁWÈÑ A LA GÙËRRÂ, Y EL MÉS XÍKUETS DE TÒTS PÔRTAWÄ UN ZÂM DE RÔSETÊS; HO PÒBRE XÏQÙET, KUE T'HÄS FËT MÀL A LÁ VÚMWA Ï TE L'HÂS TRENÇADA SÓTA EL PÊÙ Í L'AIXÄDÓR: Î TÖCA ÊL JOÜ QÛÈ ËL JOC ÊS FOS. <;¿,:[._}^ç>]¨^-/*¨{º/-`+º(&´ªª)`!%·´$%!&=/()=??$¿^·¨[]{}|@#€<>"+
                      "Hi havia una vegada, una criatura molt bonica amb moltes ganes de jugar, ¿ quantes pilotes portava la criatura ? 1 o 2, potser eren 3 o 4 i qui sap si arribaven a 5 o 6; alguns mal pensats creien que en portava 7 o 8 pero en realitat en portava 9, ja que són més que zero.";
        return bookCript (text, clau, error);
    }

    public static String bookCript (String text) { return bookCript (text, new ErrorEx()); }
    
    /**
     * Desencripta una cadena de text utilitzant el llibre de claus "book"
     *
     * @param text Cadena de text a desencriptar
     * @param book LLibre de claus a utilitzar
     * @param error
     * @return
     */
    public static String bookUncript (String text, String book, ErrorEx error) {

        String res = null;

        if (!StringUtils.isEmpty(text) && !StringUtils.isEmpty(book)) {

            StringBuilder buffer = new StringBuilder (text.length());

            int cont1 = 0;
            while (cont1<text.length()) {

                String codi = StringUtils.copy(text, cont1, 3);
                cont1+=3;
                
                int caract = StringToNumber.intValue(codi);
                
                if (caract>=book.length()) { error.setError ("CRIPT_ERROR", "DESCRIPTION: No se ha podido desencriptar el texto."); }
                buffer.append((char) book.charAt(caract));
                
                
            }
            res = buffer.toString();
        }

        return res;
    }

    /**
     * Desencripta una cadena de text utilitzant el llibre de claus per defecte
     * 
     * @param text Cadena de text a desencriptar
     * @param error 
     * @return
     */
    public static String bookUncript (String text, ErrorEx error) {

        String clau = "ì ñ'eren çync zambors, què ànáwéñ a la gùërrâ, y el més xíkuets de tòts pôrtawä un zâm de rôsetês; ho pòbre xïqùet, kue t'häs fët màl a lá vúmwa ï te l'hâs trençada sóta el pêù í l'aixädór: î töca êl joü qûè ël joc ès fos. 13051796408268457253597092389174580"+
                      "Ì Ñ'EREN ÇYNC ZAMBORS, QUÈ ÀNÁWÈÑ A LA GÙËRRÂ, Y EL MÉS XÍKUETS DE TÒTS PÔRTAWÄ UN ZÂM DE RÔSETÊS; HO PÒBRE XÏQÙET, KUE T'HÄS FËT MÀL A LÁ VÚMWA Ï TE L'HÂS TRENÇADA SÓTA EL PÊÙ Í L'AIXÄDÓR: Î TÖCA ÊL JOÜ QÛÈ ËL JOC ÊS FOS. <;¿,:[._}^ç>]¨^-/*¨{º/-`+º(&´ªª)`!%·´$%!&=/()=??$¿^·¨[]{}|@#€<>"+
                      "Hi havia una vegada, una criatura molt bonica amb moltes ganes de jugar, ¿ quantes pilotes portava la criatura ? 1 o 2, potser eren 3 o 4 i qui sap si arribaven a 5 o 6; alguns mal pensats creien que en portava 7 o 8 pero en realitat en portava 9, ja que són més que zero.";
        return bookUncript (text, clau, error);
    }
    
    public static String bookUncript (String text) { return bookUncript (text, new ErrorEx()); }

    public static void main (String[] args) throws Exception {

        String text = "abcdefghijklmnopqrstuvwxyzçñáàääèéêëìíîïòóôöùúûü0123456789"+
                      "ABCDEFGHIJKLMNOPQRSTUVWXYZÇÑÀÁÂÄÉÈÊËÌÍÎÏÒÓÔÖÙÚÛÜºª!·$%&/()=?¿[]{}`+´ç^¨`´<>,.-;:_";
        String clau = "ì ñ'eren çync zambors, què ànáwéñ a la gùërrâ, y el més xíkuets de tòts pôrtawä un zâm de rôsetês; ho pòbre xïqùet, kue t'häs fët màl a lá vúmwa ï te l'hâs trençada sóta el pêù í l'aixädór: î töca êl joü qûè ël joc ès fos. 13051796408268457253597092389174580"+
                      "Ì Ñ'EREN ÇYNC ZAMBORS, QUÈ ÀNÁWÈÑ A LA GÙËRRÂ, Y EL MÉS XÍKUETS DE TÒTS PÔRTAWÄ UN ZÂM DE RÔSETÊS; HO PÒBRE XÏQÙET, KUE T'HÄS FËT MÀL A LÁ VÚMWA Ï TE L'HÂS TRENÇADA SÓTA EL PÊÙ Í L'AIXÄDÓR: Î TÖCA ÊL JOÜ QÛÈ ËL JOC ÊS FOS. <;¿,:[._}^ç>]¨^-/*¨{º/-`+º(&´ªª)`!%·´$%!&=/()=??$¿^·¨[]{}|@#€<>";
        System.out.println (text);

        String cripted = bookCript (text, clau, new ErrorEx());
        System.out.println (cripted);

        String uncripted = bookUncript (cripted, clau, new ErrorEx());
        System.out.println (uncripted);

    }
}
