//    _____ _                       _  _  ____         _        ____                 _  _
//   / ____) |                     | || ||  __ \      | |      |  __ \              | || |
//  | |    | |  _ _ ___ ___  ___   |_||_|| |  | |  _ _| |_  _ _| |__) |  _ _ ___  ____||_|
//  | |    | |/ _` | __) __)/ _ \        | |  | |/ _` | __) _` |  __ < / _` | __)/ _ \
//  | |____| | (_| |__ \__ \  __/        | |__| | (_| | |_ (_| | |__) | (_| |__ \  __/
//   \_____)_|\__,_|___/___/\___|        |_____/ \__,_|\__)__,_|_____/ \__,_|___/\___|
//
//

package cat.lib.dataBase;

import cat.lib.errors.ErrorEx;
import cat.lib.files.LogsFile;
import cat.lib.logs.Loggable;
import cat.lib.utils.StringUtils;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

/**
 *
 * @author Miquel Abadal i Cervià
 */
public class DataBase {

    public final static String DRIVER_NOT_FOUND = "DRIVER_NOT_FOUND";
    public final static String CONNECT_ERROR = "CONNECT_ERROR";
    public final static String NOT_CONNECTED_ERROR = "NOT_CONNECTED_ERROR";
    public final static String DISCONNECT_ERROR = "DICONNECT_ERROR";
    public final static String SQL_ERROR = "SQL_ERROR";
    public final static String UNREGISTER_DRIVER_ERROR = "UNREGISTER_DRIVER_ERROR";


    LogsFile logs = null;
    int unCommitTransactions = 0;
    boolean autoCommit = true;
    boolean autoReconnect = true;
    
    private String driver = "";
    private String server = "";
    private String schema = "";
    private String username = "";
    private String password = "";

    private boolean connected = false;
    private boolean driverLoaded = false;

    Connection connection = null;


    public DataBase () { }

    public void connect (ErrorEx error) {

        if (connected) { disconnect(error); }

        if (error.getError()==null) {

            if (driverLoaded) {
                System.out.println ("Connectant amb la base de dades...");
                System.out.println ("Driver: "+driver);
                System.out.println ("Server: "+server);
                System.out.println ("Schema: "+schema);
                System.out.println ("Username: "+username);
                System.out.println ("Password: ********");

                try {
                    
                    java.sql.Driver mySqlDriver = DriverManager.getDriver (server);
                    DriverManager.registerDriver(mySqlDriver);
                    
                    connection = DriverManager.getConnection (server+"/"+schema, username, password);
                    
                    
                    connection.setAutoCommit (false);

                    connected = true;

                } catch (SQLException e) {
                    error.setError (CONNECT_ERROR, "ACTION:CONNECT", "TRACE:DataBase.connect", "DRIVER: "+driver, "SERVER: "+server, "DESCRIPTION: Imposible conectarse a la base de datos", "MESSAGE: "+e.getMessage());
                    connected = false;
                    connection = null;
                }
            }
            else { error.setError (CONNECT_ERROR, "ACTION:CONNECT", "TRACE:DataBase.connect", "DESCRIPTION: Driver no cargado", "DRIVER:"+driver); }
        }

    }

    public void disconnect (ErrorEx error) {

        if (connected) {
            try {
                connection.close ();
                connected = false;
                connection = null;

            } 
            catch (SQLException e) { error.setError (DISCONNECT_ERROR, "ACTION:DISCONNECT", "TRACE:DataBase.disconnect", "DESCRIPTION: Se ha producido un error al intentar desconectar la base de datos", "MESSAGE:"+e.getMessage ()); }
        }
        else { error.setError (DISCONNECT_ERROR, "ACTION:DISCONNECT", "TRACE:DataBase.disconnect", "DESCRIPTION: La base de datos no está conectada"); }

    }

    /**
     * Métode que permet "desregistrar" la connexió amb el servidor ODBC de dades.
     *
     * Aquest procés és necessari per a permetre al contenidor de Tomcat deployar aplicacions correctament.
     */
    public void removeDriver (ErrorEx error) {

        if (driverLoaded) {
            if (connected) { disconnect (error); }
            if (error.getError()==null) {
                try {
                    java.sql.Driver mySqlDriver = DriverManager.getDriver (server);
                    DriverManager.deregisterDriver(mySqlDriver);
                }
                catch (SQLException ex) { error.setError (UNREGISTER_DRIVER_ERROR, "ACTION:REMOVE_DRIVER", "TRACE:DataBase.removeDriver", "DESCRIPTION: No se ha podido de-registrar el driver ODBC", "DRIVER:"+driver, "EXCEPTION:"+ex.getMessage()); }
            }
        }
    }

    public void commit (ErrorEx error) {
        Query query = new Query (this);
        query.commit (error);
    }

    public void rollback () {
        Query query = new Query (this);
        query.rollback();
    }

    public void setDriver (String driver, ErrorEx error) {

        if (connected) { disconnect(error); }

        if (error.getError()==null) {
            driverLoaded = false;

            if (!StringUtils.isEmpty (driver)) {
                try {
                    Class.forName (driver);
                    this.driver = driver;
                    driverLoaded = true;
                }
                catch (ClassNotFoundException e) { error.setError (DRIVER_NOT_FOUND, "ACTION:SET_DRIVER", "TRACE:DataBase.setDriver", "DRIVER:"+driver, "DESCRIPTION: No se ha podido cargar el driver", "MESSAGE: "+e.getMessage()); }
            }
            else  { error.setError(DRIVER_NOT_FOUND, "ACTION:SET_DRIVER", "TRACE:DataBase.setDriver", "DRIVER: Null or empty", "DESCRIPTION: No se ha indicado el nombre del driver a cargar", "MESSAGE: Null driver"); }
        }
    }


    public void setServer (String server) { this.server = server; }
    public void setSchema (String schema) { this.schema =  schema; }
    public void setUsername (String username) { this.username = username; }
    public void setPassword (String password) { this.password =  password; }
    public void setAutoCommit (boolean autoCommit) { this.autoCommit = autoCommit; }
    public void setAutoReconnect (boolean autoReconnect) { this.autoReconnect = autoReconnect; }

    public void setLogsFile (LogsFile logs) { this.logs = logs; }

    public String getDriver () { return driver; }
    public String getServer () { return server; }
    public String getSchema () { return schema; }
    public String getUsername () { return username; }
    public String getPassword () { return password; }
    public int getUnCommitTransactions () { return unCommitTransactions; }
    public boolean getAutoCommit () { return autoCommit; }
    public boolean getAutoReconnect () { return autoReconnect; }

    public Connection getConnection () { return connection; }
    public boolean isAutoCommit () { return autoCommit; }
    public boolean isConnected () { return connected; }


}
