//    _____ _                       _  _    __                        _  _
//   / ____) |                     | || | / __ \                     | || |
//  | |    | |  _ _ ___ ___  ___   |_||_|| |  | |_   _  ___ _ __ _   __||_|
//  | |    | |/ _` | __) __)/ _ \        | |  | | | | |/ _ \ `__) | | |
//  | |____| | (_| |__ \__ \  __/        | |__| | |_| |  __/ |  | |_| |
//   \_____)_|\__,_|___/___/\___|         \___\_\\__,_|\___|_|   \__, |
//                                                                __/ |
//                                                               (___/


package cat.lib.dataBase;

import cat.lib.errors.ErrorEx;
import cat.lib.math.MathEx;
import cat.lib.strings.ParamsList;
import cat.lib.utils.NumberToString;
import cat.lib.utils.SetUtils;
import cat.lib.utils.StringUtils;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Date;

/**
 *
 * @author Miquel Abadal i Cervià
 */
public class Query {

    private final int UNKNOW_ORDER = -1;
    private final int QUERY_ORDER = 0;
    private final int UPDATE_ORDER = 1;
    private final int OTHER_ORDER = 2;
    private final int COMMIT_ROLLBACK_ORDER = 3;

    private final int GOTO_BEFORE_FIRST = 0;
    private final int GOTO_FIRST = 1;
    private final int GOTO_PREVIOUS = 2;
    private final int GOTO_NEXT = 3;
    private final int GOTO_LAST = 4;
    private final int GOTO_AFTER_LAST = 5;
    private final int GOTO_NUM_ROW = 6;

    private final int IS_BEFORE_FIRST = 0;
    private final int IS_FIRST = 1;
    private final int IS_LAST = 2;
    private final int IS_AFTER_LAST = 3;

    DataBase dataBase = null;
    String sql = "";
    ResultSet resultSet = null;
    PreparedStatement stmt = null;
    int affectedRows = 0;
    int numRows = 0;
    char paramStartDelimiter = ':';
    char paramEndDelimiter = ':';


    public Query (DataBase dataBase) { this.dataBase = dataBase; }

    @Override
    protected void finalize () throws Throwable  {
        close();
        dataBase = null;
        sql = null;
        resultSet = null;

        try { super.finalize(); }
        catch (Exception e) { }
    }

    // Elimina els comentaris de la sentència SQL, tenint en compte
    // que una sentencia SQL pot ocupar varies linies i per tant també pot tenir un
    // comentari per línia
    // I tenint en compte que els caracter "--" que delimiten un comentari, s'ha
    // d'ignorar si estàn dintre d'un literal delimitat per cometes dobles o simples.
    private String extractComments (String text) {

        if (text==null) { return ""; }

        boolean onLiteral = false;
        boolean onComment = false;
        char literalChar ='\0';

        StringBuilder res = new StringBuilder();

        char caract = '\0';
        char nextCaract = '\0';

        for (int cont1=0; cont1<text.length(); cont1++) {

            caract = text.charAt(cont1);
            if (cont1+1<text.length()) { nextCaract = text.charAt(cont1+1); }
            else { nextCaract = '\0'; }

            if (onComment) {
                if (caract=='\n') { onComment = false; }
            }
            else {
                if (onLiteral) {
                    if (caract==literalChar) { onLiteral = false; }
                    res.append (caract);
                }
                else {
                    if (caract=='"' || caract=='\'') {
                        onLiteral = true;
                        literalChar = caract;
                        res.append (caract);
                    }
                    else if (caract=='-' && nextCaract=='-') { onComment = true; }
                    else { res.append (caract); }

                }
            }
        }

        return res.toString();

    }

    private void execSql (String sql, ErrorEx error) {

        if (sql!=null) { sql = extractComments (sql).trim(); }

        if (!StringUtils.isEmpty(sql)) {
            //sql = StringUtils.substChar(sql, '"', '\'', StringUtils.SUBST_ALL, 0, StringUtils.CASE_SENSITIVE);

            close();
            resultSet = null;
            stmt = null;
            affectedRows = -1;
            numRows = -1;

            if (dataBase!=null && dataBase.isConnected()) {
                try {
                    stmt = dataBase.connection.prepareStatement(sql);

                    String order = StringUtils.firstWord (sql);
                    order = order.toLowerCase();

                    int orderType = UNKNOW_ORDER;
                    if (SetUtils.stringIn (order, false, "delete", "update", "insert", "create", "drop", "load")) { orderType = UPDATE_ORDER; }
                    if (SetUtils.stringIn (order, false, "commit", "rollback")) { orderType = COMMIT_ROLLBACK_ORDER; }
                    if (SetUtils.stringIn (order, false, "show", "select", "describe")) { orderType = QUERY_ORDER; }
                    if (SetUtils.stringIn (order, false, "begin", "end", "use", "alter", "declare", "set", "mysqldump")) { orderType = OTHER_ORDER; }

                    if (order.equalsIgnoreCase("use")) {
                        dataBase.setSchema (StringUtils.copy (sql, order.length()+1, -1));
                    }

                    if (orderType==OTHER_ORDER) {
                        stmt.executeUpdate();
                    }
                    else if(orderType == UPDATE_ORDER) {
                        affectedRows = stmt.executeUpdate();
                        if (affectedRows>0) { dataBase.unCommitTransactions++; }
                    }
                    else if (orderType==COMMIT_ROLLBACK_ORDER) {
                        affectedRows = stmt.executeUpdate();
                        dataBase.unCommitTransactions = 0;
                    }
                    else if (orderType==QUERY_ORDER) {
                        resultSet = stmt.executeQuery();
                        resultSet.last();
                        numRows = resultSet.getRow();
                        resultSet.first();
                    }
                    else { error.setError (DataBase.SQL_ERROR, "ACTION:EXEC_SQL", "TRACE:Query.execSql", "SQL: "+sql, "DESCRIPTION: No se ha reconocido la órden SQL", "MESSAGE:UNKNOW_ORDER", "DESCRIPTION: Orden desconocida"); }

                } catch (SQLException e) { 
                    String message = e.getMessage();
                    if (StringUtils.stringPos(message, "Connection was implicitly closed by the driver", StringUtils.FROM_BEGIN, StringUtils.CASE_INSENSITIVE)!=-1) {
                         error.setError (DataBase.NOT_CONNECTED_ERROR, "ACTION:EXEC_SQL", "TRACE:Query.execSql", "DESCRIPTION: Se ha perdido la conexión con el SGBD", "SQL: "+sql, "MESSAGE:"+message); 
                         dataBase.disconnect(error);
                    }
                    else { error.setError (DataBase.SQL_ERROR, "ACTION:EXEC_SQL", "TRACE:Query.execSql", "DESCRIPTION: Se ha producido un error al ejecutar la sentencia SQL", "SQL: "+sql, "MESSAGE:"+message); }
                }

            }
            else { error.setError (DataBase.SQL_ERROR, "ACTION:EXEC_SQL", "TRACE:Query.execSql", "DESCRIPTION: No se pueden realizar consultas sin conectar a la base de datos", "SQL: "+sql, "MESSAGE:NOT_CONNECTED"); }

            String logMessage;
            logMessage = "[SQL] "+sql;
            if (affectedRows!=-1) logMessage = logMessage+"\n[AFFECTED "+affectedRows+" ROWS]";
            if (resultSet!=null) logMessage = logMessage+"\n[RETURNED "+numRows+" ROWS]";
            if (error.getError()==null) { logMessage = logMessage+"\n[RESULT OK]"; }

            if (dataBase!=null && dataBase.logs!=null) { dataBase.logs.sql (logMessage); }
            else { System.out.println (logMessage); }
        }
    }


    public void execQuery (String sql, ErrorEx error) {

        ParamsList script = new ParamsList (sql, ';');

        int cont1 = 0;
        while (error.getError()==null && cont1<script.paramCount()) {
            execSql (script.getParam (cont1), error);
            
            // Si la base de dades està desconnectada per culpa del SGBD i està activada la opció de reconnectar automàticament
            // intenta reconnectar, i si es completa, torna a executar la sentència SQL.
            if (error.getError()!=null && error.getError().equals (DataBase.NOT_CONNECTED_ERROR)) {
                if (dataBase.getAutoReconnect()) {
                    error.clearErrors();
                    dataBase.connect (error);
                    if (error.getError()==null) { execSql (script.getParam (cont1), error); }
                }
            }
            cont1++;
        }

        if (error.getError()==null && dataBase.autoCommit==true && dataBase.unCommitTransactions>0) { execSql ("commit", error); }
    }

    public void execQuery (ErrorEx error) { execQuery (sql, error); }

    public void commit (ErrorEx error) { execSql ("commit", error); }
    public void rollback () { execSql ("rollback", new ErrorEx()); }

    public void close () {
        try {
            if (stmt!=null) { stmt.close(); }
            if (resultSet!=null) {  resultSet.close(); }
        }
        catch (SQLException e) {  }        
    }


    public void setSql (String sql) { this.sql = sql; }
    public String getSql () { return sql; }

    public int getAffectedRows () { return affectedRows; }
    public int getNumRows () { return numRows; }

    public void setParamStartDelimiter (char delimiter) { paramStartDelimiter = delimiter; }
    public void setParamEndDelimiter (char delimiter) { paramEndDelimiter = delimiter; }

    public DataBase getDataBase () { return dataBase; }

    public Cell fieldByName (String columnName, ErrorEx error) {

        // Obté el contingut del camp en format text.
        String value = null;
        
        try {  if (resultSet!=null) { value = resultSet.getString(columnName); } }
        catch (SQLException e) { error.setError (DataBase.SQL_ERROR, "ACTION:FIELD_BY_NAME", "TRACE:Query.fieldByName", "DESCRIPTION: No se ha podido obtener el campo \""+columnName+"\"", "MESSAGE:"+e.getMessage()); }

        Cell field = new Cell (columnName, value);
        return field;
    }

    public Cell fieldByColumn (int column, ErrorEx error) {

        // Obté el contingut del camp en format text.
        String value = null;

        try {  if (resultSet!=null) { value = resultSet.getString(column+1); } }
        catch (SQLException e) { error.setError (DataBase.SQL_ERROR, "ACTION:FIELD_BY_COLUMN", "TRACE:Query.fieldByColumn", "DESCRIPTION: No se ha podido obtener el campo \""+NumberToString.toString (column)+"\"", "MESSAGE:"+e.getMessage()); }

        Cell field = new Cell (NumberToString.toString (column), value);
        return field;
    }

    public ResultSetMetaData getMetaData (ErrorEx error) {
        ResultSetMetaData res = null;
        try { if (resultSet!=null) { res = resultSet.getMetaData(); } }
        catch (SQLException e) { error.setError (DataBase.SQL_ERROR, "ACTION:GET_META_DATA", "TRACE:Query.getMetaData", "DESCRIPTION: No se han podido obtener los atributos MetaData", "MESSAGE:"+e.getMessage()); }
        return res;
    }

    private String rowTo (String beginRow, String endRow, String beginField, String endField, char simbolDecimal, ErrorEx error) {

        StringBuilder res = new StringBuilder();
        if (beginRow!=null) { res.append (beginRow); }

        try {
            if (resultSet!=null) {
                int numColumns = resultSet.getMetaData().getColumnCount();
                for (int cont1=1; cont1<=numColumns; cont1++) {

                    if (beginField!=null) { res.append (beginField); }

                    int type = resultSet.getMetaData().getColumnType(cont1);
                    String value = resultSet.getString(cont1);
                    if (type==Types.DOUBLE || type==Types.DECIMAL || type==Types.FLOAT || type==Types.REAL) {
                        value = StringUtils.substChar (value, '.', simbolDecimal, StringUtils.SUBST_ONE, StringUtils.FROM_BEGIN, StringUtils.CASE_SENSITIVE);
                    }
                    res.append (value);

                    if (endField!=null) { res.append (endField); }
                }
            }
        }
        catch (SQLException e) { error.setError (DataBase.SQL_ERROR, "ACTION:ROW_TO", "TRACE:Query.rowTo", "DESCRIPTION: No se ha podido obtener el registro", "MESSAGE:"+e.getMessage()); }

        if (endRow!=null) { res.append (endRow); }
        return res.toString();
    }
    
    private String[] rowToArray (char simbolDecimal, ErrorEx error) {

        String[] res = null;
        try {
            if (resultSet!=null) {
                int numColumns = resultSet.getMetaData().getColumnCount();
                res = new String[numColumns];
                
                for (int cont1=1; cont1<=numColumns; cont1++) {

                    int type = resultSet.getMetaData().getColumnType(cont1);
                    String value = resultSet.getString(cont1);
                    if (type==Types.DOUBLE || type==Types.DECIMAL || type==Types.FLOAT || type==Types.REAL) {
                        value = StringUtils.substChar (value, '.', simbolDecimal, StringUtils.SUBST_ONE, StringUtils.FROM_BEGIN, StringUtils.CASE_SENSITIVE);
                    }
                    res[cont1-1]=value;
                }
            }
        }
        catch (SQLException e) { error.setError (DataBase.SQL_ERROR, "ACTION:ROW_TO", "TRACE:Query.rowTo", "DESCRIPTION: No se ha podido obtener el registro", "MESSAGE:"+e.getMessage()); }

        return res;
    }

    private String headerTo (String beginRow, String endRow, String beginField, String endField, ErrorEx error) {

        StringBuilder res = new StringBuilder();
        if (beginRow!=null) { res.append (beginRow); }

        try {
            if (resultSet!=null) {
                int numColumns = resultSet.getMetaData().getColumnCount();
                for (int cont1=1; cont1<=numColumns; cont1++) {

                    if (beginField!=null) { res.append (beginField); }
                    res.append (resultSet.getMetaData().getColumnName(cont1));
                    if (endField!=null) { res.append (endField); }
                }
            }
        }
        catch (SQLException e) { error.setError (DataBase.SQL_ERROR, "ACTION:HEADER_TO", "TRACE:Query.headerTo", "DESCRIPTION: No se ha podido obtener el registro", "MESSAGE:"+e.getMessage()); }

        if (endRow!=null) { res.append (endRow); }
        return res.toString();
    }
    
    

    public String rowToString (ErrorEx error) { return rowTo (null, "\r\n", null, " ", '.', error); }
    public String rowToHtml (ErrorEx error) { return rowTo ("<tr>", "</tr>", "<td>", "</td>", '.', error); }
    public String rowToCsv (ErrorEx error) { return rowTo (null, "\r\n", null, ";", ',', error); }
    public String[] rowToArray (ErrorEx error) { return rowToArray ('.', error); }

    public String headerToString (ErrorEx error) { return headerTo (null, "\r\n", null, " ", error); }
    public String headerToHtml (ErrorEx error) { return headerTo ("<tr>", "</tr>", "<td>", "</td>", error); }
    public String headerToCsv (ErrorEx error) { return headerTo (null, "\r\n", null, ";", error); }
    
    public String[] headerToArray (ErrorEx error) {

        String[] res = null;

        try {
            if (resultSet!=null) {
                int numColumns = resultSet.getMetaData().getColumnCount();
                res = new String[numColumns];
                
                for (int cont1=1; cont1<=numColumns; cont1++) {
                    res[cont1-1] = resultSet.getMetaData().getColumnName(cont1);
                }
            }
        }
        catch (SQLException e) { error.setError (DataBase.SQL_ERROR, "ACTION:HEADER_TO", "TRACE:Query.headerTo", "DESCRIPTION: No se ha podido obtener el registro", "MESSAGE:"+e.getMessage()); }

        return res;
    }

    private boolean moveCurrentRow (int action, int numRow, ErrorEx error) {

        boolean res = false;
        try {
            if (resultSet!=null && !resultSet.isClosed()) {
                switch (action) {
                    case GOTO_BEFORE_FIRST  : resultSet.beforeFirst(); break;
                    case GOTO_FIRST         : res = resultSet.first(); break;
                    case GOTO_PREVIOUS      : res = resultSet.previous(); break;
                    case GOTO_NEXT          : res = resultSet.next(); break;
                    case GOTO_LAST          : res = resultSet.last(); break;
                    case GOTO_AFTER_LAST    : resultSet.afterLast(); break;
                    case GOTO_NUM_ROW       : res = resultSet.absolute (numRow+1); break;
                }
            }
            else { error.setError (DataBase.SQL_ERROR, "ACTION:EXEC_MOVE_CURRENT_ROW", "TRACE:Query.moveCurrentRow", "MESSAGE: RESULTSET_NULL_OR_CLOSED", "DESCRIPTION: La consulta está vacia o cerrada"); }
        } catch (SQLException e) { error.setError (DataBase.SQL_ERROR, "ACTION:EXEC_MOVE_CURRENT_ROW", "TRACE:Query.moveCurrentRow", "MESSAGE:"+e.getMessage(), "DESCRIPTION: Se ha producido un error al intentar accedir al registro '"+numRow+"'"); }

        return res;
    }

    private boolean moveCurrentRow (int action, ErrorEx error) {
        return moveCurrentRow (action, 0, error);
    }

    public boolean beforeFirst (ErrorEx error) { return moveCurrentRow (GOTO_BEFORE_FIRST, error); }
    public boolean first (ErrorEx error) { return moveCurrentRow (GOTO_FIRST, error); }
    public boolean previos (ErrorEx error) { return moveCurrentRow (GOTO_PREVIOUS, error); }
    public boolean next (ErrorEx error) { return moveCurrentRow (GOTO_NEXT, error); }
    public boolean last (ErrorEx error) { return moveCurrentRow (GOTO_LAST, error); }
    public boolean afterLast (ErrorEx error) { return moveCurrentRow (GOTO_AFTER_LAST, error); }
    public boolean toRow (int numRow, ErrorEx error) { return moveCurrentRow (GOTO_NUM_ROW, numRow, error); }


    private boolean getCurrentRow (int action, ErrorEx error) {

        boolean res = false;
        try {
            switch (action) {
                case IS_BEFORE_FIRST  : if (resultSet!=null && !resultSet.isClosed()) { res = resultSet.isBeforeFirst(); }
                                        else { res = true; }
                                        break;
                case IS_FIRST         : if (resultSet!=null && !resultSet.isClosed()) { res = resultSet.isFirst(); }
                                        break;
                case IS_LAST          : if (resultSet!=null && !resultSet.isClosed()) { res = resultSet.isLast(); }
                                        break;
                case IS_AFTER_LAST    : if (resultSet!=null && !resultSet.isClosed()) { res = resultSet.isAfterLast(); }
                                        else { res = true; }
                                        break;
            }
        } catch (SQLException e) { error.setError (DataBase.SQL_ERROR, "ACTION:GET_MOVE_CURRENT_ROW", "TRACE:Query.getCurrentRow", "MESSAGE:"+e.getMessage(), "DESCRIPTION: No se ha podido acceder al registro seleccionado"); }

        return res;
    }


    public boolean isBeforeFirst (ErrorEx error) { return getCurrentRow (IS_BEFORE_FIRST, error); }
    public boolean isFirst (ErrorEx error) { return getCurrentRow (IS_FIRST, error); }
    public boolean isLast (ErrorEx error) { return getCurrentRow (IS_LAST, error); }
    public boolean isAfterLast (ErrorEx error) { return getCurrentRow (IS_AFTER_LAST, error); }

    public boolean bof (ErrorEx error) { return (getCurrentRow (IS_BEFORE_FIRST, error) || numRows==0); }
    public boolean eof (ErrorEx error) { return (getCurrentRow (IS_AFTER_LAST, error) || numRows==0); }
    public boolean isEmpty (ErrorEx error) { return (numRows<=0); }

    private void setParamByName (String paramName, String value) {
        if (paramName!=null) {
            if (paramName.length()>0 && paramName.charAt (0)!=paramStartDelimiter) paramName = paramStartDelimiter+ paramName;
            if (paramEndDelimiter!=0 && paramName.length()>0 && paramName.charAt (paramName.length()-1)!=paramEndDelimiter) paramName = paramName+paramEndDelimiter;

            if (value!=null) { sql = StringUtils.substString (sql, paramName, value, StringUtils.SUBST_ALL, 0, false); }
            else { sql = StringUtils.substString (sql, paramName, "null", StringUtils.SUBST_ALL, 0, false); }
        }
    }

    public void paramByName (String paramName, String value) {
        if (paramName!=null) {
            if (value!=null) { 
                value = StringUtils.duplicaCometesSimples (value);
                setParamByName (paramName, "'"+value+"'");
            }
            else { setParamByName (paramName, value); }
        }
    }

    public void paramByName (String paramName, char value) {
        paramByName (paramName, NumberToString.toString(value));
    }

    public void paramByName (String paramName, byte value) {
        setParamByName (paramName, NumberToString.toString(value));
    }

    public void paramByName (String paramName, short value) {
        setParamByName (paramName, NumberToString.toString(value));
    }

    public void paramByName (String paramName, int value) {
        setParamByName (paramName, NumberToString.toString(value));
    }

    public void paramByName (String paramName, long value) {
        setParamByName (paramName, NumberToString.toString(value));
    }

    public void paramByName (String paramName, float value) {
        if (Float.isNaN(value)) { setParamByName (paramName, null); }
        else { setParamByName (paramName, NumberToString.toString(value)); }
    }

    public void paramByName (String paramName, float value, int numDecimals) {
        if (Float.isNaN(value)) { setParamByName (paramName, null); }
        else { setParamByName (paramName, NumberToString.toString (MathEx.round (value, numDecimals))); }
    }

    public void paramByName (String paramName, double value) {
        if (Double.isNaN(value)) { setParamByName (paramName, null); }
        else { setParamByName (paramName, NumberToString.toString(value)); }
    }

    public void paramByName (String paramName, double value, int numDecimals) {
        if (Double.isNaN(value)) { setParamByName (paramName, null); }
        else { setParamByName (paramName, NumberToString.toString (MathEx.round (value, numDecimals))); }
    }

    public void paramByName (String paramName, Date value) {
        if (value!=null) { setParamByName (paramName, "'"+NumberToString.toString (value, "yyyy-MM-dd HH:mm:ss")+"'"); }
        else { setParamByName (paramName, null); }
    }
    
    public void paramByName (String paramName, boolean value) {
        setParamByName (paramName, "'"+NumberToString.toStringSN(value)+"'");
    }

    public void clear() {
        close(); 
        dataBase = null;
        sql = null;
        resultSet = null;
    }

}
