//    _____ _                       _  _  ____         _        ______ _ _     _  _
//   / ____) |                     | || ||  __ \      | |      |  ____)_) |   | || |
//  | |    | |  _ _ ___ ___  ___   |_||_|| |  | |  _ _| |_  _ _| |___  _| | ____||_|
//  | |    | |/ _` | __) __)/ _ \        | |  | |/ _` | __) _` |  ___)| | |/ _ \
//  | |____| | (_| |__ \__ \  __/        | |__| | (_| | |_ (_| | |    | | |  __/
//   \_____)_|\__,_|___/___/\___|        |_____/ \__,_|\__)__,_|_|    |_|_|\___|
//
//


package cat.lib.files;

import cat.lib.errors.ErrorEx;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 *
 * @author Miquel Abadal i Cervià
 */
public class DataFile extends GenericFile {

    //    _____                _              _          _          _        _ _           _                   _        _   _
    //   / ____)              | |            | |        (_)        | |      (_) |         | |                 | |      | | (_)
    //  | |     ___  _ __  ___| |_  _ _ _ __ | |_ ___    _      _ _| |_ _ __ _| |_   _   _| |_ ___     ___ ___| |_  _ _| |_ _  ____ ___
    //  | |    / _ \| `_  \ __) __) _` | `_  \ __) __)  | |   / _` | __) `__) | `_ \| | | | __) __)   / _ \ __) __) _` | __) |/ ___) __)
    //  | |____ (_) | | | |__ \ |_ (_| | | | | |_\__ \  | |  | (_| | |_| |  | | |_) | |_| | |_\__ \  |  __/__ \ |_ (_| | |_| | (___\__ \
    //   \_____)___/|_| |_|___/\__)__,_|_| |_|\__)___/  |_|   \__,_|\__)_|  |_|_,__/ \__,_|\__)___/   \___|___/\__)__,_|\__)_|\____)___/
    //
    //


    private static final int BUFFER = 8192;


    //     __   _        _ _           _
    //   / __ \| |      (_) |         | |
    //  | |__| | |_ _ __ _| |_   _   _| |_ ___
    //  |  __  | __) `__) | `_ \| | | | __) __)
    //  | |  | | |_| |  | | |_) | |_| | |_\__ \
    //  |_|  |_|\__)_|  |_|_,__/ \__,_|\__)___/
    //
    //

    private DataOutputStream outFile = null;
    private FileInputStream inFile = null;
    private DataInputStream dataFile = null;


    //   __  __      _            _                              _                    _
    //  |  \/  |    | |          | |                            | |                  | |
    //  | \  / | ___| |_ ___   __| | ___     ____ ___  _ __  ___| |_ _ __ _   _  ____| |_ ___  _ __
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \   / ___) _ \| `_  \ __) __) `__) | | |/ ___) __) _ \| `__)
    //  | |  | |  __/ |_ (_) | (_| |  __/  | (___ (_) | | | |__ \ |_| |  | |_| | (___| |_ (_) | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|   \____)___/|_| |_|___/\__)_|   \__,_|\____)\__)___/|_|
    //
    //

    public DataFile () {
        outFile = null;
        dataFile = null;
        inFile = null;
    }


    //   __  __      _            _                     _     _ _
    //  |  \/  |    | |          | |                   | |   | |_)
    //  | \  / | ___| |_ ___   __| | ___    _ __  _   _| |_  | |_  ____ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \  | `_ \| | | | `_ \| | |/ ___) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/  | |_) | |_| | |_) | | | (___\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|  |  __/ \__,_|_,__/|_|_|\____)___/
    //                                     | |
    //                                     |_|

    /**
     * Obre per a la lectura el fitxer "fileName".
     *
     */
    public void reset (String fileName, ErrorEx error) {

        if (status==ST_CLOSED) {
            if (fileName!=null) {
                try {
                    inFile = new FileInputStream (fileName);
                    dataFile = new DataInputStream (inFile);

                    if (dataFile!=null) {
                        status = ST_READ_ONLY;
                        this.fileName = fileName;
                    }
                    else { error.setError ("IOEXCEPTION", "ACTION: RESET", "TRACE:FileEx.reset", "DESCRIPTION: CANT_OPEN_FILE", "FILENAME: "+fileName); }
                }
                catch (FileNotFoundException ex) { error.setError ("IOEXCEPTION", "ACTION: RESET", "TRACE:FileEx.reset", "DESCRIPTION: FILE_NOT_FOUND", "FILENAME:"+fileName, "EXCEPTION:"+ex.getMessage()); }
            }
            else { error.setError ("IOEXCEPTION", "ACTION: RESET", "TRACE:FileEx.reset", "DESCRIPTION: NULL_FILENAME"); }
        }
        else { error.setError ("IOEXCEPTION", "ACTION: RESET", "TRACE:FileEx.reset", "DESCRIPTION: FILE_ALREEADY_OPENED"); }

    }

    /**
     * Obre per a la escripturael fitxer "fileName".
     *
     */
    public void rewrite (String fileName, ErrorEx error) {

        if (status==ST_CLOSED) {
            if (fileName!=null) {
                try {
                    outFile = new DataOutputStream (new FileOutputStream(fileName));
                    if (outFile!=null) {
                        status = ST_WRITE_ONLY;
                        this.fileName = fileName;
                    }
                    else { error.setError ("IOEXCEPTION", "ACTION: RESET", "TRACE:FileEx.reset", "DESCRIPTION: CANT_CREATE_FILE", "FILENAME: "+fileName); }
                }
                catch (FileNotFoundException ex) { error.setError ("IOEXCEPTION", "ACTION: REWRITE", "TRACE:FileEx.rewrite", "DESCRIPTION: WRITE_FILE_ERROR", "EXCEPTION:"+ex.getMessage()); }
            }
            else { error.setError ("IOEXCEPTION", "ACTION: REWRITE", "TRACE:FileEx.rewrite", "DESCRIPTION: NULL_FILENAME"); }
        }
        else { error.setError ("IOEXCEPTION", "ACTION: REWRITE", "TRACE:FileEx.rewrite", "DESCRIPTION: FILE_ALREADY_OPENED"); }

    }

    /**
     * Escriu en el fitxer el text "text".
     *
     * El fitxer ha d'estar obert per a escriptura (veure "rewrite");
     *
     */
    public void writeLine (String text) {

        if (status==ST_WRITE_ONLY) {
            try { outFile.writeBytes (text); }
            catch (IOException ex) { }
        }
    }

    /**
     * Escriu en el fitxer el text "text".
     *
     * El fitxer ha d'estar obert per a escriptura (veure "rewrite");
     *
     */
    public void writeLine (String text, ErrorEx error) {

        if (status==ST_WRITE_ONLY) {
            try { outFile.writeBytes(text);
            }
            catch (IOException ex) { error.setError ("IOEXCEPTION", "ACTION: WRITE", "TRACE:FileEx.write", "DESCRIPTION: "+ex.getMessage()); }
        }
        else { error.setError ("IOEXCEPTION", "ACTION: WRITE", "TRACE:FileEx.write", "DESCRIPTION: FILE_IS_READONLY"); }

    }


    public void blockWrite (byte[] data, int numWrite, ErrorEx error) {

        if (status==ST_WRITE_ONLY) {
            try { outFile.write (data, 0, numWrite); }
            catch (IOException ex) { error.setError ("IOEXCEPTION", "ACTION: WRITE", "TRACE:FileEx.write", "DESCRIPTION: "+ex.getMessage()); }
        }
        else { error.setError ("IOEXCEPTION", "ACTION: WRITE", "TRACE:FileEx.write", "DESCRIPTION: FILE_IS_READONLY"); }

    }

    public void blockWrite (byte[] data, ErrorEx error) { blockWrite (data, data.length, error); }

    public int blockRead (byte[] data, ErrorEx error) {

        int numRead = 0;
        if (status==ST_READ_ONLY) {
            try { numRead = dataFile.read (data); }
            catch (IOException ex) { error.setError ("IOEXCEPTION", "ACTION: READ", "TRACE:FileEx.read", "DESCRIPTION: "+ex.getMessage()); }
        }
        else { error.setError ("IOEXCEPTION", "ACTION: READ", "TRACE:FileEx.read", "DESCRIPTION: FILE_IS_WRITE_ONLY"); }

        return numRead;
    }

    public InputStream readAsStream() { return dataFile; }

    /*  NOTA: Aquest métode llegeix tot el fitxer i el passa a un String */
    public String read (ErrorEx error) {

        StringBuilder res = new StringBuilder();

        if (status==ST_READ_ONLY) {

            byte[] buffer = new byte[BUFFER];
            int numRead = 0;

            try {
                numRead = dataFile.read (buffer);
                while (numRead!=-1) {
                    for (int cont1=0; cont1<numRead; cont1++) {
                        res.append((char) buffer[cont1]);
                    }
                    numRead = dataFile.read (buffer);
                }
            }
            catch (IOException ex) { error.setError ("IOEXCEPTION", "ACTION: READ", "TRACE:FileEx.read", "DESCRIPTION: "+ex.getMessage()); }
        }
        else { error.setError ("IOEXCEPTION", "ACTION: READ", "TRACE:FileEx.read", "DESCRIPTION: FILE_IS_WRITE_ONLY"); }

        return res.toString();
    }


    public void close (ErrorEx error) {

        if (status!=ST_CLOSED) {
            try { 
                if (outFile!=null) { outFile.close(); }
                if (dataFile!=null) { dataFile.close(); }
                inFile = null;
                outFile = null;
                dataFile = null;
                fileName = null;
            }
            catch (IOException ex) { error.setError ("IOEXCEPTION", "ACTION: CLOSE", "TRACE:FileEx.close", "DESCRIPTION: "+ex.getMessage()); }
        }
        else { error.setError ("IOEXCEPTION", "ACTION: CLOSE", "TRACE:FileEx.close", "DESCRIPTION: FILE_ALREADY_CLOSED"); }
        status = ST_CLOSED;
    }


}
