//    _____ _                       _  _  ______ _ _      _    _ _   _ _    _  _
//   / ____) |                     | || ||  ____)_) |    | |  | | | (_) |  | || |
//  | |    | |  _ _ ___ ___  ___   |_||_|| |___  _| | ___| |  | | |_ _| |____||_|
//  | |    | |/ _` | __) __)/ _ \        |  ___)| | |/ _ \ |  | | __) | | __)
//  | |____| | (_| |__ \__ \  __/        | |    | | |  __/ |__| | |_| | |__ \
//   \_____)_|\__,_|___/___/\___|        |_|    |_|_|\___|\____/ \__)_|_|___/
//
//


package cat.lib.files;

import cat.lib.errors.ErrorEx;
import cat.lib.sort.SortUtils;
import cat.lib.strings.ParamsList;
import cat.lib.utils.NumberToString;
import cat.lib.utils.SetUtils;
import cat.lib.utils.StringUtils;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Collections;
import java.util.Vector;

/**
 *
 * @author Miquel Abadal i Cervià
 * @last-revision 02-11-2011
 */

public class FileUtils {



    //    _____                _              _          _          _        _ _           _                   _        _   _
    //   / ____)              | |            | |        (_)        | |      (_) |         | |                 | |      | | (_)
    //  | |     ___  _ __  ___| |_  _ _ _ __ | |_ ___    _      _ _| |_ _ __ _| |_   _   _| |_ ___     ___ ___| |_  _ _| |_ _  ____ ___
    //  | |    / _ \| `_  \ __) __) _` | `_  \ __) __)  | |   / _` | __) `__) | `_ \| | | | __) __)   / _ \ __) __) _` | __) |/ ___) __)
    //  | |____ (_) | | | |__ \ |_ (_| | | | | |_\__ \  | |  | (_| | |_| |  | | |_) | |_| | |_\__ \  |  __/__ \ |_ (_| | |_| | (___\__ \
    //   \_____)___/|_| |_|___/\__)__,_|_| |_|\__)___/  |_|   \__,_|\__)_|  |_|_,__/ \__,_|\__)___/   \___|___/\__)__,_|\__)_|\____)___/
    //
    //

    public static final boolean EXCLUDE_HIDDEN_FILES = false;
    public static final boolean INCLUDE_HIDDEN_FILES = true;
    
    public static final boolean EXCLUDE_DIRECTORIES = false;
    public static final boolean INCLUDE_DIRECTORIES = true;
    
    public static final boolean NO_ORDER = false;
    public static final boolean ORDER_BY_NAME = true;
    
    public static final boolean NO_RECURSIVE_METHOD = false;
    public static final boolean RECURSIVE_METHOD = true;


    //   __  __      _            _                        _             _
    //  |  \/  |    | |          | |                      (_)           | |
    //  | \  / | ___| |_ ___   __| | ___ ___    _ __  _ __ _ _    _  _ _| |_ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)  | `_ \| `__) | |  | | _` | __) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \  | |_) | |  | |\ \/ / (_| | |_\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/  |  __/|_|  |_| \__/ \__,_|\__)___/
    //                                         | |
    //                                         |_|

    private static int lastBarraPos (String filename) {

        int pos = -1;

        if (filename!=null) {
            int index = filename.length()-1;
            while (pos==-1 && index>=0) {
                char caract = filename.charAt(index);
                if (caract=='/' || caract=='\\') { pos = index; }
                index--;
            }
        }

        return pos;
    }

    private static void getFiles (Vector<File> files, String folderName, boolean includeHidden, boolean includeDirectory, boolean recursive, String[] fileExtensions) {

        if (files!=null && folderName!=null) {

            File directory = new File (folderName);
            File[] folderFiles = directory.listFiles();

            if (folderFiles!=null) {
                // Afegueix al llistat els fitxers que compleixen els requisits
                for (int cont1=0; cont1<folderFiles.length; cont1++) {
                    File file = folderFiles[cont1];
                    String ext = FileUtils.extractFileExt (file.getName());

                    // Si el fitxer no és ocult... o el llistat ha d'incloure els ocults
                    if (!file.isHidden() || includeHidden) {

                        // Si és un directori i cal incloure els directoris o cal recorrer els subdirectoris recursivament... cal afeguir el directori al llistat.
                        if (file.isDirectory() && (includeDirectory || recursive)) {
                            if (includeDirectory) { files.add (file); }
                            if (recursive) { getFiles (files, FileUtils.concatFolder (folderName, file.getName()), includeHidden, includeDirectory, recursive, fileExtensions); }
                        }

                        // Si és un arxiu i no s'han indicat les extensions vàlides o la extenció apareixe en el llistat, cal incloure el arxiu...
                        else if (fileExtensions==null || SetUtils.stringInArray (ext, StringUtils.CASE_INSENSITIVE, fileExtensions)) { files.add (file);  }
                    }
                }
            }
        }
    }

    //   __  __      _            _                     _     _ _
    //  |  \/  |    | |          | |                   | |   | |_)
    //  | \  / | ___| |_ ___   __| | ___    _ __  _   _| |_  | |_  ____ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \  | `_ \| | | | `_ \| | |/ ___) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/  | |_) | |_| | |_) | | | (___\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|  |  __/ \__,_|_,__/|_|_|\____)___/
    //                                     | |
    //                                     |_|


    //   __  __             _             _             _              _
    //  |  \/  |           (_)           | |           (_)            | |
    //  | \  / |  _ _ _ __  _ _ __  _   _| |  _ _  ____ _  ___      __| | ___
    //  | |\/| |/ _` | `_  \ | `_ \| | | | |/ _` |/ ___) |/ _ \    / _` |/ _ \
    //  | |  | | (_| | | | | | |_) | |_| | | (_| | (___| | (_) |  | (_| |  __/
    //  |_|  |_|\__,_|_| |_|_|  __/ \__,_|_|\__,_|\____)_|\___/    \__,_|\___|
    //                       | |
    //                       |_|
    //                  _
    //                 | |
    //    ____  _ _  __| | ___ _ __   ___ ___
    //   / ___) _` |/ _` |/ _ \ `_  \/ _ \ __)
    //  | (___ (_| | (_| |  __/ | | |  __/__ \
    //   \____)__,_|\__,_|\___|_| |_|\___|___/
    //
    //

    /**
     * Retorna la extenció del nom d'un fitxer.
     *
     * NOTA: Si el nom del fitxer indicat és <b>null</b> retorna una cadena en blanc
     * (no retorna, en cap cas, una cadena nula).
     *
     * NOTA: El nom del fitxer, pot incloure la ruta complerta
     *
     * EJEMPLES:
     * c:/mis documentos/fitxer.doc --> Retorna "doc"
     * http://www.google.es/index.html --> Retorna "html"
     * http://www.google.es/index --> Retorna ""
     * http://www.google.es/ --> Retorna ""
     *
     * @param filename
     * @return
     */
    public static String extractFileExt (String filename) {

        filename = extractFileName (filename);

        String res = null;
        if (filename!=null) {
            int pos = StringUtils.charInversePos (filename, '.', filename.length(), StringUtils.CASE_INSENSITIVE);
            if (pos!=-1) { res = StringUtils.copy (filename, pos+1, -1); }
        }
        return StringUtils.notNull (res);
    }

    /**
     * Retorna el nom del fitxer i la seva extenció de la ruta <b>filename</b>.
     *
     * NOTA: Si el nom del fitxer indicat és <b>null</b> retorna una cadena en blanc
     * (no retorna, en cap cas, una cadena nula).
     *
     * Si la URL conté paràmetres, aquest són eliminats
     * 
     * EJEMPLES:
     * c:/mis documentos/fitxer.doc --> Retorna "fitxer.doc"
     * http://www.google.es/index.html --> Retorna "index.html"
     * http://www.google.es/index --> Retorna "index"
     * http://www.google.es/ --> Retorna ""
     * http://www.google.es --> Retorna "www.google.es"
     *
     * NOTA: Per a separar el nom del fitxer de la ruta cerca l'última barra
     * ("/" o "\") de la cadena.
     *
     * @param filename
     * @return
     */
    public static String extractFileName (String filename) {

        String res = filename;
        int pos = lastBarraPos (filename);
        if (pos!=-1) { res = StringUtils.copy (filename, pos+1, -1); }
        
        res = StringUtils.copy (res, '?', StringUtils.CASE_INSENSITIVE, false);

        return StringUtils.notNull (res);
    }

    /**
     * Retorna la ruta del fitxer <b>filename</b>.
     *
     * NOTA: Si el nom del fitxer indicat és <b>null</b> retorna una cadena en blanc
     * (no retorna, en cap cas, una cadena nula).
     *
     * EJEMPLES:
     * c:/mis documentos/fitxer.doc --> Retorna "c:/mis documentos/"
     * http://www.google.es/index.html --> Retorna "http://www.google.es/"
     * http://www.google.es/index --> Retorna "http://www.google.es/"
     * http://www.google.es/ --> Retorna "http://www.google.es/"
     * http://www.google.es --> Retorna "http://"
     *
     * NOTA: Per a separar el nom del fitxer de la ruta cerca l'última barra
     * ("/" o "\") de la cadena.
     *
     * @param filename
     * @return
     */
    public static String extractFilePath (String filename) {

        String res = null;
        int pos = lastBarraPos (filename);
        if (pos!=-1) { res = StringUtils.copy (filename, 0, pos+1); }

        return StringUtils.notNull (res);
    }

    /**
     * Obtiene la ruta real del fichero, extrayendo, carpetas de retorno "/../"
     * y autoacceso "/./"
     *
     * Ejemplo: La dirección "c:\carpeta/subCarpeta/../fichero.ext"      
     * se convierte a "c:/carpeta/fichero.ext"
     * 
     */
    public static String getRealPath (String filename) {

        filename = StringUtils.substChar(filename, '\\', '/');
        ParamsList list = new ParamsList (filename, '/');

        int cont1 = 0;
        while (cont1<list.paramCount()) {
            String param = list.getParam(cont1);
            if (param.equals ("..")) {
                list.removeParam(cont1);
                list.removeParam(cont1-1);
                cont1--;
            }
            else if (param.equals(".")) {
                list.removeParam(cont1);
                cont1--;
            }
            cont1++;
        }
        return list.getText();
    }

    /**
     * Retorna una cadena, en la que s'ha substituit la extenció del fitxer
     * indicat amb el paràmetre "fileName" per la extenció indicada a "newExt".
     *
     *
     * @param fileName
     * @param newExt
     * @return
     */
    public static String changeFileNameExt (String fileName, String newExt) {

        String ext = FileUtils.extractFileExt (fileName);
        if (newExt!=null && newExt.length()>0 && newExt.charAt(0)!='.') { newExt = "."+newExt; }
        fileName = StringUtils.copy (fileName, 0, fileName.length()-ext.length()-1)+newExt;
        return fileName;
    }

    /**
     * Obté la carpeta "pare" de la carpeta actual.
     *
     * Per a realitzar aquesta acció, té en compte que la carpeta indicada pot
     * acabar o no amb els caracter '/' o '\'. En tots els casos n'extreu la carpeta
     * superior:
     *
     * Ej: http://nt.tusoporte.es/ijo/images  --> http://nt.tusoporte.es/ijo/
     *     http://nt.tusoporte.es/ijo/images/ --> http://nt.tusoporte.es/ijo/
     */
    public static String getBackFolder (String folder) {

        if (folder!=null) {
            if (folder.length()>0 && folder.charAt(folder.length()-1)!='/' && folder.charAt(folder.length()-1)!='\\') { folder = folder + "/"; }
            folder = StringUtils.copy(folder, 0, folder.length()-1);
        }

        return FileUtils.extractFilePath(folder);
    }

    public static String concatFolder (String folder, String subFolder) {

        folder = StringUtils.substChar (folder, '\\', '/', StringUtils.SUBST_ALL, StringUtils.FROM_BEGIN, StringUtils.CASE_SENSITIVE);
        subFolder = StringUtils.substChar (subFolder, '\\', '/', StringUtils.SUBST_ALL, StringUtils.FROM_BEGIN, StringUtils.CASE_SENSITIVE);

        // Si la carpeta original no acaba amb "/"... li posa el caracter.
        if (folder.length()>0 && folder.charAt(folder.length()-1)!='/' ) { folder = folder + "/"; }

        // Si la subcarpeta comença amb "/".. el treu.
        if (subFolder.length()>0 && subFolder.charAt(0)=='/') { subFolder = StringUtils.copy(subFolder, 1, -1); }

        return folder + subFolder;
    }


    //   __  __             _             _             _              _
    //  |  \/  |           (_)           | |           (_)            | |
    //  | \  / |  _ _ _ __  _ _ __  _   _| |  _ _  ____ _  ___      __| | ___
    //  | |\/| |/ _` | `_  \ | `_ \| | | | |/ _` |/ ___) |/ _ \    / _` |/ _ \
    //  | |  | | (_| | | | | | |_) | |_| | | (_| | (___| | (_) |  | (_| |  __/
    //  |_|  |_|\__,_|_| |_|_|  __/ \__,_|_|\__,_|\____)_|\___/    \__,_|\___|
    //                       | |
    //    ___ _ _            |_|
    //   / __)_) |
    //  | |_  _| |__   _____ _ __ ___
    //  |  _|| | __) \/ / _ \ `__) __)
    //  | |  | | |_ )  (  __/ |  \__ \
    //  |_|  |_|\__)_/\_\___|_|  (___/
    //
    //

    /**
     * Ordena, alfabéticament, les linies d'un fitxer de text.
     *
     */
    public static void sortFile (String fileName, String outFileName, ErrorEx error) {


        Vector lines = new Vector();

        // Obre el fitxer original
        TextFile origFile = new TextFile ();
        origFile.reset (fileName, error);

        // Llegueix el fitxer i posa les lines en un vector.
        if (error.getError()==null) {
            String line = origFile.readLine();
            while (line!=null && error.getError()==null) {
                lines.add(line);
                line = origFile.readLine (error);
            }
            origFile.close (error);
        }

        if (error.getError()==null) {
            // Ordena el vector.
            Collections.sort (lines);

            // Grava el fitxer ordenat
            TextFile destiFile = new TextFile();
            destiFile.rewrite(outFileName, error);
            int cont1 = 0;
            while (cont1<lines.size() && error.getError()==null) {
                destiFile.writeLine((String) lines.get(cont1), error);
                cont1++;
            }

            destiFile.close(error);
        }
        
    }

    public static boolean fileExist (String filename) {

        File file = new File (filename);
        return file.exists();
    }

    public static boolean isDirectory (String filename) {

        File file = new File (filename);
        return file.isDirectory();
    }

    public static void renameFile (String fileName, String newFileName, ErrorEx error) {

        File file = new File (fileName);
        File newFile = new File (newFileName);

        boolean success = file.renameTo (newFile);
        if (!success) { error.setError("FILE_RENAME_ERROR", "DESCRIPTION: No se ha podido renombrar el fichero", "FILENAME: "+fileName, "NEW NAME: "+newFileName); }
    }

    public static void deleteFile (String fileName, ErrorEx error) {

        File file = new File (fileName);

        boolean success = file.delete();
        if (!success) { error.setError("FILE_DELETE_ERROR", "DESCRIPTION: No se ha podido borrar el fichero", "FILENAME: "+fileName); }
    }


    /**
     * Retorna la llista de fitxers de la carpeta "folderName".
     *
     * Si el paràmetre <b>orderByFileName</b> està activat, ordena el llistat obtingut.
     *
     * Si el paràmetre <B>showHidden</b> està habilitat inclou en el llistat els fitxers
     * ocults.
     *
     * Si el paràmetre <b>showDirectory</b> està habilitat inclou en el llistat
     * els directoris.
     *
     * Si el paràmetre <b>recursive</b> està habilitat recorre recursivament les
     * subcarpetes i n'inclou els seus arxius.
     *
     * Si el paràmetre <b>fileExtensions</b> es diferent de <b>null</b> només inclou al
     * llistat els fitxers que tinguin una de les extencions permeses. Atenció:
     * Aquest paràmetre NO afecta als directoris obtinguts.
     *     
     */
    public static Vector<File> getFiles (String folderName, boolean orderByFileName, boolean includeHidden, boolean includeDirectory, boolean recursive, String[] fileExtensions) {

        Vector<File> files = new Vector();
        folderName = FileUtils.concatFolder (folderName, "");
        getFiles (files, folderName, includeHidden, includeDirectory, recursive, fileExtensions);

        // Ordena els fitxers...
        if (orderByFileName) { SortUtils.sort (files, new String [] {"isDirectory", "getName"}); }

        return files;

    }

    public static Vector<File> getFiles (String folderName, String[] fileExtensions) {
        return getFiles (folderName, ORDER_BY_NAME, EXCLUDE_HIDDEN_FILES, INCLUDE_DIRECTORIES, NO_RECURSIVE_METHOD, fileExtensions);
    }

    public static File[] getFilesAsArray (String folderName, boolean orderByFileName, boolean includeHidden, boolean includeDirectory, boolean recursive, String[] fileExtensions) {

        Vector<File> files = getFiles (folderName, orderByFileName, includeHidden, includeDirectory, recursive, fileExtensions);

        File[] res = new File[files.size()];
        for (int cont1=0; cont1<files.size(); cont1++) { res[cont1] = files.get(cont1); }

        return res;
    }

    /**
     * Retorna la llista de fitxers i carpetes NO OCULTS de la carpeta "folderName".
     *
     * Si el paràmetre <b>orderByFileName</b> està activat, ordena el llistat obtingut.
     */
    public static File[] getFilesAsArray (String folderName, boolean orderByFileName) { return getFilesAsArray (folderName, orderByFileName, EXCLUDE_HIDDEN_FILES, INCLUDE_DIRECTORIES, NO_RECURSIVE_METHOD, null); }

    /**
     * Retorna la llista ordenada de fitxers i carpetes NO OCULTS de la carpeta "folderName".
     */
    public static File[] getFilesAsArray (String folderName) { return getFilesAsArray (folderName, ORDER_BY_NAME, EXCLUDE_HIDDEN_FILES, INCLUDE_DIRECTORIES, NO_RECURSIVE_METHOD,  null); }

    public static void copyFile (String fileName, String newFileName, boolean createFolder, ErrorEx error) {

        String toFolder = extractFilePath (newFileName);
        if (createFolder) { createFolder (toFolder, true, false, error); }
        
	DataFile fileInput = new DataFile();
        DataFile fileOutput = new DataFile();

        fileInput.reset (fileName, error);
        fileOutput.rewrite (newFileName, error);

        byte[] buffer = new byte[4096];
        int numRead = fileInput.blockRead (buffer, error);
        while (numRead!=-1 && error.getError()==null) {
            fileOutput.blockWrite (buffer, numRead, error);
            numRead = fileInput.blockRead (buffer, error);
        }

        fileInput.close (error);
        fileOutput.close (error);
    }

    public static void copyFile (String fileName, String newFileName, ErrorEx error) { copyFile (fileName, newFileName, true, error); }
    
    public static void createFolder (String folderName, boolean makeDirs, boolean errorIfExist, ErrorEx error) {

        File folder = new File (folderName);
        
        if (!folder.exists()) {
            String exceptionMessage = null;
            boolean complited = false;
            try { 
                if (makeDirs) { complited = folder.mkdirs(); }
                else { complited = folder.mkdir(); }            
            }
            catch (Exception e) { exceptionMessage = e.getMessage(); }

            if (!complited) { error.setError("MAKE_DIR", "DESCRIPTION: No se ha podido crear la carpeta", "FOLDER: "+folderName, "MAKEDIRS: "+NumberToString.toStringTrueFalse(makeDirs), "EXCEPTION: "+exceptionMessage); }
        }
        else if (errorIfExist) { error.setError("MAKE_DIR", "DESCRIPTION: Ya existe una carpeta con esta ruta", "FOLDER: "+folderName); }
    }
    
    public static void createFolder (String folderName, ErrorEx error) { createFolder (folderName, true, false, error); }

    // Grava les dades d'un "InputStream" al fitxer "FileName".
    public static void inputStreamToFile (InputStream in, String fileName, ErrorEx error) {

        File file = new File(fileName);

        try {
            OutputStream out = new FileOutputStream (file);

            byte buffer[] = new byte[1024];
            int size = in.read (buffer);
            while (size>0) {
                out.write (buffer,0,size);
                size = in.read (buffer);
            }
            out.close();
            in.close();

        }
        catch (IOException e) { error.setError("SAVE", "DESCRIPTION: Se ha producido un error al intentar grabar el archivo", "FILENAME: "+fileName); }
    }

}
