/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package cat.lib.files;

import cat.lib.errors.ErrorEx;
import cat.lib.utils.NumberToString;
import cat.lib.utils.StringUtils;
import java.util.Date;

/**
 *
 * @author Miquel Abadal Cervià
 */

class LogFile {


    //    _____                _              _          _          _        _ _           _                   _        _   _
    //   / ____)              | |            | |        (_)        | |      (_) |         | |                 | |      | | (_)
    //  | |     ___  _ __  ___| |_  _ _ _ __ | |_ ___    _      _ _| |_ _ __ _| |_   _   _| |_ ___     ___ ___| |_  _ _| |_ _  ____ ___
    //  | |    / _ \| `_  \ __) __) _` | `_  \ __) __)  | |   / _` | __) `__) | `_ \| | | | __) __)   / _ \ __) __) _` | __) |/ ___) __)
    //  | |____ (_) | | | |__ \ |_ (_| | | | | |_\__ \  | |  | (_| | |_| |  | | |_) | |_| | |_\__ \  |  __/__ \ |_ (_| | |_| | (___\__ \
    //   \_____)___/|_| |_|___/\__)__,_|_| |_|\__)___/  |_|   \__,_|\__)_|  |_|_,__/ \__,_|\__)___/   \___|___/\__)__,_|\__)_|\____)___/
    //
    //

    private final int MAX_HISTORIC = 20;
    private final int DEFAULT_MAX_LINES = 1000000;
    private final int DEFAULT_MAX_SIZE = 10000000;



    //     __   _        _ _           _
    //   / __ \| |      (_) |         | |
    //  | |__| | |_ _ __ _| |_   _   _| |_ ___
    //  |  __  | __) `__) | `_ \| | | | __) __)
    //  | |  | | |_| |  | | |_) | |_| | |_\__ \
    //  |_|  |_|\__)_|  |_|_,__/ \__,_|\__)___/
    //
    //

    private TextFile file = null;
    private int lines = 0;
    private int size = 0;
    private int maxLines = DEFAULT_MAX_LINES;
    private int maxSize = DEFAULT_MAX_SIZE;
    private ErrorEx error = null;

    private String contextName = null;
    private String logsFolder = null;
    private String logsName = null;

    private String[] historic = null;
    private int historicIndex = 0;

    private String encoding = null;
    private boolean showDate = false;


    //   __  __      _            _                              _                    _
    //  |  \/  |    | |          | |                            | |                  | |
    //  | \  / | ___| |_ ___   __| | ___     ____ ___  _ __  ___| |_ _ __ _   _  ____| |_ ___  _ __
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \   / ___) _ \| `_  \ __) __) `__) | | |/ ___) __) _ \| `__)
    //  | |  | |  __/ |_ (_) | (_| |  __/  | (___ (_) | | | |__ \ |_| |  | |_| | (___| |_ (_) | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|   \____)___/|_| |_|___/\__)_|   \__,_|\____)\__)___/|_|
    //
    //

    public LogFile (String contextName, String logsFolder, String logsName, String encoding) {

        if (StringUtils.isEmpty(encoding)) { encoding = "utf-8"; }
        this.contextName = contextName;
        this.logsFolder = logsFolder;
        this.logsName = logsName;
        this.encoding = encoding;

        file = null;
        error = new ErrorEx();
        lines = 0;
        maxLines = DEFAULT_MAX_LINES;
        maxSize = DEFAULT_MAX_SIZE;

        historic = new String[MAX_HISTORIC];
        for (int cont1=0; cont1<MAX_HISTORIC; cont1++) { historic[cont1] = null; }
    }


    //   __  __      _            _             _  _   _____ ______ _______ _  _
    //  |  \/  |    | |          | |           | || | / ____)  ____)__   __) || |
    //  | \  / | ___| |_ ___   __| | ___ ___   |_||_|| |  __| |__     | |  |_||_|
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)        | | (_ |  __)    | |
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \        | |__| | |____   | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/         \_____|______)  |_|
    //
    //

    public int getMaxLines () { return maxLines; }
    public int getMaxSize () { return maxSize; }
    public boolean getShowDate () { return showDate; }


    //   __  __      _            _             _  _   _____ ______ _______ _  _
    //  |  \/  |    | |          | |           | || | / ____)  ____)__   __) || |
    //  | \  / | ___| |_ ___   __| | ___ ___   |_||_|| (___ | |__     | |  |_||_|
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)         \___ \|  __)    | |
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \         ____) | |____   | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/        (_____/|______)  |_|
    //
    //

    public void setMaxLines (int maxLines) { this.maxLines = maxLines; }
    public void setMaxSize (int maxSize) { this.maxSize = maxSize; }
    public void setShowDate (boolean showDate) { this.showDate = showDate; }


    //   __  __      _            _                        _             _
    //  |  \/  |    | |          | |                      (_)           | |
    //  | \  / | ___| |_ ___   __| | ___ ___    _ __  _ __ _ _    _  _ _| |_ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)  | `_ \| `__) | |  | | _` | __) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \  | |_) | |  | |\ \/ / (_| | |_\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/  |  __/|_|  |_| \__/ \__,_|\__)___/
    //                                         | |
    //                                         |_|


    //   __  __      _            _                     _     _ _
    //  |  \/  |    | |          | |                   | |   | |_)
    //  | \  / | ___| |_ ___   __| | ___    _ __  _   _| |_  | |_  ____ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \  | `_ \| | | | `_ \| | |/ ___) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/  | |_) | |_| | |_) | | | (___\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|  |  __/ \__,_|_,__/|_|_|\____)___/
    //                                     | |
    //                                     |_|

    public void writeLog (String text) {

        if (StringUtils.isNotEmpty(text)) {
            if (file==null || (lines>=maxLines && maxLines!=-1) || (size>=maxSize && maxSize!=-1)) {
                if (file!=null) { file.close(error); }

                String fileName = FileUtils.concatFolder (logsFolder, contextName+"_"+logsName+" "+NumberToString.toDateTimeString (new Date())+".log");

                file = new TextFile();
                file.rewrite (fileName, encoding, error);
                lines = 0;
                size = 0;
            }


            if (showDate) { text = "["+NumberToString.toDateTimeString (new Date())+"] "+text; }

            file.writeLine (text, error);
            lines++;
            size+=text.length()+2;

            historic[historicIndex] = text;
            historicIndex++;
            if (historicIndex>=MAX_HISTORIC) { historicIndex = 0; }
        }
    }

    public String getLastLogs() {

        StringBuilder res = new StringBuilder();

        int index = historicIndex;

        for (int cont1=0; cont1<MAX_HISTORIC; cont1++) {

            if (StringUtils.isNotEmpty (historic[index])) { res.append (historic[index]).append('\n'); }
            index++;
            if (index>=MAX_HISTORIC) { index = 0; }
        }
        return res.toString();
    }
}

public class LogsFile {


    //    _____                _              _          _          _        _ _           _                   _        _   _
    //   / ____)              | |            | |        (_)        | |      (_) |         | |                 | |      | | (_)
    //  | |     ___  _ __  ___| |_  _ _ _ __ | |_ ___    _      _ _| |_ _ __ _| |_   _   _| |_ ___     ___ ___| |_  _ _| |_ _  ____ ___
    //  | |    / _ \| `_  \ __) __) _` | `_  \ __) __)  | |   / _` | __) `__) | `_ \| | | | __) __)   / _ \ __) __) _` | __) |/ ___) __)
    //  | |____ (_) | | | |__ \ |_ (_| | | | | |_\__ \  | |  | (_| | |_| |  | | |_) | |_| | |_\__ \  |  __/__ \ |_ (_| | |_| | (___\__ \
    //   \_____)___/|_| |_|___/\__)__,_|_| |_|\__)___/  |_|   \__,_|\__)_|  |_|_,__/ \__,_|\__)___/   \___|___/\__)__,_|\__)_|\____)___/
    //
    //

    private final int REAL_MODE = 0;
    private final int DEVELOPER_MODE = 1;


    //     __   _        _ _           _
    //   / __ \| |      (_) |         | |
    //  | |__| | |_ _ __ _| |_   _   _| |_ ___
    //  |  __  | __) `__) | `_ \| | | | __) __)
    //  | |  | | |_| |  | | |_) | |_| | |_\__ \
    //  |_|  |_|\__)_|  |_|_,__/ \__,_|\__)___/
    //
    //

    private final int MAX_LOGS_FILES = 6;

    private final int INFO_INDEX = 0;
    private final int WARNING_INDEX = 1;
    private final int ERROR_INDEX = 2;
    private final int FATAL_INDEX = 3;
    private final int TRACE_INDEX = 4;
    private final int SQL_INDEX = 5;

    private LogFile[] logs = null;

    private int mode = REAL_MODE;


    //   __  __      _            _                              _                    _
    //  |  \/  |    | |          | |                            | |                  | |
    //  | \  / | ___| |_ ___   __| | ___     ____ ___  _ __  ___| |_ _ __ _   _  ____| |_ ___  _ __
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \   / ___) _ \| `_  \ __) __) `__) | | |/ ___) __) _ \| `__)
    //  | |  | |  __/ |_ (_) | (_| |  __/  | (___ (_) | | | |__ \ |_| |  | |_| | (___| |_ (_) | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|   \____)___/|_| |_|___/\__)_|   \__,_|\____)\__)___/|_|
    //
    //

    public LogsFile (String fileName, String logsFolder, String encoding) {

        logs = new LogFile[MAX_LOGS_FILES];
        logs[INFO_INDEX] = new LogFile (fileName, logsFolder, "info", encoding);
        logs[WARNING_INDEX] = new LogFile (fileName, logsFolder, "warning", encoding);
        logs[ERROR_INDEX] = new LogFile (fileName, logsFolder, "error", encoding);
        logs[FATAL_INDEX] = new LogFile (fileName, logsFolder, "fatal", encoding);
        logs[TRACE_INDEX] = new LogFile (fileName, logsFolder, "trace", encoding);
        logs[SQL_INDEX] = new LogFile (fileName, logsFolder, "sql", encoding);

    }


    //   __  __      _            _             _  _   _____ ______ _______ _  _
    //  |  \/  |    | |          | |           | || | / ____)  ____)__   __) || |
    //  | \  / | ___| |_ ___   __| | ___ ___   |_||_|| |  __| |__     | |  |_||_|
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)        | | (_ |  __)    | |
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \        | |__| | |____   | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/         \_____|______)  |_|
    //
    //

    /**
     *
     * @return Número màxim de linies que pot incloure cadascun dels fitxers de logs
     * o -1 si no tenen límit.
     */
    public int getMaxLines () { return logs[INFO_INDEX].getMaxLines(); }

    /**
     *
     * @return Número màxim de bytes que pot incloure cadascun dels fitxers de logs
     * o -1 si no tenen límit.
     */
    public int getMaxSize () { return logs[INFO_INDEX].getMaxSize(); }

    /**
     *
     * @return Indica si al afeguir un nou log, també s'ha de registrar la data i
     * la hora.
     */
    public boolean getShowDate () { return logs[INFO_INDEX].getShowDate(); }


    //   __  __      _            _             _  _   _____ ______ _______ _  _
    //  |  \/  |    | |          | |           | || | / ____)  ____)__   __) || |
    //  | \  / | ___| |_ ___   __| | ___ ___   |_||_|| (___ | |__     | |  |_||_|
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)         \___ \|  __)    | |
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \         ____) | |____   | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/        (_____/|______)  |_|
    //
    //

    /**
     * Defineix el número màxim de linies que poden tenir els fitxers de logs; quan es supera, es crea un
     * fitxer de logs nou. Per defecte el valor és de 100.000 línies.
     *
     * @param maxLines Número màxim de línies del fitxer de logs o -1 si no es vol utiliztar cap màxim
     */
    public void setMaxLines (int maxLines) {
        logs[INFO_INDEX].setMaxLines (maxLines);
        logs[WARNING_INDEX].setMaxLines (maxLines);
        logs[ERROR_INDEX].setMaxLines (maxLines);
        logs[TRACE_INDEX].setMaxLines (maxLines);
        logs[FATAL_INDEX].setMaxLines (maxLines);
        logs[SQL_INDEX].setMaxLines (maxLines);
    }
    
    /**
     * Defineix el número màxim de bytes que poden tenir els fitxers de logs; quan es supera, es crea un 
     * fitxer de logs nou. Per defecte el valor és de 100 Mb.
     * 
     * @param maxLines Número màxim de bytes del fitxer de logs o -1 si no es vol utiliztar cap màxim
     */
    public void setMaxSize (int setMaxSize) {
        logs[INFO_INDEX].setMaxSize (setMaxSize);
        logs[WARNING_INDEX].setMaxSize (setMaxSize);
        logs[ERROR_INDEX].setMaxSize (setMaxSize);
        logs[TRACE_INDEX].setMaxSize (setMaxSize);
        logs[FATAL_INDEX].setMaxSize (setMaxSize);
        logs[SQL_INDEX].setMaxSize (setMaxSize);
    }

    /**
     * Determina si al afeguir logs s'ha de registrar o no la data i la hora.
     */
    public void setShowDate (boolean showDate) {
        logs[INFO_INDEX].setShowDate (showDate);
        logs[WARNING_INDEX].setShowDate (showDate);
        logs[ERROR_INDEX].setShowDate (showDate);
        logs[TRACE_INDEX].setShowDate (showDate);
        logs[FATAL_INDEX].setShowDate (showDate);
        logs[SQL_INDEX].setShowDate (showDate);
    }
    

    //   __  __      _            _                        _             _
    //  |  \/  |    | |          | |                      (_)           | |
    //  | \  / | ___| |_ ___   __| | ___ ___    _ __  _ __ _ _    _  _ _| |_ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)  | `_ \| `__) | |  | | _` | __) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \  | |_) | |  | |\ \/ / (_| | |_\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/  |  __/|_|  |_| \__/ \__,_|\__)___/
    //                                         | |
    //                                         |_|


    //   __  __      _            _                     _     _ _
    //  |  \/  |    | |          | |                   | |   | |_)
    //  | \  / | ___| |_ ___   __| | ___    _ __  _   _| |_  | |_  ____ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \  | `_ \| | | | `_ \| | |/ ___) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/  | |_) | |_| | |_) | | | (___\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|  |  __/ \__,_|_,__/|_|_|\____)___/
    //                                     | |
    //                                     |_|

    /**
     * Activa el mode de "desenvolupament"; en aquest mode, es registren tots els
     * logs de "pas a pas" ("trace")
     */
    public void developerMode() { mode = DEVELOPER_MODE; }

    /**
     * Activa el mode de "real"; en aquest mode, NO es registren els
     * logs de "pas a pas" ("trace")
     */
    public void realMode() { mode = REAL_MODE; }

    /**
     * Afegueix el log "text" al fitxer de logs d'informació.
     */
    public void info (String text) { logs[INFO_INDEX].writeLog (text); }

    /**
     * Afegueix el log "text" al fitxer de logs d'avisos/alertes.
     */
    public void warning (String text) { logs[WARNING_INDEX].writeLog (text); }

    /**
     * Afegueix el log "text" al fitxer de logs d'errors.
     */
    public void error (String text) { logs[ERROR_INDEX].writeLog (text); }

    /**
     * Afegueix el log "text" al fitxer de logs d'errors fatals.
     */
    public void fatal (String text) { logs[FATAL_INDEX].writeLog (text); }

    /**
     * Afegueix el log "text" al fitxer de logs de execució pas a pas.
     *
     * Els logs de "pas a pas" només es processen quan al sistema de logs
     * s'activa el "mode desenvolupament".
     */
    public void trace (String text) {
        if (mode==DEVELOPER_MODE) { logs[TRACE_INDEX].writeLog (text); }
    }

    /**
     * Afegueix el log "text" al fitxer de logs SQL.
     */
    public void sql (String text) { logs[SQL_INDEX].writeLog (text); }

    /**
     * @return Retorna els últims 20 logs d'informació generats.
     */
    public String getInfoLogs() { return logs[INFO_INDEX].getLastLogs(); }

    /**
     * @return Retorna una taula amb els últims 20 logs d'alertes generats.
     */
    public String getWarningLogs() { return logs[WARNING_INDEX].getLastLogs(); }

    /**
     * @return Retorna una taula amb els últims 20 logs d'errors generats.
     */
    public String getErrorLogs() { return logs[ERROR_INDEX].getLastLogs(); }

    /**
     * @return Retorna una taula amb els últims 20 logs d'errors fatals generats.
     */
    public String getFatalLogs() { return logs[FATAL_INDEX].getLastLogs(); }

    /**
     * @return Retorna una taula amb els últims 20 logs de "pas a pas" generats.
     */
    public String getTraceLogs() { return logs[TRACE_INDEX].getLastLogs(); }

    /**
     * @return Retorna una taula amb els últims 20 logs de SQL.
     */
    public String getSqlLogs() { return logs[SQL_INDEX].getLastLogs(); }

}
