//    _____ _                       _  _  _______       _    _ _   _ _    _  _
//   / ____) |                     | || |(___  /_)     | |  | | | (_) |  | || |
//  | |    | |  _ _ ___ ___  ___   |_||_|   / / _ _ __ | |  | | |_ _| |____||_|
//  | |    | |/ _` | __) __)/ _ \          / / | | `_ \| |  | | __) | | __)
//  | |____| | (_| |__ \__ \  __/         / /__| | |_) | |__| | |_| | |__ \
//   \_____)_|\__,_|___/___/\___|        /_____)_|  __/ \____/ \__)_|_|___/
//                                               | |
//                                               |_|


package cat.lib.files;

import cat.lib.errors.ErrorEx;
import cat.lib.utils.SetUtils;
import cat.lib.utils.StringUtils;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Vector;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 *
 * @author Miquel Abadal Cervià
 * @date 02-11-2011
 * 
 */
public class ZipUtils {

    //    _____                _              _          _          _        _ _           _                   _        _   _
    //   / ____)              | |            | |        (_)        | |      (_) |         | |                 | |      | | (_)
    //  | |     ___  _ __  ___| |_  _ _ _ __ | |_ ___    _      _ _| |_ _ __ _| |_   _   _| |_ ___     ___ ___| |_  _ _| |_ _  ____ ___
    //  | |    / _ \| `_  \ __) __) _` | `_  \ __) __)  | |   / _` | __) `__) | `_ \| | | | __) __)   / _ \ __) __) _` | __) |/ ___) __)
    //  | |____ (_) | | | |__ \ |_ (_| | | | | |_\__ \  | |  | (_| | |_| |  | | |_) | |_| | |_\__ \  |  __/__ \ |_ (_| | |_| | (___\__ \
    //   \_____)___/|_| |_|___/\__)__,_|_| |_|\__)___/  |_|   \__,_|\__)_|  |_|_,__/ \__,_|\__)___/   \___|___/\__)__,_|\__)_|\____)___/
    //
    //

    public static final int BUFFER_SIZE = 8192;


    //   __  __      _            _                        _             _
    //  |  \/  |    | |          | |                      (_)           | |
    //  | \  / | ___| |_ ___   __| | ___ ___    _ __  _ __ _ _    _  _ _| |_ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)  | `_ \| `__) | |  | | _` | __) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \  | |_) | |  | |\ \/ / (_| | |_\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/  |  __/|_|  |_| \__/ \__,_|\__)___/
    //                                         | |
    //                                         |_|

    private static void copyStream (InputStream input, OutputStream output, ErrorEx error) {

        if (input!=null && output!=null) {
            byte data[]=new byte[BUFFER_SIZE];

            try {
                int numRead = input.read (data, 0, BUFFER_SIZE);
                while (numRead!=-1) {
                    output.write (data, 0, numRead);
                    numRead = input.read (data, 0, BUFFER_SIZE);
                }
            }
            catch (IOException ex) { error.setError ("IOEXCEPTION", "ACTION: COPYING STREAM", "DESCRIPTION: Se ha producido un error al copiar", "EXCEPTION: "+ex.getMessage()); }
        }
        else { error.setError ("IOEXCEPTION", "ACTION: COPYING STREAM", "DESCRIPTION: Null pointer exception; no se ha indicado el stream de entrada o el de salida"); }
    }

    //   __  __      _            _                     _     _ _
    //  |  \/  |    | |          | |                   | |   | |_)
    //  | \  / | ___| |_ ___   __| | ___    _ __  _   _| |_  | |_  ____ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \  | `_ \| | | | `_ \| | |/ ___) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/  | |_) | |_| | |_) | | | (___\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|  |  __/ \__,_|_,__/|_|_|\____)___/
    //                                     | |
    //                                     |_|

    /**
     * Descomprimeix els arxius del fitxer "zipFileName" a la carpeta "toFolder".
     *
     * NOTA: Si no s'indica el nom del arxiu a descomprimit o la carpeta destí,
     * no es realitza cap acció i es retorna un 0.
     *
     * @param zipFileName Nom del fitxer a descomprimir
     * @param toFolder Carpeta on s'han de gravar els documents descomprimits.
     * @param error
     * @return Retorna el número de fitxers descomprimits.
     */
    public static int unzipFiles (String zipFileName, String toFolder, ErrorEx error) {

        int numFiles = 0;

        if (!StringUtils.isEmpty (zipFileName) && !StringUtils.isEmpty (toFolder)) {
            try {
                // Fitxer ZIP a descomprimir
                ZipFile zipFile = new ZipFile (zipFileName);

                // Obté la llista de fitxers comprimits
                Enumeration entries = zipFile.entries();

                while (entries.hasMoreElements() && error.getError()==null) {
                    ZipEntry entry = (ZipEntry) entries.nextElement();

                    // Obté el nom del fitxer a extreure
                    String toFileName = StringUtils.notNull (toFolder)+entry.getName();

                    // Si el fitxer a extreure correspon a una carpeta
                    if (entry.isDirectory()) {
                        File folder = new File (toFileName);
                        folder.mkdirs();
                    }
                    else {
                        // Grava el fitxer
                        InputStream input = zipFile.getInputStream(entry);
                        OutputStream outFile = new FileOutputStream (toFileName);
                        copyStream (input, outFile, error);
                        outFile.flush();
                        outFile.close();
                        input.close();
                        numFiles++;
                    }
                }
                zipFile.close();
            }
            catch (IOException ex) { error.setError ("IOEXCEPTION", "ACTION: UNZIP_FILES", "DESCRIPTION: Se ha producido un error al descomprimir el archivo", "FILENAME: "+zipFileName, "EXCEPTION: "+ex.getMessage()); }
            catch (Exception ex) { error.setError ("EXCEPTION", "ACTION: UNZIP_FILES", "DESCRIPTION: Se ha producido un error al descomprimir el archivo", "FILENAME: "+zipFileName, "EXCEPTION: "+ex.getMessage()); }
        }
        else { error.setError ("EXCEPTION", "ACTION: UNZIP_FILES", "DESCRIPTION: No se ha indicado el nombre del archivo a descomprimir o la carpeta destino", "FILENAME: "+zipFileName, "EXCEPTION: Null or empty exception"); }

        return numFiles;
    }

    /**
     * Esborra del fitxer comprimit "zipFileName" els fitxers indicats a la taula
     * "fileNames".
     *
     * NOTA: Per a realitzar l'operació, es re-anomena el fitxer comprimir assignant-li
     * l'extenció <b>.tmp</b>; després, crea l'arxiu "zipFileName" i hi copia els 
     * arxius, excepte els que apareixen a la taula "fileNames". Finalment, 
     * esborra el fitxer temporal.
     *
     * NOTA: Si no s'indica el nom del fitxer ZIP o no s'indiquen la llista de fitxers
     * a afeguir, no es realitza cap operació i es retorna un 0.
     *
     * @param zipFileName Nom del fitxer a descomprimir
     * @param fileNames Llista de fitxers que s'han d'eliminar
     * @param error
     * @return Retorna el número d'arxius que conté el fitxer comprimit
     */
    public static int deleteFiles (String zipFileName, String[] fileNames, ErrorEx error) {

        int numFiles = 0;
        if (zipFileName!=null && fileNames!=null && fileNames.length>0) {
            try {
                String tmpFileName = FileUtils.changeFileNameExt (zipFileName, "tmp");
                FileUtils.renameFile (zipFileName, tmpFileName, error);
                if (error.getError()==null) {

                    // Fitxer ZIP a descomprimir
                    ZipFile zipFile = new ZipFile (tmpFileName);
                    ZipOutputStream outZipFile = new ZipOutputStream (new FileOutputStream(zipFileName));

                    // Obté la llista de fitxers comprimits
                    Enumeration entries = zipFile.entries();

                    while (entries.hasMoreElements() && error.getError()==null) {
                        ZipEntry entry = (ZipEntry) entries.nextElement();

                        // Si el fitxer no està a la llista de fitxers a esborrar....
                        if (!SetUtils.stringInArray (entry.getName(), StringUtils.CASE_INSENSITIVE, fileNames)) {

                            // Si el fitxer a extreure correspon a una carpeta
                            if (!entry.isDirectory()) {
                                outZipFile.putNextEntry(entry);
                                InputStream input = zipFile.getInputStream(entry);

                                copyStream (input, outZipFile, error);
                                outZipFile.closeEntry();
                                input.close();
                                numFiles++;
                            }
                        }
                    }
                    zipFile.close();
                    outZipFile.close();

                    if (error.getError()==null) { FileUtils.deleteFile (tmpFileName, error); }
                }

            }
            catch (IOException ex) { error.setError ("IOEXCEPTION", "ACTION: DELETE_FILES", "DESCRIPTION: Se ha producido un error al eliminar ficheros comprimidos", "FILENAME: "+zipFileName, "EXCEPTION: "+ex.getMessage()); }
            catch (Exception ex) { error.setError ("EXCEPTION", "ACTION: DELETE_FILES", "DESCRIPTION: Se ha producido un error al eliminar ficheros comprimidos", "FILENAME: "+zipFileName, "EXCEPTION: "+ex.getMessage()); }
        }
        else { error.setError ("EXCEPTION", "ACTION: DELETE_FILES", "DESCRIPTION: No se ha indicado el nombre del archivo comprimido o los nombres de los ficheros a eliminar", "FILENAME: "+zipFileName, "EXCEPTION: Null or empty exception"); }

        return numFiles;
    }

    /**
     * Afegueix, al fitxer "zipFileName" els arxius inclosos a la taula "fileNames";
     * si el fitxer "zipFileName" no existeix, el crea.
     *
     * NOTA: Per a realitzar l'operació, es re-anomena el fitxer comprimir assignant-li
     * l'extenció <b>.tmp</b>; després, crea l'arxiu "zipFileName" i hi copia els
     * arxius, excepte els que apareixen a la taula "fileNames". Finalment,
     * hi comprimeix els arxius incloses a la taula "fileNames" iesborra el fitxer
     * temporal.
     *
     * D'aquesta manera, si la llista "fileNames" inclou un arxiu amb el mateix
     * nom que algun dels arxius del fitxer comprimit, aquest queda substituït.
     *
     * NOTA: La carpeta "fromFolder" determina la carpeta arrel on estan localitzats
     * els arxius indicats a la taula "fileNames"; aquests, per tant, inclouen
     * només la ruta relativa a la carpeta "fromFolder" així com el nom amb
     * que apareixen dintre del llistat d'arxius comprimits.
     *
     * NOTA: Si no s'indica el nom del fitxer ZIP o no s'indiquen la llista de fitxers
     * a afeguir, no es realitza cap operació i es retorna un 0.
     *
     * @param zipFileName Nom del fitxer a descomprimir
     * @param fromFolder Carpeta on estan localitzats els arxius a comprimir.
     * @param fileNames Llista de fitxers que s'han d'eliminar
     * @param error
     * @return Retorna el número d'arxius afeguits al fitxer comprimit.
     */
    public static int zipFiles (String zipFileName, String fromFolder, String[] fileNames, ErrorEx error) {

        int numFiles = 0;
        if (!StringUtils.isEmpty (zipFileName) && fileNames!=null && fileNames.length>0) {
            try {

                String tmpFileName = null;

                // Si el fitxer zip existeix, el reanomena
                if (FileUtils.fileExist(zipFileName)) {
                    tmpFileName = FileUtils.changeFileNameExt(zipFileName, "tmp");
                    FileUtils.renameFile (zipFileName, tmpFileName, error);
                }

                if (error.getError()==null) {

                    // Crea el fitxer ZIP
                    ZipOutputStream outZipFile = new ZipOutputStream (new FileOutputStream (zipFileName));

                    // Si el fitxer zip original existia, copia els arxius que conté a dintre del nou fitxer ZIP.
                    if (tmpFileName!=null) {
                        // Fitxer ZIP a copiar
                        ZipFile zipFile = new ZipFile (tmpFileName);

                        // Obté la llista de fitxers comprimits
                        Enumeration entries = zipFile.entries();

                        while (entries.hasMoreElements() && error.getError()==null) {
                            ZipEntry entry = (ZipEntry) entries.nextElement();

                            // Si el fitxer no està a la llista de fitxers a esborrar....
                            if (!SetUtils.stringInArray (entry.getName(), StringUtils.CASE_INSENSITIVE, fileNames)) {

                                // Si el fitxer a extreure correspon a una carpeta
                                if (!entry.isDirectory()) {
                                    outZipFile.putNextEntry(entry);
                                    InputStream input = zipFile.getInputStream(entry);

                                    copyStream (input, outZipFile, error);
                                    outZipFile.closeEntry();
                                    input.close();
                                }
                            }
                        }
                        zipFile.close();
                    }

                    // Afegueix al fitxer ZIP comprimit, la lista de fitxers a comprimir.
                    if (error.getError()==null) {
                        int cont1 = 0;
                        while (cont1<fileNames.length && error.getError()==null) {

                            String fileName = FileUtils.concatFolder(fromFolder, fileNames[cont1]);
                            FileInputStream input = new FileInputStream (new File (fileName));
                            ZipEntry entry = new ZipEntry (fileNames[cont1]);
                            outZipFile.putNextEntry (entry);
                            copyStream (input, outZipFile, error);
                            input.close();
                            numFiles++;

                            cont1++;
                        }
                    }

                    outZipFile.close();

                    if (tmpFileName!=null && error.getError()==null) { FileUtils.deleteFile (tmpFileName, error); }
                }

            }
            catch (IOException ex) { error.setError ("IOEXCEPTION", "ACTION: ZIP_FILES", "DESCRIPTION: Se ha producido un error al comprimir los ficheros comprimidos", "FILENAME: "+zipFileName, "EXCEPTION: "+ex.getMessage()); }
            catch (Exception ex) { error.setError ("EXCEPTION", "ACTION: ZIP_FILES", "DESCRIPTION: Se ha producido un error al comprimir los ficheros comprimidos", "FILENAME: "+zipFileName, "EXCEPTION: "+ex.getMessage()); }
        }
        else { error.setError ("EXCEPTION", "ACTION: DELETE_FILES", "DESCRIPTION: No se ha indicado el nombre del archivo al que comprimir o los nombres de los ficheros a comprimir", "FILENAME: "+zipFileName, "EXCEPTION: Null or empty exception"); }

        return numFiles;
    }


    /**
     * Afegueix, al fitxer "zipFileName" els arxius inclosos a la taula "fileNames";
     * si el fitxer "zipFileName" no existeix, el crea.
     *
     * NOTA: Per a realitzar l'operació, es re-anomena el fitxer comprimir assignant-li
     * l'extenció <b>.tmp</b>; després, crea l'arxiu "zipFileName" i hi copia els
     * arxius, excepte els que apareixen a la taula "fileNames". Finalment,
     * hi comprimeix els arxius incloses a la taula "fileNames" iesborra el fitxer
     * temporal.
     *
     * D'aquesta manera, si la llista "fileNames" inclou un arxiu amb el mateix
     * nom que algun dels arxius del fitxer comprimit, aquest queda substituït.
     *
     * NOTA: Els arxius s'enmagatzemen amb la ruta complerta (inclosa la unitat).
     *
     * NOTA: Si no s'indica el nom del fitxer ZIP o no s'indiquen la llista de fitxers
     * a afeguir, no es realitza cap operació i es retorna un 0.
     *
     * @param zipFileName Nom del fitxer a descomprimir
     * @param fromFolder Carpeta on estan localitzats els arxius a comprimir.
     * @param fileNames Llista de fitxers que s'han d'eliminar
     * @param error
     * @return Retorna el número d'arxius afeguits al fitxer comprimit.
     */
    public static int zipFiles (String zipFileName, String[] fileNames, ErrorEx error) {
        return zipFiles (zipFileName, null, fileNames, error);
    }

    public static int zipFiles (String zipFileName, String fromFolder, Vector<File> files, ErrorEx error) {

        fromFolder = FileUtils.concatFolder (fromFolder, "");

        String[] names = new String[files.size()];
        fromFolder = StringUtils.substChar (fromFolder, '\\', '/', StringUtils.SUBST_ALL, StringUtils.FROM_BEGIN, StringUtils.CASE_SENSITIVE);

        // Genera la llista de fitxers amb el seu nom amb una ruta relativa a
        // la carpeta "fromFolder".
        for (int cont1=0; cont1<files.size(); cont1++) {

            File file = files.get(cont1);
            names[cont1] = StringUtils.substString (file.getPath(), File.separator, "/", StringUtils.SUBST_ALL, StringUtils.FROM_BEGIN, StringUtils.CASE_SENSITIVE);
            
            if (StringUtils.copy (names[cont1], fromFolder.length()).equalsIgnoreCase(fromFolder)) {
                names[cont1] = StringUtils.copy (names[cont1], fromFolder.length(), -1);
            }
        }

        return zipFiles (zipFileName, fromFolder, names, error);

    }
}
