/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package cat.lib.geometry;

import cat.lib.math.MathEx;

/**
 *    _____ _                       _  _  _     _             _  _
 *   / ____) |                     | || || |   (_)           | || |
 *  | |    | |  _ _ ___ ___  ___   |_||_|| |    _ _ __   ___ |_||_|
 *  | |    | |/ _` | __) __)/ _ \        | |   | | `_  \/ _ \
 *  | |____| | (_| |__ \__ \  __/        | |___| | | | |  __/
 *   \_____)_|\__,_|___/___/\___|        |_____)_|_| |_|\___|
 *
 *
 * Aquesta classe encapsula una linea definida a partir de les coordenades (X,Y) dels dos extrems.
 *
 * Aporta el métode "toVector" per a passar la línea a un vector i un métode "toArea" per a passar la línea
 * a un objecte de tipus "Area".
 *
 * @author Miquel Abadal i Cervià
 */
public class Line {


    //     __   _        _ _           _
    //   / __ \| |      (_) |         | |
    //  | |__| | |_ _ __ _| |_   _   _| |_ ___
    //  |  __  | __) `__) | `_ \| | | | __) __)
    //  | |  | | |_| |  | | |_) | |_| | |_\__ \
    //  |_|  |_|\__)_|  |_|_,__/ \__,_|\__)___/
    //
    //

    private int ROUND_PRECISION = 100;

    protected double x1;
    protected double y1;
    protected double x2;
    protected double y2;

    // Linia expressada com a funció matemàtica: y = ax + b
    protected double a = 0;
    protected double b = 0;


    //   __  __      _            _                              _                    _
    //  |  \/  |    | |          | |                            | |                  | |
    //  | \  / | ___| |_ ___   __| | ___     ____ ___  _ __  ___| |_ _ __ _   _  ____| |_ ___  _ __
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \   / ___) _ \| `_  \ __) __) `__) | | |/ ___) __) _ \| `__)
    //  | |  | |  __/ |_ (_) | (_| |  __/  | (___ (_) | | | |__ \ |_| |  | |_| | (___| |_ (_) | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|   \____)___/|_| |_|___/\__)_|   \__,_|\____)\__)___/|_|
    //
    //

    public Line (double x1, double y1, double x2, double y2) {
        this.x1 = x1;
        this.y1 = y1;
        this.x2 = x2;
        this.y2 = y2;
        
        calculaRecta();
    }

    public Line (Point point1, Point point2) {
        this.x1 = point1.x;
        this.y1 = point1.y;
        this.x2 = point2.x;
        this.y2 = point2.y;

        calculaRecta();
    }

    private void calculaRecta() {

        // Calcula la recta com a fórmula matemàtica expressada com a "y = ax + b"
        //   x -x1       y - y1
        // ---------- = -----------
        //  x2 - x1      y2 - y1

        double denominador1 = x2-x1;
        double denominador2 = y2-y1;

        //     x -x1            y - y1
        // -------------- = --------------
        //  denominador1     denominador2

        //   (  x -x1 ) *denominador2
        // --------------------------- = y - y1
        //         denominador1

        //   (  x -x1 ) * factor1 = y - y1

        a = denominador2/denominador1;

        //   (  x * factor1 ) - (x1 * factor1) + y1 = y

        b = y1 - (x1*a);

    }

    //   __  __      _            _             _  _   _____ ______ _______ _  _
    //  |  \/  |    | |          | |           | || | / ____)  ____)__   __) || |
    //  | \  / | ___| |_ ___   __| | ___ ___   |_||_|| (___ | |__     | |  |_||_|
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)         \___ \|  __)    | |
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \         ____) | |____   | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/        (_____/|______)  |_|
    //
    //

    public void setX1 (double x1) { this.x1 = x1; }
    public void setY1 (double y1) { this.y1 = y1; }
    public void setX2 (double x2) { this.x2 = x2; }
    public void setY2 (double y2) { this.y2 = y2; }


    //   __  __      _            _             _  _   _____ ______ _______ _  _
    //  |  \/  |    | |          | |           | || | / ____)  ____)__   __) || |
    //  | \  / | ___| |_ ___   __| | ___ ___   |_||_|| |  __| |__     | |  |_||_|
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)        | | (_ |  __)    | |
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \        | |__| | |____   | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/         \_____|______)  |_|
    //
    //

    public double getX1 () { return x1; }
    public double getY1 () { return y1; }
    public double getX2 () { return x2; }
    public double getY2 () { return y2; }

    public double getLength () {

        double base = x1-x2;
        double alt = y1-y2;
        return Math.sqrt ((base*base)+(alt*alt));
    }

    private boolean inRange (double minValue, double maxValue, double value) {

        boolean res = false;

        minValue = Math.round (minValue*ROUND_PRECISION);
        maxValue = Math.round (maxValue*ROUND_PRECISION);
        value = Math.round (value*ROUND_PRECISION);

        if (maxValue<minValue) {
            double temp = minValue;
            minValue = maxValue;
            maxValue = temp;
        }
        if (value>=minValue && value<=maxValue) { res = true; }
        return res;
    }

    //   __  __      _            _                     _     _ _
    //  |  \/  |    | |          | |                   | |   | |_)
    //  | \  / | ___| |_ ___   __| | ___    _ __  _   _| |_  | |_  ____ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \  | `_ \| | | | `_ \| | |/ ___) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/  | |_) | |_| | |_) | | | (___\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|  |  __/ \__,_|_,__/|_|_|\____)___/
    //                                     | |
    //                                     |_|

    public LineVector toVector () {

        double base = x1-x2;
        double alt = y1-y2;
        double modul = Math.sqrt ((base*base)+(alt*alt));
        double angle = 0;
        if (modul!=0) { angle = Math.asin (base / modul) + (Math.PI / 2); }
        if (alt<0) { angle= (Math.PI - angle) + Math.PI; }

        angle = MathEx.radiansToGraus (angle);

        return new LineVector (modul, angle);
    }

    public Area toArea() { return new Area (x1, y1, x2, y2); }

    public Point lineIntersection (Line line) {

        Point res = null;

        if (!Double.isInfinite(a)) {
            double x = (line.b - b) / (a - line.a);
            double y = x*a + b;

            if (inRange (x1, x2, x) && inRange (y1, y2, y) &&
                inRange (line.x1, line.x2, x) && inRange (line.y1, line.y2, y)) { res = new Point (x, y); }
        }
        else {
            double x = x1;
            double y = (line.a * x1) + line.b;

            if (inRange (x1, x2, x) && inRange (y1, y2, y) &&
                inRange (line.x1, line.x2, x) && inRange (line.y1, line.y2, y)) { res = new Point (x, y); }
        }
        return res;
    }
}
