/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package cat.lib.geometry;

import cat.lib.utils.StringToNumber;
import cat.lib.utils.StringUtils;

/**
 *
 *   _____ _                       _  _  ____       _       _    _  _
 *  / ____) |                     | || ||  __ \    (_)     | |  | || |
 * | |    | |  _ _ ___ ___  ___   |_||_|| |__) |__  _ _ __ | |_ |_||_|
 * | |    | |/ _` | __) __)/ _ \        |  ___/ _ \| | `_  \ __)
 * | |____| | (_| |__ \__ \  __/        | |  | (_) | | | | | |_
 *  \_____)_|\__,_|___/___/\___|        |_|   \___/|_|_| |_|\__)
 *
 *
 * Aquesta classe encapsula un punt definit a partir de les seves coordenades (X,Y).
 *
 * Aporta un métode "puntInArea" per a verifiacar si el punt es troba a dins de l'àrea indicada.
 *
 * @author Miquel Abadal i Cervià
 */
public class Point {

    //     __   _        _ _           _
    //   / __ \| |      (_) |         | |
    //  | |__| | |_ _ __ _| |_   _   _| |_ ___
    //  |  __  | __) `__) | `_ \| | | | __) __)
    //  | |  | | |_| |  | | |_) | |_| | |_\__ \
    //  |_|  |_|\__)_|  |_|_,__/ \__,_|\__)___/
    //
    //

    protected double x;
    protected double y;


    //   __  __      _            _                              _                    _
    //  |  \/  |    | |          | |                            | |                  | |
    //  | \  / | ___| |_ ___   __| | ___     ____ ___  _ __  ___| |_ _ __ _   _  ____| |_ ___  _ __
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \   / ___) _ \| `_  \ __) __) `__) | | |/ ___) __) _ \| `__)
    //  | |  | |  __/ |_ (_) | (_| |  __/  | (___ (_) | | | |__ \ |_| |  | |_| | (___| |_ (_) | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|   \____)___/|_| |_|___/\__)_|   \__,_|\____)\__)___/|_|
    //
    //

    public Point(double x, double y) {
        this.x = x;
        this.y = y;
    }

    public Point (String punts) {

        String xStr = StringUtils.getParam(punts, ',', 0);
        String yStr = StringUtils.getParam(punts, ',', 1);
        this.x = StringToNumber.doubleValue (xStr);
        this.y = StringToNumber.doubleValue (yStr);
    }



    //   __  __      _            _             _  _   _____ ______ _______ _  _
    //  |  \/  |    | |          | |           | || | / ____)  ____)__   __) || |
    //  | \  / | ___| |_ ___   __| | ___ ___   |_||_|| (___ | |__     | |  |_||_|
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)         \___ \|  __)    | |
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \         ____) | |____   | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/        (_____/|______)  |_|
    //
    //

    public void setX (double x) { this.x = x; }
    public void setY (double y) { this.y = y; }


    //   __  __      _            _             _  _   _____ ______ _______ _  _
    //  |  \/  |    | |          | |           | || | / ____)  ____)__   __) || |
    //  | \  / | ___| |_ ___   __| | ___ ___   |_||_|| |  __| |__     | |  |_||_|
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)        | | (_ |  __)    | |
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \        | |__| | |____   | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/         \_____|______)  |_|
    //
    //

    public double getX () { return x; }
    public double getY () { return y; }


    //   __  __      _            _                     _     _ _
    //  |  \/  |    | |          | |                   | |   | |_)
    //  | \  / | ___| |_ ___   __| | ___    _ __  _   _| |_  | |_  ____ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \  | `_ \| | | | `_ \| | |/ ___) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/  | |_) | |_| | |_) | | | (___\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|  |  __/ \__,_|_,__/|_|_|\____)___/
    //                                     | |
    //                                     |_|

    public boolean pointInArea (Area area) {
        return (x>=area.left && x<=area.right && y>=area.top && y<=area.bottom);
    }

    public double distance (Point point) {

        if (point!=null) {
            double distX = point.getX() - x;
            double distY = point.getY() - y;
            return (int) Math.sqrt (distX * distX + distY * distY);
        }
        return Double.MAX_VALUE;
    }

    /**
     * Obté un punt situat entre el punt de la instància i el punt indicat amb el paràmetre "Point"
     * @param point
     * @return
     */
    public Point getEquidistantPoint (Point point) {

        if (point!=null) {
            double distX = (point.getX() - x)/2;
            double distY = (point.getY() - y)/2;
            return new Point (x+distX, y+distY);
        }
        return null;
    }

    /**
     * Comproba si el valor enter de les coordenades del punt son iguals a les coordenades del punt indicat
     * @param point
     * @return
     */
    public boolean intEqual (Point point) {
        
        boolean equal = false;
        int x1 = (int) x;
        int y1 = (int) y;
        int x2 = (int) point.x;
        int y2 = (int) point.y;

        if (x1==x2 && y1==y2) { equal = true; }
        return equal;

    }
}
