//    _____ _                       _  _   _____            _   _    _ _   _ _    _  _
//   / ____) |                     | || | / ____)          | | | |  | | | (_) |  | || |
//  | |    | |  _ _ ___ ___  ___   |_||_|| (___   ___  _ __| |_| |  | | |_ _| |____||_|
//  | |    | |/ _` | __) __)/ _ \         \___ \ / _ \| `__) __) |  | | __) | | __)
//  | |____| | (_| |__ \__ \  __/         ____) | (_) | |  | |_| |__| | |_| | |__ \
//   \_____)_|\__,_|___/___/\___|        (_____/ \___/|_|   \__)\____/ \__)_|_|___/
//
//

package cat.lib.sort;

import cat.lib.utils.StringUtils;
import java.util.Collections;
import java.util.List;

/**
 *
 * Aquesta classe implementa diferents métodes estátics per a ordenar llistes.
 *
 * @author Miquel Abadal i Cervià
 */
public class SortUtils {

    /**
     * Ordena la llista "list" utilitzant el atribut "field".
     *
     * Per a que aquest métode es comporti adequadament, cal que tots els elements
     * que formen part de la llista siguin de la mateixa classe i implementin el
     * corresponent métode "method".
     *
     * Els elements que no compleixin aquest requisits NO s'ordenaran i es comportaran
     * com si tinguessin valor "null".
     *
     * Per ejemple, si tenim una llista formada per instàncies de la classe "Client",
     * la qual implemente el métode "getCodi()", podem ordenar-la executant:
     *
     * |--------------------------------------|
     * | SortUtils.sort (llista, "getCodi");  |
     * |--------------------------------------|
     *
     * Si tenim un mapa i el volem ordenar en una llista, llavors, abans d'ordenar,
     * hem de passar el contingut del mapa a una llista:
     *
     * |----------------------------------------------------|
     * | Map metodes = applicationBean.getMetodes();        |
     * | List llista = new LinkedList (metodes.values());   |
     * | SortUtils.sort (llista, "getCodi");                |
     * |----------------------------------------------------| 
     *
     * @param list Llista a ordenar
     * @param field Atribut a utilitzar per definir el ordre.
     */
    public static void sort (List list, String method) {

        Collections.sort (list, new MultiFieldComparator (new String[] {method}));
    }

    public static void sort (List list, String[] methods) {

        Collections.sort (list, new MultiFieldComparator (methods));
    }

    public static void sort (List list, String method, String prefix) {

        Collections.sort (list, new MultiFieldComparator (new String[] {prefix+StringUtils.firstCharToUpperCase(method)}));

    }
    
    public static void sortDesc (List list, String method) {

        Collections.sort (list, new MultiFieldComparator (new String[] {method}, true));
    }

    public static void sortDesc (List list, String[] methods) {

        Collections.sort (list, new MultiFieldComparator (methods, true));
    }

    public static void sortDesc (List list, String method, String prefix) {

        Collections.sort (list, new MultiFieldComparator (new String[] {prefix+StringUtils.firstCharToUpperCase(method)}, true));

    }    



    /**
     * Desordena aleatòriament els elements de la llista "list".
     * 
     * @param list
     */
    public static void shuffle (List list) {

        Collections.shuffle (list);
    }
}
