//    _____ _                       _  _   _____ _        _             ____                            _  _
//   / ____) |                     | || | / ____) |      (_)           |  __ \                         | || |
//  | |    | |  _ _ ___ ___  ___   |_||_|| (___ | |_ _ __ _ _ __   __ _| |__) |_ _ _ __  _ _ _ _ __  ____||_|
//  | |    | |/ _` | __) __)/ _ \         \___ \| __) `__) | `_  \/ _` |  ___/ _` | `__) _` | ` `  \/ __)
//  | |____| | (_| |__ \__ \  __/         ____) | |_| |  | | | | | (_| | |  | (_| | | | (_| | |\ |\ |__ \
//   \_____)_|\__,_|___/___/\___|        (_____/ \__)_|  |_|_| |_|\__, |_|   \__,_|_|  \__,_|_||_||_|___/
//                                                                 __| |
//                                                                (___/


package cat.lib.strings;

import cat.lib.utils.StringToNumber;
import java.util.Vector;


/**
 *
 * La classe <code>StringsParams</code> permet segmentar una cadena de text
 * que utilitza un caracter "separador" en diverses cadenes i després
 * recuperarles a mitjançant un índex.
 *
 * Internament quan es crea un objecte de la classe <code>StringParams</code> o
 * quan s'invoca el metode <code>setText</code>, el text es segmenta en diverses
 * cadenes i s'enmagatzemen en un <code>Vector</code>, de manera que quan s'han
 * de recuperar l'accés és molt eficient.
 *
 * La característica interessant que aporta aquesta classe, es que al detectar
 * el caracter "separador" comprova si està o no dintre d'un literal, ja que en
 * aquest cas no l'utilitza per a segmentar la cadena. Els literals, tan poden
 * utilitzar el caràcter cometes dobles (") com cometes simples ('), l'única
 * restricció, és que el literal no es donarà per completat fins que no aparegui
 * la mateixa cometa amb la qual s'ha iniciat.
 *
 * Per ejemple la cadena:
 * "Hola, el meu nom és: "Abadal, Miquel", i sóc de Barcelona", quedaria de la
 * següent manera:
 *
 * Linea 0 : "Hola"
 * Linea 1 : "el meu nom és "Abadal, Miquel"
 * Linea 2 : "i sóc de Barcelona"
 *
 * Un cop construïda la llista de paràmetres els podem recuperar amb
 * <code>getParam</code>, o modificar amb <code>setParam</code>, també podem
 * afeguir paràmetres nous amb <code>addParam</code> o esborrar-ne d'existents
 * amb <code>removeParam</code> o esborrarlos tots amb <code>clear</code>.
 *
 * Amb el métode <code>setText</code> podem entrar una nova cadena de paràmetres
 * a segmentar i amb el métode <code>getText</code> podem reconstruir la cadena
 * complerta amb els caràcters separadors originals, però incluïnt els canvis
 * que s'haguin pogut realitzar sobre els diferents paràmetres.
 *
 * Finalment, també cal destacar que aquesta classe aporta métodes per a
 * convertir, automàticament, el text enmagatzemat en un cadascun dels paràmetres
 * als valors numérics <code>byte</code>, <code>short</code>, <code>int</code>,
 * <code>long</code>, <code>float/code> i <code>double</code>.
 *
 * @author Miquel Abadal i Cervià
 */

public class ParamsList extends StringsToVector {



    //   __  __      _            _                                  _                    _
    //  |  \/  |    | |          | |                                | |                  | |
    //  | \  / | ___| |_ ___   __| | ___ ___     ____ ___  _ __  ___| |_ _ __ _   _  ____| |_ ___  _ __ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)   / ___) _ \| `_  \ __) __) `__) | | |/ ___) __) _ \| `__) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \  | (___ (_) | | | |__ \ |_| |  | |_| | (___| |_ (_) | |  \__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/   \____)___/|_| |_|___/\__)_|   \__,_|\____)\__)___/|_|  (___/
    //
    //

    public ParamsList (String text, char separador, boolean autoTrim, boolean detectLiterals) { super (text, separador, autoTrim,  detectLiterals); }
    public ParamsList (String text, char separador, boolean detectLiterals) { super (text, separador, AUTO_TRIM,  detectLiterals); }
    public ParamsList (String text, char separador) { super (text, separador, AUTO_TRIM,  DETECT_LITERALS); }
    public ParamsList (char separador) { super ("", separador, AUTO_TRIM,  DETECT_LITERALS); }


    //   __  __      _            _                         _     _ _
    //  |  \/  |    | |          | |                       | |   | |_)
    //  | \  / | ___| |_ ___   __| | ___ ___    _ __  _   _| |_  | |_  ____ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)  | `_ \| | | | `_ \| | |/ ___) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \  | |_) | |_| | |_) | | | (___\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/  |  __/ \__,_|_,__/|_|_|\____)___/
    //                                         | |
    //                                         |_|

    /**
     * Retorna el número de paràmetres disponibles.
     *
     * @return Número de paràmetres
     */
    public int paramCount () { return super.getSize (); }

    /**
     * Afegueix un paràmetre amb el valor <code>value</code>
     *
     * @param value Valor a assignar al paràmetre.
     */
    public void addParam (String value) { super.add (value); }


    /**
     * Esborra el paràmetre situat a la posició <code>index</code>.
     *
     * <code>Nota:</code>Si el índex està fora de rang no realitza cap acció.
     *
     * @param index Posició del paràmetre a eliminar.
     */
    public void removeParam (int index) { super.remove (index); }


    /**
     * Esborra tots els parametres.
     *
     */
    @Override
    public void clear () { super.clear(); }

    /**
     * Esborra tots els paràmetres i en genera de nous utilitzant el
     * <code>text</code> indicat i el separador que cal utilitzar.
     *
     * <code>Nota:</code>Al construïr la llista de paràmetres, no utilitza
     * els caràcters separadors situats a dintre d'un literal.
     *
     * @param text Text original
     * @param separador Caràcter utilitzat per a separar els paràmetres
     */
    @Override
    public void setText (String text, char separador) { super.setText (text, separador); }

    /**
     * Modifica el contingut del paràmetre situat a la posició <code>index</code>
     * i li assigna el valor <code>value</code>.
     *
     * <code>Nota:</code>Si el índex està fora de rang no realitza cap acció.
     *
     * @param index Índex del paràmetre a modificar
     * @param value Nou valor a assignar al paràmetre
     */
    public void setParam (int index, String value) { super.setLine(index, value); }

    /**
     * Reconstrueix la cadena de text a partir dels paràmetres actuals i unint-los
     * mitjançant el caracter separador.
     *
     * @return Retorna una cadena de text amb tots els paràmetres separats amb el
     * caràcter separador.
     */
    @Override
    public String getText () { return super.getText(); }

    /**
     * @return Retorna la cadena de text en forma de Vector on cada paràmetre és un 
     * element.
     */
    @Override
    public Vector getLines() { return super.getLines(); }

    /**
     * Obté el valor del paràmetre situat a la posició <code>index</code>.
     *
     * <code>Nota:</code>Si el índex està fora de rang retorna una cadena en
     * blanc.
     *
     * @param index Índex del paràmetre
     * @return Retorna el valor del paràmetre
     */
    public String getParam (int index) { return super.getLine (index); }

    public boolean booleanValue (int index) { return StringToNumber.booleanValue (super.getLine (index)); }
    public byte byteValue (int index) { return StringToNumber.byteValue (super.getLine (index)); }
    public short shortValue (int index) { return StringToNumber.shortValue (super.getLine (index)); }
    public int intValue (int index) { return StringToNumber.intValue (super.getLine (index)); }
    public long longValue (int index) { return StringToNumber.longValue (super.getLine (index)); }
    public float floatValue (int index) { return StringToNumber.floatValue (super.getLine (index)); }
    public double doubleValue (int index) { return StringToNumber.doubleValue (super.getLine (index)); }


}
