//    _____ _                       _  _   _____ _        _             _     _     _  _  _
//   / ____) |                     | || | / ____) |      (_)           | |   (_)   | || || |
//  | |    | |  _ _ ___ ___  ___   |_||_|| (___ | |_ _ __ _ _ __   __ _| |    _ ___| |__||_|
//  | |    | |/ _` | __) __)/ _ \         \___ \| __) `__) | `_  \/ _` | |   | | __) __)
//  | |____| | (_| |__ \__ \  __/         ____) | |_| |  | | | | | (_| | |___| |__ \ |_
//   \_____)_|\__,_|___/___/\___|        (_____/ \__)_|  |_|_| |_|\__, |_____)_|___/\__)
//                                                                 __| |
//                                                                (___/


package cat.lib.strings;

import java.util.Vector;


/**
 *
 * La classe <code>StringList</code> permet enmagatzemar una llista de cadenes
 * (utilitzant un Vector) i modificarles i/o recuperarles a partir del seu índex.
 *
 * La característica interessant que aporta aquesta classe, es que l'entrada
 * de cadenes es pot realitzat manualment amb els métodes <code>add</code>,
 * <code>remove</code>, <code>setLine</code>... però també mitjançant una cadena
 * única en la qual s'utilitza un caràcter salt de línea '\n' per  a determinar
 * el separador entre línies.
 *
 * Per ejemple la cadena:
 * "Hola,\nem dic Miquel,\ni sóc de Barcelona", quedaria de la següent manera:
 *
 * Linea 0 : "Hola,"
 * Linea 1 : "em dic Miquel,"
 * Linea 2 : "i sóc de Barcelona"
 *
 * Un cop construïda la llista de cadenes podem recuperar-ne les linies,
 * modificar-les, afeguir-ne de noves o treure'n i inclús reconstruïr la nova
 * cadena de text com una única cadena amb els salts de linea originals.
 *
 * Finalment, també cal destacar que aquesta classe aporta métodes per a
 * convertir, automàticament, el text enmagatzemat en una de les línies a valors
 * numérics <code>byte</code>, <code>short</code>, <code>int</code>,
 * <code>long</code>, <code>float/code> i <code>double</code>.
 *
 * @author Miquel Abadal i Cervià
 */
public class StringsList extends StringsToVector {


    //   __  __      _            _                                  _                    _
    //  |  \/  |    | |          | |                                | |                  | |
    //  | \  / | ___| |_ ___   __| | ___ ___     ____ ___  _ __  ___| |_ _ __ _   _  ____| |_ ___  _ __ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)   / ___) _ \| `_  \ __) __) `__) | | |/ ___) __) _ \| `__) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \  | (___ (_) | | | |__ \ |_| |  | |_| | (___| |_ (_) | |  \__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/   \____)___/|_| |_|___/\__)_|   \__,_|\____)\__)___/|_|  (___/
    //
    //

    public StringsList (String text, boolean autoTrim) { super (text, '\n', autoTrim, DONT_DETECT_LITERALS); }

    public StringsList (String text) { super (text, '\n', NO_AUTO_TRIM, DONT_DETECT_LITERALS); }

    public StringsList () { super ("", '\n', NO_AUTO_TRIM, DONT_DETECT_LITERALS); }

    //   __  __      _            _                         _     _ _
    //  |  \/  |    | |          | |                       | |   | |_)
    //  | \  / | ___| |_ ___   __| | ___ ___    _ __  _   _| |_  | |_  ____ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)  | `_ \| | | | `_ \| | |/ ___) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \  | |_) | |_| | |_) | | | (___\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/  |  __/ \__,_|_,__/|_|_|\____)___/
    //                                         | |
    //                                         |_|

    /** Esborra totes les linies.
     *
     */
    @Override
    public void clear () { super.clear(); }


    /**
     * Afegueix una línia a la llista
     *
     * @param text Text a afeguir a la llista
     */
    @Override
    public void add (String text) { super.add (text); }


    /**
     * Esborra la línia situada a la posició <code>index</code>.
     *
     * <code>Nota:</code>Si el índex està fora de rang no realitza cap acció.
     *
     * @param index Posició de la línia a eliminar.
     */
    @Override
    public void remove (int index) { super.remove (index); }


    //   __  __      _            _             _  _   _____ ______ _______ _  _
    //  |  \/  |    | |          | |           | || | / ____)  ____)__   __) || |
    //  | \  / | ___| |_ ___   __| | ___ ___   |_||_|| (___ | |__     | |  |_||_|
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)         \___ \|  __)    | |
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \         ____) | |____   | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/        (_____/|______)  |_|
    //
    //

    /**
     * Esborra totes les línies i en genera de noves utilitzant el
     * <code>text</code> indicat.
     *
     * @param text Text original
     */
    public void setText (String text) { super.setText (text, '\n'); }


    /**
     * Modifica el contingut de la línia situada a la posició <code>index</code>
     * i li assigna el valor <code>text</code>.
     *
     * <code>Nota:</code>Si el índex està fora de rang no realitza cap acció.
     *
     * @param index Índex de la línia a modificar
     * @param text Nou text a assignar a la línia
     */
    @Override
    public void setLine (int index, String text) { super.setLine (index, text); }

    @Override
    public void setLastLine (String text) { super.setLastLine (text); }

    /**
     * Modifica el atribut <code>autoTrim</code> que seveix per a indicarli als
     * métodes de recuperació de línies (inclos el métode <code>getText</code>
     * que reconstrueix la cadena, que ha d'eliminar tots els caràcters en blanc
     * que apareguin al principi i final de cadascuna de les línies
     *
     * @param autoTrim <code>true</code> si es vol activar la funció <code>
     * autoTrim</code> o <code>false</code> en cas contrari.
     */
    @Override
    public void setAutoTrim (boolean autoTrim) { super.setAutoTrim(autoTrim); }



    //   __  __      _            _             _  _   _____ ______ _______ _  _
    //  |  \/  |    | |          | |           | || | / ____)  ____)__   __) || |
    //  | \  / | ___| |_ ___   __| | ___ ___   |_||_|| |  __| |__     | |  |_||_|
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)        | | (_ |  __)    | |
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \        | |__| | |____   | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/         \_____|______)  |_|
    //
    //

    /**
     * Reconstrueix la cadena de text a partir de les línies actuals i unint-los
     * mitjançant el salt de línia '\n'.
     *
     * @return Retorna una cadena de text amb totes les línies separades amb el
     * salt de línia '\n'.
     */
    @Override
    public String getText () { return super.getText(); }

    /**
     * Obté el contingut de la línia situada a la posició <code>index</code>.
     *
     * <code>Nota:</code>Si el índex està fora de rang retorna amb el valor
     * <code>null</code>
     *
     * @param index Índex de la línia
     * @return Retorna el contingut de la línia
     */
    @Override
    public String getLine (int index) { return super.getLine (index); }

    /**
     * @return Retorna la cadena de text en forma de Vector on cada paràmetre és un
     * element.
     */
    @Override
    public Vector getLines() { return super.getLines(); }

    /**
     * Retorna el número de línies disponibles.
     *
     * @return Número de línies
     */
    public int size () { return super.getSize (); }


    /**
     * Obté el estat del atribut <code>autoTrim</code>.
     *
     * @return <code>true</code> si està activat o <code>false</code> en cas
     * contrari.
     */
    @Override
    public boolean isAutoTrim () { return super.isAutoTrim(); }



}

