//    _____ _                       _  _  _    _ _   _         _    _ _   _ _    _  _
//   / ____) |                     | || || |  | | | | |       | |  | | | (_) |  | || |
//  | |    | |  _ _ ___ ___  ___   |_||_|| |__| | |_| |_ _ __ | |  | | |_ _| |____||_|
//  | |    | |/ _` | __) __)/ _ \        |  __  | __) __) `_ \| |  | | __) | | __)
//  | |____| | (_| |__ \__ \  __/        | |  | | |_| |_| |_) | |__| | |_| | |__ \
//   \_____)_|\__,_|___/___/\___|        |_|  |_|\__)\__)  __/ \____/ \__)_|_|___/
//                                                      | |
//                                                      |_|

package cat.lib.tcpIP;

import cat.lib.errors.ErrorEx;
import cat.lib.utils.StreamUtils;
import cat.lib.utils.StringUtils;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.Iterator;
import java.util.Map;

/**
 *
 * @author Miquel Abadal i Cervià
 */
public class HttpUtils {

    //   __  __      _            _                      _        _   _
    //  |  \/  |    | |          | |                    | |      | | (_)
    //  | \  / | ___| |_ ___   __| | ___ ___     ___ ___| |_  _ _| |_ _  ____ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)   / _ \ __) __) _` | __) |/ ___) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \  |  __/__ \ |_ (_| | |_| | (___\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/   \___|___/\__)__,_|\__)_|\____)___/
    //
    //

    public static final int METHOD_GET = 0;
    public static final int METHOD_POST = 1;
    public static final int DEFAULT_TIME_OUT = 5000; // 5 segons

    public static String getCookies (HttpURLConnection connection) {
        String res = null;
        try { res = connection.getHeaderField("Set-Cookie"); }
        catch (Exception e) {}
        return res;
    }

    public static String getCookie (HttpURLConnection connection, String cookieName) {

        String res = null;
        String cookies = getCookies (connection);
        if (cookies!=null) {
            int posIni = StringUtils.stringPos(cookies, cookieName);
            if (posIni!=-1) {
                posIni = posIni+cookieName.length()+1;
                int posFin = StringUtils.charPos (cookies, ';', posIni);
                res = StringUtils.copy (cookies, posIni, posFin-posIni);
            }
        }
        return res;
    }

    /*
     * Obté el identificador de sessió.
     *
     * Si el servidor al que es realitza la connexió utilitza Java, la cookie utilitzada
     * per transmetre la sessió és "JSESSIONID"; si les un servidor ASP, llavors utilitza
     * la cookie "ASPSESSIONID".
     *
     * Altres tecnologies poden utilitzar identificadors diferents.
     *
     */
    public static String getSessionId (HttpURLConnection connection) {
        String res = getCookie (connection, "JSESSIONID");
        if (res==null) { res = getCookie (connection, "ASPSESSIONID"); }
        return res;
    }

    public static int getResponseCode (HttpURLConnection connection, ErrorEx error) {
        
        int res = 500;
        try { res = connection.getResponseCode(); }
        catch (IOException e) { error.setError ("REQUEST_ERROR", "TRACE: HttpUtils.getResponseCode", "CODE: IO_EXCEPTION", "DESCRIPTION: No se ha obtenido respuesta del servidor", "EXEPTION: "+e.getMessage()); }
        catch (Exception e) { error.setError ("REQUEST_ERROR", "TRACE: HttpUtils.getResponseCode", "CODE: IO_EXCEPTION", "DESCRIPTION: No se ha obtenido respuesta del servidor", "EXEPTION: "+e.getMessage()); }

        return res;
    }


    /**
     * ATENCIÓ: !!! MOLT IMPORTANT !!! Al acabar d'utilitzar el stream "InputStream" cal fer un close().
     */
    public static InputStream getInputStream (HttpURLConnection connection, String url, ErrorEx error) {

        InputStream res = null;

        if (connection!=null) {
            try {
                int responseCode = getResponseCode (connection, error);

                if (error.getError()==null) {
                    if (responseCode>=400) {
                        res = connection.getErrorStream();
                        error.setError ("RESPONSE_ERROR", "TRACE: HttpUtils.getInputStream", "DESCRIPTION: El servidor a devuelto como respuesta un mensaje de error", "ERROR_CODE: "+responseCode);
                    }
                    else { res = connection.getInputStream(); }
                }
            }
            catch (IOException ex) { error.setError ("REQUEST_ERROR", "TRACE: HttpUtils.getInputStream", "CODE: IO_EXCEPTION", "DESCRIPTION: Se ha producido un error durante la transmisión", "URL: "+url, "EXCEPTION: "+ex.getMessage()); }
        }
        else { error.setError ("REQUEST_ERROR", "TRACE: HttpUtils.getInputStream", "CODE: IO_EXCEPTION", "DESCRIPTION: No se ha podido establecer la comunicación", "CONNECTION: null", "URL: "+url); }


        return res;
    }

    public static void disconnect (HttpURLConnection connection, ErrorEx error) {

        if (connection!=null) { connection.disconnect(); }
        else { error.setError ("DISCONNECT_ERROR", "TRACE: HttpUtils.disconnect", "CODE: IO_EXCEPTION", "DESCRIPTION: No se ha podido desconectar la comunicación", "CONNECTION: null"); }
    }

    public static String encodeParams (Map params) {

        String res = null;

        if (params!=null) {
            Iterator i = params.entrySet().iterator();
            while (i.hasNext()) {
                Map.Entry entry = (Map.Entry) i.next();
                String key = entry.getKey().toString();
                Object valueObj = entry.getValue();
                String value = null;
                if (valueObj!=null) { value = valueObj.toString(); }
                if (!StringUtils.isEmpty (key) && !StringUtils.isEmpty (value)) {
                    try {
                        String param = URLEncoder.encode (key, "utf-8")+"="+URLEncoder.encode (value, "utf-8");
                        res = StringUtils.concat (res, "&", param);
                    }
                    catch (UnsupportedEncodingException e) {}
                }
            }
        }
        return res;
    }

    /**
     * Accede a la url "requestUrl" mediante el método indicado "POST/GET" y pasando,
     * si los hay, los parámetros indicados en la cadena "params".
     *
     * Si se utiliza el método "GET", se construye la url concatenando la url con
     * los parámetros (separándolos con el carácter "?"); aunque para que no se produzcan
     * errores, la url resultante no puede exceder de 128 caráctes.
     *
     * Si se utiliza el método "POST" (recomendado) los parámetros se envian mediante un
     * stream de salida por lo que no hay límite en el tamaño de la información enviada.
     *
     * NOTA: Los parámetrops deben indicarse mediante una cadena formada por parejas
     * "clave=valor" separadas con el caácter "&".
     *
     * Devuelve la respuesta obtenida de la petición HTTP en forma de stream
     */
    public static HttpURLConnection requestUrlConnection (String requestUrl, int method, String params, Map properties, String sessionId, int timeOut, ErrorEx error) {

        requestUrl = StringUtils.substString (requestUrl, " ", "%20", StringUtils.SUBST_ALL, StringUtils.FROM_BEGIN, StringUtils.CASE_INSENSITIVE);

        if (method==METHOD_GET) { requestUrl = StringUtils.concat (requestUrl, "?", params); }

        HttpURLConnection connection = null;

        try {
            URL url = new URL (requestUrl);
            connection = (HttpURLConnection) url.openConnection();

            if (timeOut>=0) {
                connection.setConnectTimeout(timeOut);
                connection.setReadTimeout(timeOut);
            }

            if (sessionId!=null) {
                // Si no s'indica la cookie utilitzada per enviar el id. de sessió, s'utilitza el JSESSIONID
                if (StringUtils.charPos(sessionId, '=')==-1) { sessionId = "JSESSIONID="+sessionId; }
                connection.addRequestProperty("Cookie", sessionId);
            }

            if (method==METHOD_POST) {
                connection.setDoOutput (true);
                connection.connect();

                if (properties!=null) {
                    Iterator iterator = properties.entrySet().iterator();
                    while (iterator.hasNext()) {
                        Map.Entry entry = (Map.Entry) iterator.next();
                        String property = (String) entry.getKey();
                        String value = (String) entry.getValue();
                        connection.setRequestProperty (property, value);
                    }
                }
                
                OutputStreamWriter out = new OutputStreamWriter (connection.getOutputStream());
                out.write (params);
                out.flush();
                out.close();
            }
            else if (method==METHOD_GET) { connection.connect(); }
        }
        catch (MalformedURLException ex) { error.setError ("REQUEST_ERROR", "TRACE: HttpUtils.requestUrl", "CODE: MALFORMED_URL", "DESCRIPTION: La url no és una dirección HTTP válida", "URL: "+requestUrl, "EXCEPTION: "+ex.getMessage()); }
        catch (SocketTimeoutException ex) { error.setError ("REQUEST_ERROR", "TRACE: HttpUtils.requestUrl", "CODE: TIME_OUT_EXCEPTION", "DESCRIPTION: No se ha podido establecer conexión con el servidor", "URL: "+requestUrl, "EXCEPTION: "+ex.getMessage()); }
        catch (IOException ex) { error.setError ("REQUEST_ERROR", "TRACE: HttpUtils.requestUrl", "CODE: IO_EXCEPTION", "DESCRIPTION: No se ha podido establecer conexión con el servidor", "URL: "+requestUrl, "EXCEPTION: "+ex.getMessage()); }
        catch (Exception ex) { error.setError ("REQUEST_ERROR", "TRACE: HttpUtils.requestUrl", "CODE: EXCEPTION", "DESCRIPTION: Se ha producido un error durante el proceso", "URL: "+requestUrl, "EXCEPTION: "+ex.getMessage()); }

        return connection;
    }


    public static String requestUrl (String url, int timeout, ErrorEx error) {

        String res = null;

        HttpURLConnection connection = HttpUtils.requestUrlConnection (url, HttpUtils.METHOD_GET, null, null, null, timeout, error);
        InputStream in = HttpUtils.getInputStream (connection, url, error);
        if (in!=null && error.getError()==null) { res = StreamUtils.inputStreamToString (in, "utf-8", error); }
        HttpUtils.disconnect(connection, error);

        return res;
    }

    public static String requestUrl (String url, int method, Map params, int timeout, ErrorEx error) {

        String res = null;

        HttpURLConnection connection = HttpUtils.requestUrlConnection (url, HttpUtils.METHOD_POST, HttpUtils.encodeParams (params), null, null, timeout, error);
        InputStream in = HttpUtils.getInputStream (connection, url, error);
        if (in!=null && error.getError()==null) { res = StreamUtils.inputStreamToString (in, "utf-8", error); }
        HttpUtils.disconnect(connection, error);

        return res;
    }
}
