/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package cat.lib.utils;

import cat.lib.locale.LocaleUtils;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;


class LapsedTimeLocale {

    String ISO1language = null;
    String[] segments = null;

    public LapsedTimeLocale (String ISO1language, String[] segments) {
        this.ISO1language = ISO1language;
        this.segments = segments;
    }
}

/**
 * Aquesta classe està destinada a poder mostrar les dates indicant el temps transcorregut i
 * utilitzant llenguatge natural en varis idiomes.
 *
 * NOTA:
 *
 * És recomanable construir-la en el projecte i conservar-la durant tota l'execució, per no haber-la
 * de construir i destruir.
 *
 * @author Miquel
 */
public class LapsedTime {

    private final int INDEX_PREV1 = 0;
    private final int INDEX_INSIDE = 1;
    private final int INDEX_AND = 2;
    private final int INDEX_AGO = 3;
    private final int INDEX_YEARS = 4;
    private final int INDEX_MONTHS = 5;
    private final int INDEX_DAYS = 6;
    private final int INDEX_HOURS = 7;
    private final int INDEX_MINUTES = 8;
    private final int INDEX_SECONDS = 9;
    private final int INDEX_YEAR = 10;
    private final int INDEX_MONTH = 11;
    private final int INDEX_DAY = 12;
    private final int INDEX_HOUR = 13;
    private final int INDEX_MINUTE = 14;
    private final int INDEX_SECOND = 15;

    private Map<String, LapsedTimeLocale> lapsedTimeLocales = null;

    public LapsedTime() {

        lapsedTimeLocales = new HashMap();

        LapsedTimeLocale lapsedTime_EN = new LapsedTimeLocale ("en", new String[] {"",       "Inside",    "&", "ago", "years",  "months", "days",  "hours", "minutes", "seconds",  "year", "month", "day", "hour", "minute", "second"});
        LapsedTimeLocale lapsedTime_ES = new LapsedTimeLocale ("es", new String[] {"Hace ",  "Dentro de", "y", "",    "años",   "meses",  "dias",  "horas", "minutos", "segundos", "año",  "mes",   "dia", "hora", "minuto", "segundo"});
        LapsedTimeLocale lapsedTime_CA = new LapsedTimeLocale ("ca", new String[] {"Fa ",    "D'aquí a",  "i", "",    "anys",   "mesos",  "dies",  "hores", "minuts",  "segons",   "any",  "mes",   "dia", "hora", "minut",  "segon"});

        lapsedTimeLocales.put (lapsedTime_EN.ISO1language, lapsedTime_ES);
        lapsedTimeLocales.put (lapsedTime_ES.ISO1language, lapsedTime_ES);
        lapsedTimeLocales.put (lapsedTime_CA.ISO1language, lapsedTime_CA);

    }

    public String toLapsedTime (Date valor, String format, Locale locale) {

        String text = null;
        if (locale!=null) {
            String language = LocaleUtils.getISO1Language (locale).toLowerCase();
            LapsedTimeLocale lapsedTimeLocale = lapsedTimeLocales.get (language);
            if (lapsedTimeLocale==null) { lapsedTimeLocale = lapsedTimeLocales.get("en"); }

            if (valor!=null) {

                if (lapsedTimeLocale!=null) {
                    long dataActual = System.currentTimeMillis();
                    long data = valor.getTime();
                    long lapsedTime = dataActual-data;

                    text = lapsedTimeLocale.segments[INDEX_PREV1];
                    if (lapsedTime<0) {
                        lapsedTime = data-dataActual;
                        text = lapsedTimeLocale.segments[INDEX_INSIDE];
                    }

                    long segons = lapsedTime / 1000;
                    long minuts = segons / 60; segons = segons - (minuts*60);
                    long hores = minuts / 60; minuts = minuts - (hores*60);
                    long dies = hores / 24; hores = hores - (dies*24);
                    long anys = dies / 365; dies = dies - (anys*365);
                    long mesos = dies / 30;

                    if (anys>10) { text = NumberToString.toString (valor, format); }
                    else if (anys==1 && mesos==1) { text = text+anys+" "+lapsedTimeLocale.segments[INDEX_YEAR]+" "+lapsedTimeLocale.segments[INDEX_AND]+" "+mesos+" "+lapsedTimeLocale.segments[INDEX_MONTH]; }
                    else if (anys==1) { text = text+anys+" "+lapsedTimeLocale.segments[INDEX_YEAR]+" "+lapsedTimeLocale.segments[INDEX_AND]+" "+mesos+" "+lapsedTimeLocale.segments[INDEX_MONTHS]; }
                    else if (anys>0 && mesos==1) { text = text+anys+" "+lapsedTimeLocale.segments[INDEX_YEARS]+" "+lapsedTimeLocale.segments[INDEX_AND]+" "+mesos+" "+lapsedTimeLocale.segments[INDEX_MONTH]; }
                    else if (anys>0) { text = text+anys+" "+lapsedTimeLocale.segments[INDEX_YEARS]+" "+lapsedTimeLocale.segments[INDEX_AND]+" "+mesos+" "+lapsedTimeLocale.segments[INDEX_MONTHS]; }
                    else if (mesos==1) { text = text+mesos+" "+lapsedTimeLocale.segments[INDEX_MONTH]; }
                    else if (mesos>0) { text = text+mesos+" "+lapsedTimeLocale.segments[INDEX_MONTHS]; }
                    else if (dies==1) { text = text+dies+" "+lapsedTimeLocale.segments[INDEX_DAY]; }
                    else if (dies>0) { text = text+dies+" "+lapsedTimeLocale.segments[INDEX_DAYS]; }
                    else if (hores==1) { text = text+hores+" "+lapsedTimeLocale.segments[INDEX_HOUR]; }
                    else if (hores>0) { text = text+hores+" "+lapsedTimeLocale.segments[INDEX_HOURS]; }
                    else if (minuts==1) { text = text+minuts+" "+lapsedTimeLocale.segments[INDEX_MINUTE]; }
                    else if (minuts>0) { text = text+minuts+" "+lapsedTimeLocale.segments[INDEX_MINUTES]; }
                    else if (segons==1) { text = text+segons+" "+lapsedTimeLocale.segments[INDEX_SECOND]; }
                    else if (segons>0) { text = text+segons+" "+lapsedTimeLocale.segments[INDEX_SECOND]; }

                    text = text + lapsedTimeLocale.segments[INDEX_AGO];
                }
                else { text = NumberToString.toString (valor, format); }
            }
        }
        else if (valor!=null) { text = NumberToString.toString (valor, format); }

        return text;
    }

}
