//    _____ _                       _  _  __   _               _            _______    _____ _        _            _  _
//   / ____) |                     | || ||  \ | |             | |          (__   __)  / ____) |      (_)          | || |
//  | |    | |  _ _ ___ ___  ___   |_||_||   \| |_   _ _ _ __ | |_    ___ _ __| | ___| (___ | |_ _ __ _ _ __   __ __||_|
//  | |    | |/ _` | __) __)/ _ \        | |\ ` | | | | ` `  \| `_ \ / _ \ `__) |/ _ \\___ \| __) `__) | `_  \/ _` |
//  | |____| | (_| |__ \__ \  __/        | | \  | |_| | |\ |\ | |_) |  __/ |  | | (_) |___) | |_| |  | | | | | (_| |
//   \_____)_|\__,_|___/___/\___|        |_|  \_|\__,_|_||_||_|_,__/ \___|_|  |_|\___/_____/ \__)_|  |_|_| |_|\__, |
//                                                                                                             __| |
//                                                                                                            (___/


package cat.lib.utils;

import java.math.RoundingMode;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

/**
 *
 * @author Miquel Abadal i Cervià
 */
public class NumberToString {

    private static String[] mesos = {"de gener", "de febrer", "de març", "d'abril", "de maig", "de juny", "de juliol", "d'agost", "de setembre", "d'octubre", "de novembre", "de desembre"};


    //   __  __      _            _                     _     _ _
    //  |  \/  |    | |          | |                   | |   | |_)
    //  | \  / | ___| |_ ___   __| | ___    _ __  _   _| |_  | |_  ____ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \  | `_ \| | | | `_ \| | |/ ___) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/  | |_) | |_| | |_) | | | (___\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|  |  __/ \__,_|_,__/|_|_|\____)___/
    //                                     | |
    //                                     |_|

    public static String toString (boolean valor, String trueValue, String falseValue) {
        if (valor) { return trueValue; }
        return falseValue;
    }

    public static String toStringSN (boolean valor) {
        return toString (valor, "S", "N");
    }

    public static String toStringTF (boolean valor) {
        return toString (valor, "T", "F");
    }

    public static String toStringTrueFalse (boolean valor) {
        return toString (valor, "true", "false");
    }

    public static String toString01 (boolean valor) {
        return toString (valor, "1", "0");
    }


    public static String toString (byte valor, int len) {
        return StringUtils.fillLeftZeros (Byte.toString (valor), len);
    }

    public static String toString (short valor, int len) {
        return StringUtils.fillLeftZeros (Short.toString (valor), len);
    }

    public static String toString (int valor, int len) {
        return StringUtils.fillLeftZeros (Integer.toString (valor), len);
    }

    public static String toString (long valor, int len) {
        return StringUtils.fillLeftZeros (Long.toString (valor), len);
    }

    public static String toString (byte valor) { return Byte.toString (valor);}
    public static String toString (short valor) { return  Short.toString (valor); }
    public static String toString (int valor) { return  Integer.toString (valor); }
    public static String toString (long valor) { return Long.toString (valor); }
    public static String toString (float valor) { return  Float.toString (valor); }
    public static String toString (double valor) { return Double.toString (valor); }
    public static String toString (char valor) { return Character.toString (valor); }

    public static String toString (Date valor, String format) {

        String res = null;
        
        if (valor!=null) {
            SimpleDateFormat dateFormat = new SimpleDateFormat (format);
            res = dateFormat.format (valor);
        }
        return StringUtils.notNull (res);
    }

    public static String toString (Date valor) { return toString (valor, "dd-MMM-yyyy"); }

    public static String toDateString (Date valor) { return toString (valor, "dd-MMM-yyyy"); }
    public static String toDateTimeString (Date valor) { return toString (valor, "dd-MMM-yyyy H-mm-ss"); }

    public static String toStringCatala (Date valor) {

        String res = null;

        if (valor!=null) {
            int dia = valor.getDate();
            int mes = valor.getMonth();
            int any = valor.getYear()+1900;
            res = (dia)+" "+mesos[mes]+" de "+any;
        }
        return StringUtils.notNull (res);
    }


    //   ______                        _       _             _
    //  |  ____)                      | |     (_)           | |
    //  | |___ ___  _ __ _ _ __    _ _| |_ ___ __   __    __| | ___    _ __  _   _ _ _ __   ___ _ __ ___  ___
    //  |  ___) _ \| `__) ` `  \ / _` | __) _ \ | \/ /   / _` |/ _ \  | `_  \ | | | ` `  \ / _ \ `__) _ \/ __)
    //  | |  | (_) | |  | |\ |\ | (_| | |_  __/ |)  (   | (_| |  __/  | | | | |_| | |\ |\ |  __/ | | (_) |__ \
    //  |_|   \___/|_|  |_||_||_|\__,_|\__)___|_|_/\_\   \__,_|\___|  |_| |_|\__,_|_||_||_|\___|_|  \___/(___/
    //
    //

    private static String toNumber (double valor, int numMinDecimals, int numMaxDecimals) {
        NumberFormat n = NumberFormat.getInstance();
        n.setMaximumFractionDigits (numMaxDecimals);
        n.setMinimumFractionDigits (numMinDecimals);

        // NO ÉS COMPETIBLE AMB LA VERSIO ANDROID DE LA CARME
        //n.setRoundingMode (RoundingMode.HALF_UP);
        
        String res = n.format (valor);

        // El valor retornat, utilitza una comma enlloc d'un punt decimal
        //return StringUtils.substChar (res, ',', '.', StringUtils.SUBST_ONE, StringUtils.FROM_BEGIN, StringUtils.CASE_SENSITIVE);
        return res;
    }

    public static String toNumber (double valor, int numDecimals) {
        return toNumber (valor, numDecimals, numDecimals);
    }

    public static String toNumber (double valor) {
        return toNumber (valor, 0, 8);
    }

    public static String toUnit (double valor, int minDecimals, int maxDecimals, String unitName) { return StringUtils.concat (toNumber (valor, minDecimals, maxDecimals)," ",unitName); }
    public static String toUnit (double valor, int numDecimals, String unitName) { return toUnit (valor, 0, numDecimals, unitName); }
    public static String toUnit (double valor, String unitName) { return toUnit (valor, 0, unitName); }

    public static String toEUR (double valor) { return toUnit (valor, 2, 2, "EUR"); }
    public static String toEuro (double valor) { return toUnit (valor, 2, 2, "€"); }
    public static String toDollar (double valor) { return toUnit (valor, 2, "$"); }
    public static String toPta (double valor) { return toUnit (valor, 2, "Pta"); }
    
    public static String toPercent (double valor, int numDecimals) { return toUnit (valor, numDecimals, "%"); }
    public static String toPercent (double valor) { return toPercent (valor, 0); }

}
