//    _____ _                       _  _   _____ _                          _    _ _   _ _    _  _
//   / ____) |                     | || | / ____) |                        | |  | | | (_) |  | || |
//  | |    | |  _ _ ___ ___  ___   |_||_|| (___ | |_ _ __ ___   _ _ _ _ __ | |  | | |_ _| |____||_|
//  | |    | |/ _` | __) __)/ _ \         \___ \| __) `__) _ \/ _` | ` `  \| |  | | __) | | __)
//  | |____| | (_| |__ \__ \  __/         ____) | |_| | |  __/ (_| | |\ |\ | |__| | |_| | |__ \
//   \_____)_|\__,_|___/___/\___|        (_____/ \__)_|  \___|\__,_|_||_||_|\____/ \__)_|_|___/
//
//


package cat.lib.utils;

import cat.lib.errors.ErrorEx;
import cat.lib.files.FileUtils;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Reader;
import java.io.UnsupportedEncodingException;

/**
 *
 * @author Miquel Abadal i Cervià
 */
public class StreamUtils {

    public static final String DEFAULT_ENCODING ="utf-8";

    public static Reader inputStreamToReader (InputStream in, String encoding, ErrorEx error) {

        InputStreamReader res = null;
        if (in!=null) {

            if (StringUtils.isEmpty(encoding)) { encoding = DEFAULT_ENCODING; }

            try { res = new InputStreamReader (in, encoding); }

            catch (UnsupportedEncodingException ex) { error.setError ("IOEXCEPTION", "ACTION:ENCODE_INPUT_STEAM", "TRACE:StreamUtils.encodeInputStream", "DESCRIPTION: No se ha podido identificar la codificación", "CODIFICACION:"+encoding, "MESSAGE:"+ex.getMessage());  }
            catch (IOException ex) { error.setError ("IOEXCEPTION", "ACTION:ENCODE_INPUT_STEAM", "TRACE:StreamUtils.encodeInputStream", "DESCRIPTION: Se ha producido un error de lectura", "MESSAGE:"+ex.getMessage());  }
        }
        return res;
    }

    /**
     * Converteix un stream de text d'entrada a un String.
     *
     * Si s'indica que utilitzi la codificació "Unicode" busca els codis especials
     * i els converteix a caràcters.
     *
     * El format "Unicode" inclou els caràcters especials utilitzant, enlloc del
     * seu valor "word" la codificació: "\u0000" on "0000" representa el codi
     * del caràcter en hexadecimal.
     *
     * @param in
     * @return
     */
    public static String inputStreamToString (InputStream in, String encode, ErrorEx error) {

        StringBuilder res = new StringBuilder ();

        if (in!=null) {

            InputStreamReader data = (InputStreamReader) inputStreamToReader (in, encode, error);

            int bufferSize = 8192;
            char[] buffer = new char [bufferSize];
            char caract = '\0';

            try {
                int numRead = 0;
                while (numRead!=-1) {
                    numRead = data.read (buffer);

                    for (int cont1=0; cont1<numRead; cont1++) {
                        caract = buffer[cont1];
                        res.append (caract);
                    }
                }

            }
            catch (IOException ex) { error.setError ("IOEXCEPTION", "ACTION:INPUT_STREAM_TO_STRING", "TRACE:StreamUtils.inputStreamToString()", "DESCRIPTION: No se ha podido leer los datos", "MESSAGE:"+ex.getMessage());  }

            closeInputStream (in, error);
        }

        return res.toString();
    }


    /*
     * Envia la informació rebuda pel InputStream cap al OutputStream
     */
    public static void sendInputStreamToOutputStream (InputStream in, OutputStream out) {

        if (in!=null && out!=null) {

            // Envia el buffer del InputStream a la sortida del OutputStream.
            byte[] buffer = new byte[1024];
            try {
                int numRead = in.read(buffer);
                while (numRead>0) {
                    out.write (buffer, 0, numRead);
                    numRead = in.read (buffer);
                }
                out.close();
            } catch (IOException ex) {}

        }
    }

    public static InputStream stringToInputStream (String text, String encoding) {

        InputStream in = null;

        try { in = new ByteArrayInputStream (text.getBytes (encoding)); }
        catch (UnsupportedEncodingException e) { }

        return in;
    }

    public static InputStream stringToInputStream (String text) { return stringToInputStream (text, DEFAULT_ENCODING); }

    public static void closeInputStream (InputStream in, ErrorEx error) {

        try { 
            if (in!=null) { in.close(); }
        }
        catch (IOException ex) { error.setError ("IOEXCEPTION", "ACTION:CLOSE_INPUT_STREAM", "TRACE:StreamUtils.closeInputStream()", "DESCRIPTION: No se ha podido cerrar la fuente de datos", "MESSAGE:"+ex.getMessage());  }
    }

    public static void inputStreamToFile (InputStream in, String fileName, ErrorEx error) {
        FileUtils.inputStreamToFile (in, fileName, error);
    }

}
