//    _____ _                       _  _   _____ _        _          _______    __   _               _              _  _
//   / ____) |                     | || | / ____) |      (_)        (__   __)  |  \ | |             | |            | || |
//  | |    | |  _ _ ___ ___  ___   |_||_|| (___ | |_ _ __ _ _ __   __ _| | ___ |   \| |_   _ _ _ __ | |_    ___ _ ___||_|
//  | |    | |/ _` | __) __)/ _ \         \___ \| __) `__) | `_  \/ _` | |/ _ \| |\ ` | | | | ` `  \| `_ \ / _ \ `__)
//  | |____| | (_| |__ \__ \  __/         ____) | |_| |  | | | | | (_| | | (_) | | \  | |_| | |\ |\ | |_) |  __/ |
//   \_____)_|\__,_|___/___/\___|        (_____/ \__)_|  |_|_| |_|\__, |_|\___/|_|  \_|\__,_|_||_||_|_,__/ \___|_|
//                                                                 __| |
//                                                                (___/


package cat.lib.utils;

import cat.lib.errors.ErrorEx;
import cat.lib.math.Roman;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 *
 * @author Miquel Abadal i Cervià
 */
public class StringToNumber {



    //     __   _        _ _           _
    //   / __ \| |      (_) |         | |
    //  | |__| | |_ _ __ _| |_   _   _| |_ ___
    //  |  __  | __) `__) | `_ \| | | | __) __)
    //  | |  | | |_| |  | | |_) | |_| | |_\__ \
    //  |_|  |_|\__)_|  |_|_,__/ \__,_|\__)___/
    //
    //

    private static final String[] trueValues = new String[] {"true", "t", "1", "s", "si", "y", "yes", "ok", "on", "enabled"};
    private static final String[] falseValues = new String[] {"false", "f", "0", "n", "no", "off", "disabled"};


    //   __  __      _            _             _  _   _____ ______ _______ _  _
    //  |  \/  |    | |          | |           | || | / ____)  ____)__   __) || |
    //  | \  / | ___| |_ ___   __| | ___ ___   |_||_|| |  __| |__     | |  |_||_|
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)        | | (_ |  __)    | |
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \        | |__| | |____   | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/         \_____|______)  |_|
    //
    //


    //    _____                                _              _ _   _____ _        _
    //   / ____)                              (_)            | | | / ____) |      (_)
    //  | |     ___  _ __  _    _ ___ _ __ ___ _  ___      __| |_|| (___ | |_ _ __ _ _ __   __ _
    //  | |    / _ \| `_  \ |  | | _ \ `__) __) |/ _ \    / _` |   \___ \| __) `__) | `_  \/ _` |
    //  | |____ (_) | | | |\ \/ /  __/ |  \__ \ | (_) |  | (_| |   ____) | |_| |  | | | | | (_| |
    //   \_____)___/|_| |_| \__/ \___|_|  (___/_|\___/    \__,_|  (_____/ \__)_|  |_|_| |_|\__, |
    //                                                                                      __| |
    //                                                                                     (___/
    //            _   _                    _          _
    //           | | (_)                  (_)        | |
    //     _ _   | |_ _ _ __  _   _ ___    _     ____| |  _ _ ___ ___  ___ ___
    //   / _` |  | __) | `_ \| | | | __)  | |   / ___) |/ _` | __) __)/ _ \ __)
    //  | (_| |  | |_| | |_) | |_| |__ \  | |  | (___| | (_| |__ \__ \  __/__ \
    //   \__,_|   \__)_|  __/ \__,_|___/  |_|   \____)_|\__,_|___/___/\___|___/
    //                 | |
    //                 |_|


    /**
     * Converteix el valor de la cadena de <b>text</b> al seu corresponent valor
     * <b>boolean</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Els valors reconeguts com a vàlids són els següents: "true", "t", "1",
     * "s", "si", "y", "yes", "ok", "false", "f", "0", "n" i "no".
     *
     * <p>- Si la cadena no conté un valor vàlid, crea una instància d'error amb
     * la corresponent descripció. (veure {@link StringEx#getError})
     *
     * @return Retorna el valor de la cadena de text convertit al seu corresponent
     * valor <b>boolean</b> o <b>false</b> en cas de contingui una cadena no vàlida.
     */
    public static boolean booleanValue (String text, boolean errorValue, ErrorEx error) {
        
        boolean res = errorValue;
        boolean ok = false;
        boolean inverse = false;

        if (text!=null) {
            text = text.trim();
            if (!StringUtils.isEmpty(text) && text.charAt(0)=='!') {
                inverse = true;
                text = StringUtils.copy (text, 1, -1).trim();
            }

            if (SetUtils.stringIn (text, StringUtils.CASE_INSENSITIVE, trueValues)) {
                res = true;
                ok = true;
            }
            else if (SetUtils.stringIn (text, StringUtils.CASE_INSENSITIVE, falseValues)) {
                res = false;
                ok = true;
            }
        }

        if (ok && inverse) { res = !res; }
        if (!ok && error!=null) { error.setError ("CAST_ERROR", "DESCRIPTION: El valor no es de tipo 'boolean'", "VALUE: "+text); }
        
        return res;
    }

    public static boolean booleanValue (String text, boolean errorValue) { return booleanValue (text, errorValue, null); }
    public static boolean booleanValue (String text) { return booleanValue (text, false, null); }



    /**
     * Converteix el valor de la cadena de <b>text</b> al seu corresponent valor
     * <b>byte</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena no conté un valor vàlid, crea una instància d'error amb
     * la corresponent descripció. (veure {@link StringEx#getError})
     *
     * @return Retorna el valor de la cadena de text convertit al seu corresponent
     * valor <b>byte</b> o <b>0</b> en cas de contingui una cadena no vàlida.
     */
    public static byte byteValue (String text, int base, byte errorValue, ErrorEx error) {

        boolean ok = false;
        byte res = errorValue;
        if (!StringUtils.isEmpty(text)) {
            try { res = Byte.parseByte (text.trim(), base); ok = true; }
            catch (NumberFormatException e) { }
        }
        if (!ok && error!=null) { error.setError ("CAST_ERROR", "DESCRIPTION: El valor no es de tipo 'byte'", "VALUE: "+text); }

        return res;
    }

    public static byte byteValue (String text, byte errorValue, ErrorEx error) { return byteValue (text, 10, errorValue, error); }
    public static byte byteValue (String text, byte errorValue) { return byteValue (text, 10, errorValue, null); }
    public static byte byteValue (String text) { return byteValue (text, 10, (byte) 0, null); }


    /**
     * Converteix el valor de la cadena de <b>text</b> al seu corresponent valor
     * <b>short</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena no conté un valor vàlid, crea una instància d'error amb
     * la corresponent descripció. (veure {@link StringEx#getError})
     *
     * @return Retorna el valor de la cadena de text convertit al seu corresponent
     * valor <b>short</b> o <b>0</b> en cas de contingui una cadena no vàlida.
     */
    public static short shortValue (String text, int base, short errorValue, ErrorEx error) {

        boolean ok = false;
        short res = errorValue;
        if (!StringUtils.isEmpty(text)) {
            try { res = Short.parseShort (text.trim(), base); ok = true; }
            catch (NumberFormatException e) { }
        }
        if (!ok && error!=null) { error.setError ("CAST_ERROR", "DESCRIPTION: El valor no es de tipo 'short'", "VALUE: "+text); }

        return res;
    }

    public static short shortValue (String text, short errorValue, ErrorEx error) { return shortValue (text, 10, errorValue, error); }
    public static short shortValue (String text, short errorValue) { return shortValue (text, 10, errorValue, null); }
    public static short shortValue (String text) { return shortValue (text, 10, (short) 0, null); }

    /**
     * Converteix el valor de la cadena de <b>text</b> al seu corresponent valor
     * <b>int</b>, utilitzant
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena no conté un valor vàlid, crea una instància d'error amb
     * la corresponent descripció. (veure {@link StringEx#getError})
     *
     * @return Retorna el valor de la cadena de text convertit al seu corresponent
     * valor <b>int</b> o <b>0</b> en cas de contingui una cadena no vàlida.
     */
    public static int intValue (String text, int base, int errorValue, ErrorEx error) {

        boolean ok = false;
        int res = errorValue;
        if (!StringUtils.isEmpty(text)) {
            try { res = Integer.parseInt (text.trim(), base); ok = true; }
            catch (NumberFormatException e) { }
        }
        if (!ok && error!=null) { error.setError ("CAST_ERROR", "DESCRIPTION: El valor no es de tipo 'int'", "VALUE: "+text); }

        return res;
    }

    public static int intValue (String text, int errorValue, ErrorEx error) { return intValue (text, 10, errorValue, error); }
    public static int intValue (String text, int errorValue) { return intValue (text, 10, errorValue, null); }
    public static int intValue (String text) { return intValue (text, 10, 0, null); }


    /**
     * Converteix el valor de la cadena de <b>text</b> al seu corresponent valor
     * <b>long</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena no conté un valor vàlid, crea una instància d'error amb
     * la corresponent descripció. (veure {@link StringEx#getError})
     *
     * @return Retorna el valor de la cadena de text convertit al seu corresponent
     * valor <b>long</b> o <b>0</b> en cas de contingui una cadena no vàlida.
     */
    public static long longValue (String text, int base, long errorValue, ErrorEx error) {

        boolean ok = false;
        long res = errorValue;
        if (!StringUtils.isEmpty(text)) {
            try { res = Long.parseLong (text.trim(), base); ok = true; }
            catch (NumberFormatException e) { }
        }
        if (!ok && error!=null) { error.setError ("CAST_ERROR", "DESCRIPTION: El valor no es de tipo 'long'", "VALUE: "+text); }

        return res;
    }

    public static long longValue (String text, long errorValue, ErrorEx error) { return longValue (text, 10, errorValue, error); }
    public static long longValue (String text, long errorValue) { return longValue (text, 10, errorValue, null); }
    public static long longValue (String text) { return longValue (text, 10, 0, null); }


    /**
     * Converteix el valor de la cadena de <b>text</b> al seu corresponent valor
     * <b>float</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena no conté un valor vàlid, crea una instància d'error amb
     * la corresponent descripció. (veure {@link StringEx#getError})
     *
     * @return Retorna el valor de la cadena de text convertit al seu corresponent
     * valor <b>float</b> o <b>0</b> en cas de contingui una cadena no vàlida.
     */
    public static float floatValue (String text, float errorValue, ErrorEx error) {

        boolean ok = false;
        float res = errorValue;
        if (!StringUtils.isEmpty(text)) {
            try { res = Float.parseFloat (text.trim()); ok = true; }
            catch (NumberFormatException e) { }
        }
        if (!ok && error!=null) { error.setError ("CAST_ERROR", "DESCRIPTION: El valor no es de tipo 'float'", "VALUE: "+text); }

        return res;
    }

    public static float floatValue (String text, float errorValue) { return floatValue (text, errorValue, null); }
    public static float floatValue (String text) { return floatValue (text, 0, null); }
    public static float floatValue (String text, float errorValue, char thowsandChar, char decimalChar) {
        text = StringUtils.substChar(text, thowsandChar, '\0');
        text = StringUtils.substChar(text, decimalChar, ',');
        return floatValue (text, errorValue, null);
    }

    /**
     * Converteix el valor de la cadena de <b>text</b> al seu corresponent valor
     * <b>double</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena no conté un valor vàlid, crea una instància d'error amb
     * la corresponent descripció. (veure {@link StringEx#getError})
     *
     * @return Retorna el valor de la cadena de text convertit al seu corresponent
     * valor <b>double</b> o <b>0</b> en cas de contingui una cadena no vàlida.
     */
    public static double doubleValue (String text, double errorValue, ErrorEx error) {

        boolean ok = false;
        double res = errorValue;
        if (!StringUtils.isEmpty(text)) {
            try { res = Double.parseDouble (text.trim()); ok = true; }
            catch (NumberFormatException e) { }
        }
        if (!ok && error!=null) { error.setError ("CAST_ERROR", "DESCRIPTION: El valor no es de tipo 'double'", "VALUE: "+text); }

        return res;
    }

    public static double doubleValue (String text, double errorValue) { return doubleValue (text, errorValue, null); }
    public static double doubleValue (String text) { return doubleValue (text, 0, null); }
    public static double doubleValue (String text, double errorValue, char thowsandChar, char decimalChar) {

        text = StringUtils.substChar(text, thowsandChar, '\0');
        text = StringUtils.substChar(text, decimalChar, '.');
        return doubleValue (text, errorValue, null);
    }

    /**
     * Converteix el valor de la cadena de <b>text</b> al seu corresponent valor
     * <b>Roman</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena no conté un valor vàlid, crea una instància d'error amb
     * la corresponent descripció. (veure {@link StringEx#getError})
     *
     * @return Retorna el valor de la cadena de text convertit al seu corresponent
     * valor <b>Roman</b> o <b>0</b> en cas de contingui una cadena no vàlida.
     */
    public static String romanValue (String text, String errorValue, ErrorEx error) {

        boolean ok = false;
        String res = errorValue;
        if (!StringUtils.isEmpty(text)) {
            try { res = Roman.parseRoman (text.trim()); ok = true; }
            catch (NumberFormatException e) { }
        }
        if (!ok && error!=null) { error.setError ("CAST_ERROR", "DESCRIPTION: El valor no es de tipo 'Roman'", "VALUE: "+text); }

        return res;
    }

    public static String romanValue (String text, String errorValue) { return romanValue (text, errorValue, null); }
    public static String romanValue (String text) { return romanValue (text, "", null); }


    public static Date dateValue (String text, String format, ErrorEx error) {

        boolean ok = false;
        SimpleDateFormat dateFormat = new SimpleDateFormat (format);
        Date res = null;
        if (!StringUtils.isEmpty(text)) {
            try { res = dateFormat.parse (text.trim()); ok = true; }
            catch (ParseException e) { }
        }
        if (!ok && error!=null) { error.setError ("CAST_ERROR", "DESCRIPTION: El valor no es de tipo 'Date'", "VALUE: "+text); }

        return res;
    }

    public static Date dateValue (String text, String format) { return dateValue (text, format, null); }
    public static Date dateValue (String text) { return dateValue (text, "dd-MMM-yyyy", null); }

    /**
     * Comproba si la cadena de <b>text</b> conté un valor <b>boolean</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Els valors reconeguts com a vàlids són els següents: "true", "t", "1",
     * "s", "si", "y", "yes", "ok", "false", "f", "0", "n" i "no".
     *
     * @return Retorna <b>true</b> si la cadena de text conté un valor <b>boolean</b>
     * vàlid; o <b>false</b> en cas contrari.
     */
    public static boolean isBoolean (String text) {

        boolean res = false;
        if (!StringUtils.isEmpty(text)) {
            text = text.trim();
            if (text.charAt(0)=='!') { text = StringUtils.copy (text, 1, -1).trim(); }
            if (SetUtils.stringIn (text, StringUtils.CASE_INSENSITIVE, trueValues)) { res = true; }
            else if (SetUtils.stringIn (text, StringUtils.CASE_INSENSITIVE, falseValues)) { res = true; }
        }
        return res;
    }

    public static boolean isStrictBoolean (String text) {

        boolean res = false;
        if (!StringUtils.isEmpty(text)) {
            text = text.trim();
            if (text.equalsIgnoreCase("false") || text.equalsIgnoreCase("true")) { res = true; }
        }
        return res;
    }

    /**
     * Comproba si la cadena de <b>text</b> conté un valor <b>byte</b>.
     *
     * @return Retorna <b>true</b> si la cadena de text conté un valor <b>byte</b>
     * vàlid; o <b>false</b> en cas contrari.
     */
    public static boolean isByte (String text) {
        boolean res = false;
        if (!StringUtils.isEmpty(text)) {
            try {
                Byte.parseByte (text.trim());
                res = true;
            } catch (NumberFormatException e) {}
        }
        return res;
    }

    /**
     * Comproba si la cadena de <b>text</b> conté un valor <b>short</b>.
     *
     * @return Retorna <b>true</b> si la cadena de text conté un valor <b>short</b>
     * vàlid; o <b>false</b> en cas contrari.
     */
    public static boolean isShort (String text) {
        boolean res = false;
        if (!StringUtils.isEmpty(text)) {
            try {
                Short.parseShort (text.trim());
                res = true;
            } catch (NumberFormatException e) {}
        }
        return res;
    }

    /**
     * Comproba si la cadena de <b>text</b> conté un valor <b>int</b>.
     *
     * @return Retorna <b>true</b> si la cadena de text conté un valor <b>int</b>
     * vàlid; o <b>false</b> en cas contrari.
     */
    public static boolean isInt (String text) {
        boolean res = false;
        if (!StringUtils.isEmpty(text)) {
            try {
                Integer.parseInt (text.trim());
                res = true;
            } catch (NumberFormatException e) {}
        }
        return res;
    }

    /**
     * Comproba si la cadena de <b>text</b> conté un valor <b>long</b>.
     *
     * @return Retorna <b>true</b> si la cadena de text conté un valor <b>long</b>
     * vàlid; o <b>false</b> en cas contrari.
     */
    public static boolean isLong (String text) {
        boolean res = false;
        if (!StringUtils.isEmpty(text)) {
            try {
                Long.parseLong (text.trim());
                res = true;
            } catch (NumberFormatException e) {}
        }
        return res;
    }

    /**
     * Comproba si la cadena de <b>text</b> conté un valor <b>float</b>.
     *
     * @return Retorna <b>true</b> si la cadena de text conté un valor <b>float</b>
     * vàlid; o <b>false</b> en cas contrari.
     */
    public static boolean isFloat (String text) {
        boolean res = false;
        if (!StringUtils.isEmpty(text)) {
            try {
                Float.parseFloat (text.trim());
                res = true;
            } catch (NumberFormatException e) {}
        }
        return res;
    }

    /**
     * Comproba si la cadena de <b>text</b> conté un valor <b>double</b>.
     *
     * @return Retorna <b>true</b> si la cadena de text conté un valor <b>double</b>
     * vàlid; o <b>false</b> en cas contrari.
     */
    public static boolean isDouble (String text) {
        boolean res = false;
        if (!StringUtils.isEmpty(text)) {
            try {
                Double.parseDouble (text.trim());
                res = true;
            } catch (NumberFormatException e) {}
        }
        return res;
    }

    /**
     * Comproba si la cadena de <b>text</b> conté un valor <b>Roman</b>.
     *
     * @return Retorna <b>true</b> si la cadena de text conté un valor <b>Roman</b>
     * vàlid; o <b>false</b> en cas contrari.
     */
    public static boolean isRoman (String text) {
        boolean res = false;
        if (!StringUtils.isEmpty(text)) {
            try {
                Roman.parseRoman (text.trim());
                res = true;
            } catch (NumberFormatException e) {}
        }
        return res;
    }
    
    /**
     * Comproba si la cadena de <b>text</b> conté un valor numéric.
     *
     * @param text
     * @return Retorna <b>true</b> si la cadena de text conté un valor numéric
     * vàlid; o <b>false</b> en cas contrari.
     */
    public static boolean isNumber (String text) {
        char[] numbers = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'};
        
        if (StringUtils.isNotEmpty(text)) { 

            boolean res = true;
            int cont1=0;
            while (res && cont1<text.length()) {
                res = SetUtils.charInArray (text.charAt(cont1), StringUtils.CASE_SENSITIVE, numbers);
                cont1++;
            }
            return res;
        }
        else { return false; }
    }

}
