//    _____ _                       _  _   _____ _        _             _    _ _   _ _    _  _
//   / ____) |                     | || | / ____) |      (_)           | |  | | | (_) |  | || |
//  | |    | |  _ _ ___ ___  ___   |_||_|| (___ | |_ _ __ _ _ __   __ _| |  | | |_ _| |____||_|
//  | |    | |/ _` | __) __)/ _ \         \___ \| __) `__) | `_  \/ _` | |  | | __) | | __)
//  | |____| | (_| |__ \__ \  __/         ____) | |_| |  | | | | | (_| | |__| | |_| | |__ \
//   \_____)_|\__,_|___/___/\___|        (_____/ \__)_|  |_|_| |_|\__, |\____/ \__)_|_|___/
//                                                                 __| |
//                                                                (___/


package cat.lib.utils;

import cat.lib.errors.ErrorEx;
import cat.lib.locale.LocaleUtils;
import cat.lib.math.MathEx;
import cat.lib.math.RandomEx;
import java.util.Locale;


/**
 *   _____ _                       _  _   _____ _        _             _    _ _   _ _    _  _
 *  / ____) |                     | || | / ____) |      (_)           | |  | | | (_) |  | || |
 * | |    | |  _ _ ___ ___  ___   |_||_|| (___ | |_ _ __ _ _ __   __ _| |  | | |_ _| |____||_|
 * | |    | |/ _` | __) __)/ _ \         \___ \| __) `__) | `_  \/ _` | |  | | __) | | __)
 * | |____| | (_| |__ \__ \  __/         ____) | |_| |  | | | | | (_| | |__| | |_| | |__ \
 *  \_____)_|\__,_|___/___/\___|        (_____/ \__)_|  |_|_| |_|\__, |\____/ \__)_|_|___/
 *                                                                __| |
 *                                                               (___/
 *
 * @author Miquel Abadal i Cervià
 */
public class StringUtils  {

    //    _____                _              _
    //   / ____)              | |            | |
    //  | |     ___  _ __  ___| |_  _ _ _ __ | |_ ___
    //  | |    / _ \| `_  \ __) __) _` | `_  \ __) __)
    //  | |____ (_) | | | |__ \ |_ (_| | | | | |_\__ \
    //   \_____)___/|_| |_|___/\__)__,_|_| |_|\__)___/
    //
    //

    public final static byte LEFT_FILL = 0;
    public final static byte RIGHT_FILL  = 1;

    public final static byte SUBST_ONE = 0;
    public final static byte SUBST_ALL  = 1;

    public final static byte TO_UPPER_CASE = 0;
    public final static byte TO_LOWER_CASE  = 1;

    public final static boolean CASE_INSENSITIVE = false;
    public final static boolean CASE_SENSITIVE = true;

    public final static int FROM_BEGIN = 0;


    //     __   _        _ _           _
    //   / __ \| |      (_) |         | |
    //  | |__| | |_ _ __ _| |_   _   _| |_ ___
    //  |  __  | __) `__) | `_ \| | | | __) __)
    //  | |  | | |_| |  | | |_) | |_| | |_\__ \
    //  |_|  |_|\__)_|  |_|_,__/ \__,_|\__)___/
    //
    //

    protected static char[] withoutTildeTABLE = null;


    
    //   __  __      _            _                        _             _
    //  |  \/  |    | |          | |                      (_)           | |
    //  | \  / | ___| |_ ___   __| | ___ ___    _ __  _ __ _ _    _  _ _| |_ ___
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)  | `_ \| `__) | |  | | _` | __) __)
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \  | |_) | |  | |\ \/ / (_| | |_\__ \
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/  |  __/|_|  |_| \__/ \__,_|\__)___/
    //                                         | |
    //                                         |_|

    public static char lowerCase (char caract, boolean caseSensitive) {

        if (!caseSensitive) { caract = Character.toLowerCase (caract); }
        return caract;
    }

    public static String lowerCase (String text, boolean caseSensitive) {

        text = notNull (text);
        if (!caseSensitive) { text = text.toLowerCase(); }
        return text;
    }

    protected static char senseAccents (char caract) {
        if (caract<withoutTildeTABLE.length) { caract = withoutTildeTABLE[caract]; }
        return caract;
    }

    /**
     * Inicialitza la taula de codificació per a obtenir cadenes sense accents.
     *
     * @return  Retorna la taula de codificació.
     */
    protected static char[] initWithoutTilde () {

        char[] tildeTable = new char[256];
        char caract;

        for (int cont1=0; cont1<256; cont1++) {
            caract = (char) cont1;

            if (SetUtils.charIn (caract, true, 'À', 'Á', 'Ä', 'Â')) caract = 'A';
            else if (SetUtils.charIn (caract, true, 'È', 'É', 'Ë', 'Ê')) caract = 'E';
            else if (SetUtils.charIn (caract, true, 'Ì', 'Í', 'Ï', 'Î')) caract = 'I';
            else if (SetUtils.charIn (caract, true, 'Ò', 'Ó', 'Ö', 'Ô')) caract = 'O';
            else if (SetUtils.charIn (caract, true, 'Ù', 'Ú', 'Ü', 'Û')) caract = 'U';
            else if (SetUtils.charIn (caract, true, 'à', 'á', 'ä', 'â')) caract = 'a';
            else if (SetUtils.charIn (caract, true, 'è', 'é', 'ë', 'ê')) caract = 'e';
            else if (SetUtils.charIn (caract, true, 'ì', 'í', 'ï', 'î')) caract = 'i';
            else if (SetUtils.charIn (caract, true, 'ò', 'ó', 'ö', 'ô')) caract = 'o';
            else if (SetUtils.charIn (caract, true, 'ù', 'ú', 'ü', 'û')) caract = 'u';
            else if (SetUtils.charIn (caract, true, 'Ç')) caract = 'C';
            else if (SetUtils.charIn (caract, true, 'ç')) caract = 'c';
            else if (SetUtils.charIn (caract, true, 'Ñ')) caract = 'N';
            else if (SetUtils.charIn (caract, true, 'ñ')) caract = 'n';

            tildeTable[cont1]=caract;
        }

       return tildeTable;
    }


    protected static byte validaFillStyle (byte fillStyle) {

        if (fillStyle!=LEFT_FILL && fillStyle!=RIGHT_FILL) { fillStyle = LEFT_FILL; }
        return fillStyle;
    }

    protected static byte validaSubstStyle (byte substStyle) {

        if (substStyle!=SUBST_ONE && substStyle!=SUBST_ALL) { substStyle = SUBST_ALL; }
        return substStyle;
    }

    protected static byte validaCaseStyle (byte caseStyle) {

        if (caseStyle!=TO_UPPER_CASE && caseStyle!=TO_LOWER_CASE) { caseStyle = TO_UPPER_CASE; }
        return caseStyle;
    }

    protected static int validaStartPos (String text, int startPos) {

        if (startPos<0) { startPos = 0; }
        else if (isEmpty (text)) { startPos = 0; }
        else if (startPos>=text.length()) { startPos = text.length(); }
        return startPos;
    }

    protected static int validaSize (int size) {

        if (size<0) { size = 0; }
        return size;
    }

    private static String substSensitiveString (String text, String cercaText, String nouText, byte substStyle, int startPos) {

        startPos = validaStartPos (text, startPos);
        substStyle = validaSubstStyle (substStyle);

        StringBuffer buffer = null;

        if (text!=null) {

            int cercaTextSize = 0;
            if (cercaText!=null) { cercaTextSize = cercaText.length(); }
            nouText = notNull (nouText);

            buffer = new StringBuffer (text.length());
            StringBuffer bufferTemp = new StringBuffer (text.length());

            int cont1 = 0;
            int cont2 = 0;
            boolean completat = false;
            char caract;
            while (cont1<text.length()) {
                caract = text.charAt (cont1);

                if (completat==false && cont1>=startPos && cercaTextSize>0 && caract==cercaText.charAt (cont2)) {
                    bufferTemp.append(caract);

                    cont2++;
                    if (cont2==cercaTextSize) {
                        bufferTemp.setLength (0);
                        buffer.append (nouText);
                        cont2 = 0;
                        if (substStyle==SUBST_ONE) { completat = true; }
                    }
                }
                else {
                    if (bufferTemp.length()>0) {
                        buffer.append (bufferTemp.charAt (0));
                        bufferTemp.setLength (0);
                    }
                    else buffer.append (caract);
                    cont1 = cont1 - cont2;
                    cont2 = 0;
                }
                cont1++;
            }

            if (bufferTemp.length()>0) { buffer.append (bufferTemp); }

        }
        else { buffer = new StringBuffer (""); }

        return buffer.toString();
    }

    private static String substInsensitiveString (String text, String cercaText, String nouText, byte substStyle, int startPos) {

        startPos = validaStartPos (text, startPos);
        substStyle = validaSubstStyle (substStyle);

        StringBuffer buffer = null;

        if (text!=null) {

            int cercaTextSize = 0;
            if (cercaText!=null) { cercaTextSize = cercaText.length(); }
            nouText = notNull (nouText);

            buffer = new StringBuffer (text.length());
            StringBuffer bufferTemp = new StringBuffer (text.length());

            cercaText = toLowerCase (cercaText);

            int cont1 = 0;
            int cont2 = 0;
            boolean completat = false;
            char caract;
            while (cont1<text.length()) {
                caract = text.charAt (cont1);

                if (completat==false && cont1>=startPos && cercaTextSize>0 && toLowerCase (caract)==cercaText.charAt (cont2)) {
                    bufferTemp.append(caract);

                    cont2++;
                    if (cont2==cercaTextSize) {
                        bufferTemp.setLength (0);
                        buffer.append (nouText);
                        cont2 = 0;
                        if (substStyle==SUBST_ONE) { completat = true; }
                    }
                }
                else {
                    if (bufferTemp.length()>0) {
                        buffer.append (bufferTemp.charAt (0));
                        bufferTemp.setLength (0);
                    }
                    else buffer.append (caract);
                    cont1 = cont1 - cont2;
                    cont2 = 0;
                }
                cont1++;
            }

            if (bufferTemp.length()>0) { buffer.append (bufferTemp); }

        }
        else { buffer = new StringBuffer (""); }

        return buffer.toString();
    }


    //   __  __      _            _             _  _   _____ ______ _______ _  _
    //  |  \/  |    | |          | |           | || | / ____)  ____)__   __) || |
    //  | \  / | ___| |_ ___   __| | ___ ___   |_||_|| |  __| |__     | |  |_||_|
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)        | | (_ |  __)    | |
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \        | |__| | |____   | |
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/         \_____|______)  |_|
    //
    //
    

    //   __  __      _            _                 _                                 _
    //  |  \/  |    | |          | |               | |                               (_)
    //  | \  / | ___| |_ ___   __| | ___ ___     __| | ___     ____  _ _ _ __  _    _ _
    //  | |\/| |/ _ \ __) _ \ / _` |/ _ \ __)   / _` |/ _ \   / ___) _` | `_  \ |  | | |
    //  | |  | |  __/ |_ (_) | (_| |  __/__ \  | (_| |  __/  | (___ (_| | | | |\ \/ /| |
    //  |_|  |_|\___|\__)___/ \__,_|\___|___/   \__,_|\___|   \____)__,_|_| |_| \__/ |_|
    //
    //       _         _   _                              ___ _
    //      | |       | | (_)                            / __)_)
    //    __| | ___   | |_ _ _ __   ___   __ _ _ __  _ _| |_  _   _ _
    //   / _` |/ _ \  | __) | `_ \ / _ \ / _` | `__) _` |  _|| |/ _` |
    //  | (_| |  __/  | |_| | |_) | (_) | (_| | | | (_| | |  | | (_| |
    //   \__,_|\___|   \__)_|  __/ \___/ \__, |_|  \__,_|_|  |_|\__,_|
    //                      | |           __| |
    //                      |_|          (___/

    /**
     * Converteix a mínuscules el <b>text</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original conté el valor <b>null</b>, retorna
     * una cadena en blanc.
     *
     * @return Retorna el <b>text</b> passat a minúscules.
     */
    public static String toLowerCase (String text) { return notNull (text).toLowerCase(); }

    /**
     * Converteix a majúscules el <b>text</b>.
     *
     * <p><b>Notes:</b>
     * 
     * <P>- Si la cadena de text original conté el valor <b>null</b>, retorna
     * una cadena en blanc.
     *
     * @return Retorna el <b>text</b> passat a majúscules.
     */
    public static String toUpperCase (String text) { return notNull (text).toUpperCase(); }

    /**
     * Converteix a mínuscules el <b>caracter</b>.
     *
     * @return Retorna el <b>caracter</b> passat a minúscules.
     */
    public static char toLowerCase (char caract) { return Character.toLowerCase (caract); }

    /**
     * Converteix a majúscules el <b>caracter</b>.
     *
     * @return Retorna el <b>caracter</b> passat a majúscules.
     */
    public static char toUpperCase (char caract) { return Character.toUpperCase (caract); }

    /**
     * Converteix el primer caràcter de les paraules del <b>text</b> a
     * majúsculeso o a minúscules.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original conté el valor <b>null</b>, retorna
     * una cadena en blanc.
     *
     * <p>- Només passa a majúscules/minúscules el primer caracter de la primera paraula i
     * el primer caracter de les paraules de més d'un caracter.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b>.
     *
     * <p>- Si el valor assignat al paràmetre <b>caseStyle</b> no és vàlid
     * asumeix el valor <b>TO_UPPER_CASE</b>.
     *
     * @param substStyle <b>SUBST_ONE</b> per passar a majúscules/minúscules només el
     * primer caràcter de la primera paraula; <b>SUBST_ALL</b> per passar a
     * majúscules/minúscules el primer caracter de totes les paraules.
     *
     * @param caseStyle <b>TO_UPPER_CASE</b> passa el primer caràcter a majúscules;
     * <b>TO_LOWER_CASE</b> passa el primer caràcter a minúscules.
     *
     * @return Retorna el <b>text</b> amb el primer caracter de la primera
     * o de totes les paraules passat a majúscules/minúscules.
     */
    private static String firstCharTo (String text, byte substStyle, byte caseStyle) {

        substStyle = validaSubstStyle (substStyle);
        caseStyle = validaCaseStyle (caseStyle);

        StringBuilder res = null;
        
        if (text!=null) {
            res = new StringBuilder (text);


            char lastChar = ' ';
            char caract = '\0';
            boolean completat = false;
            boolean firstWord = true;
            int cont1 = 0;

            while (completat==false && cont1<res.length()) {

                caract = res.charAt (cont1);

                // Si el caracter anterior és un espai en blanc i el caracter actual no...
                // estem al inici d'una paraula.
                if (caract!=' ' && lastChar==' ') {

                    // Si és la primera paraula o
                    // si existeix un caracter posterior i és diferent d'espai en blanc...
                    // realment es tracta d'una paraula de més d'un caràcter.
                    if (firstWord || (cont1+1<res.length() && res.charAt (cont1+1)!=' ')) {

                        if (caseStyle==TO_UPPER_CASE) { caract = Character.toUpperCase (caract); }
                        else { caract = Character.toLowerCase (caract); }

                        res.setCharAt (cont1, caract);

                        // Si només s'ha de convertir la primera paraula a majúscules...
                        // s'ha de completar el procés.
                        if (substStyle==SUBST_ONE) { completat = true; }
                    }

                    firstWord = false;
                }
                lastChar = caract;
                cont1++;
            }
        }
        else { res = new StringBuilder (""); }

        return res.toString();
    }

    /**
     * Converteix el primer caràcter de les paraules del <b>text</b> a
     * majúscules.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original conté el valor <b>null</b>, retorna
     * una cadena en blanc.
     *
     * <p>- Només passa a majúscules el primer caracter de la primera paraula i
     * el primer caracter de les paraules de més d'un caracter.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * @param substStyle <b>SUBST_ONE</b> per passar a majúscules només el
     * primer caràcter de la primera paraula; <b>SUBST_ALL</b> per passar a
     * majúscules el primer caracter de totes les paraules.
     *
     * @return Retorna el <b>text</b> amb el primer caracter de la primera
     * o de totes les paraules passat a majúscules.
     */
    public static String firstCharToUpperCase (String text, byte substStyle) {
        return firstCharTo (StringUtils.toLowerCase (text), substStyle, TO_UPPER_CASE);
    }

    
    /**
     * Converteix el primer caràcter de les paraules del <b>text</b> a
     * minúscules.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original conté el valor <b>null</b>, retorna
     * una cadena en blanc.
     *
     * <p>- Només passa a minúscules el primer caracter de la primera paraula i
     * el primer caracter de les paraules de més d'un caracter.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * @param substStyle <b>SUBST_ONE</b> per passar a majúscules només el
     * primer caràcter de la primera paraula; <b>SUBST_ALL</b> per passar a
     * majúscules el primer caracter de totes les paraules.
     *
     * @return Retorna el <b>text</b> amb el primer caracter de la primera
     * o de totes les paraules passat a majúscules.
     */
    public static String firstCharToLowerCase (String text, byte substStyle) {
        return firstCharTo (StringUtils.toUpperCase (text), substStyle, TO_LOWER_CASE);
    }

    /**
     * Converteix el primer caràcter de la primera paraules del <b>text</b> a
     * majúscules.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original conté el valor <b>null</b>, retorna
     * una cadena en blanc.
     *
     * <p>- Només es passa a majúscules el primer caracter de la primera paraula.     
     *
     * @return Retorna el <b>text</b> amb el primer caracter de la primera
     * paraules passat a majúscules.
     */
    public static String firstCharToUpperCase (String text) {
        return firstCharToUpperCase (text, SUBST_ONE);
    }

    /**
     * Converteix el primer caràcter de la primera paraules del <b>text</b> a
     * minúscules.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original conté el valor <b>null</b>, retorna
     * una cadena en blanc.
     *
     * <p>- Només es passa a minúscules el primer caracter de la primera paraula.
     *
     * @return Retorna el <b>text</b> amb el primer caracter de la primera
     * paraules passat a minúscules.
     */
    public static String firstCharToLowerCase (String text) {
        return firstCharToLowerCase (text, SUBST_ONE);
    }

    //    _____                           _                                    _
    //   / ____)                         | |                                  | |
    //  | |     ___ _ __ ____  _ _     __| | ___     ____  _ _ _ __  _ _  ____| |_ ___ _ __ ___
    //  | |    / _ \ `__) ___) _` |   / _` |/ _ \   / ___) _` | `__) _` |/ ___) __) _ \ `__) __)
    //  | |____  __/ | | (___ (_| |  | (_| |  __/  | (___ (_| | | | (_| | (___| |_  __/ |  \__ \
    //   \_____)___|_|  \____)__,_|   \__,_|\___|   \____)__,_|_|  \__,_|\____)\__)___|_|  (___/
    //
    //    _                   _
    //   (_)                 | |
    //    _     ____  _ _  __| | ___ _ __   ___ ___
    //   | |   / ___) _` |/ _` |/ _ \ `_  \/ _ \ __)
    //   | |  | (___ (_| | (_| |  __/ | | |  __/__ \
    //   |_|   \____)__,_|\__,_|\___|_| |_|\___|___/
    //
    //

    public static boolean isEmpty (String text) { return (text==null || text.length()==0 || text.trim().length()==0);  }
    public static boolean isNotEmpty (String text) { return (text!=null && text.length()>0 && text.trim().length()>0);  }

    /**
     * Cerca dintre de la cadena <b>text</b> la subcadena <b>cercaText</b>
     * i en retorna la seva posició.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text o la subcadena són nulles, retorna un -1.
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * @param cercaText Subcadena a cercar
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @param caseSensitive <b>CASE_SENSITIVE</b> o <b>true</b> si la
     * recerca ha de tenir en compte la tipografia; <b>CASE_INSENSITIVE</b> o
     * <b>false</b> en cas contrari.
     * @return Retorna la posició en la que ha trobat la cadena o <b>-1</b>
     * si no hi és.
     */
    public static int stringPos (String text, String cercaText, int startPos, boolean caseSensitive) {

        startPos = validaStartPos (text, startPos);

        int trobat = -1;
        if (text!=null && !isEmpty (cercaText)) {

            int cercaTextSize = cercaText.length();
            cercaText = lowerCase (cercaText, caseSensitive);

            int cont1 = startPos;
            int cont2 = 0;
            while (cont1<text.length()-cercaTextSize+cont2+1 && cont2<cercaTextSize) {
                if (lowerCase (text.charAt(cont1), caseSensitive)==cercaText.charAt(cont2)) { cont2++; }
                else {
                    cont1 = cont1 - cont2;
                    cont2 = 0;
                }
                cont1++;
            }
            if (cont2==cercaTextSize && cercaTextSize>0) trobat = cont1-cont2;
        }

        return trobat;
    }

    /**
     * Cerca dintre de la cadena <b>text</b> la subcadena <b>cercaText</b>
     * i en retorna la seva posició.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text o la subcadena són nulles, retorna un -1.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * @param cercaText Subcadena a cercar
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @return Retorna la posició en la que ha trobat la cadena o <b>-1</b>
     * si no hi és.
     */
    public static int stringPos (String text, String cercaText, int startPos) {

        return stringPos (text, cercaText, startPos, CASE_SENSITIVE);
    }

    /**
     * Cerca dintre de la cadena <b>text</b> la subcadena <b>cercaText</b>
     * i en retorna la seva posició.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text o la subcadena són nulles, retorna un -1.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- La recerca es realitza desde el inici de la cadena.
     *
     * @param cercaText Subcadena a cercar
     * @return Retorna la posició en la que ha trobat la cadena o <b>-1</b>
     * si no hi és.
     */
    public static int stringPos (String text, String cercaText) {
        return stringPos (text, cercaText, FROM_BEGIN, CASE_SENSITIVE);
    }

    /**
     * Cerca dintre de la cadena <b>text</b> el caràcter <b>caract</b>
     * i en retorna la seva posició.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text és nul.la, retorna un -1.
     *
     * <p>- Si el valor assignat als paràmetres <b>startPos</b> 
     * és negatiu o superior a la longitud del text, asumeix respectivament el
     * valor <b>0</b> o la longitud de la cadena.
     *
     * @param caract Caracter a cercar
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @param caseSensitive <b>CASE_SENSITIVE</b> o <b>true</b> si la
     * recerca ha de tenir en compte la tipografia; <b>CASE_INSENSITIVE</b> o
     * <b>false</b> en cas contrari.
     * @return Retorna la posició en la que ha trobat el caràcter o <b>-1</b>
     * si no hi és.
     */
    public static int charPos (String text, char caract, int startPos, boolean caseSensitive) {

        startPos = validaStartPos (text, startPos);
        int trobat = -1;

        if (text!=null) {
            int cont1 = startPos;

            if (caseSensitive) {
                while (trobat==-1 && cont1<text.length()) {
                    if (text.charAt(cont1)==caract) { trobat = cont1; }
                    cont1++;
                }
            }
            else {
                caract = toLowerCase (caract);

                while (trobat==-1 && cont1<text.length()) {
                    if (toLowerCase (text.charAt(cont1))==caract) { trobat = cont1; }
                    cont1++;
                }
            }

        }
        return trobat;
    }

    /**
     * Cerca dintre de la cadena <b>text</b>, començant del final cap al inici,
     * el caràcter <b>caract</b> i en retorna la seva posició.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text és nul.la, retorna un -1.
     *
     * <p>- Si el valor assignat als paràmetres <b>startPos</b>
     * és negatiu o superior a la longitud del text, asumeix respectivament el
     * valor <b>0</b> o la longitud de la cadena.
     *
     * @param caract Caracter a cercar
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @param caseSensitive <b>CASE_SENSITIVE</b> o <b>true</b> si la
     * recerca ha de tenir en compte la tipografia; <b>CASE_INSENSITIVE</b> o
     * <b>false</b> en cas contrari.
     * @return Retorna la posició en la que ha trobat el caràcter o <b>-1</b>
     * si no hi és.
     */
    public static int charInversePos (String text, char caract, int startPos, boolean caseSensitive) {

        startPos = validaStartPos (text, startPos);
        int trobat = -1;

        if (text!=null) {
            int cont1 = startPos;
            if (cont1>=text.length()) { cont1 = text.length()-1; }

            if (caseSensitive) {
                while (trobat==-1 && cont1>=0) {
                    if (text.charAt(cont1)==caract) { trobat = cont1; }
                    cont1--;
                }
            }
            else {
                caract = toLowerCase (caract);
                while (trobat==-1 && cont1>=0) {
                    if (toLowerCase (text.charAt(cont1))==caract) { trobat = cont1; }
                    cont1--;
                }
            }

        }
        return trobat;
    }


    /**
     * Cerca dintre de la cadena <b>text</b> el caràcter <b>caract</b>
     * i en retorna la seva posició.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text és nul.la, retorna un -1.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * @param caract Caracter a cercar
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @return Retorna la posició en la que ha trobat el caràcter o <b>-1</b>
     * si no hi és.
     */
    public static int charPos (String text, char caract, int startPos) {
        return charPos (text, caract, startPos, CASE_SENSITIVE);
    }

    /**
     * Cerca dintre de la cadena <b>text</b> el caràcter <b>caract</b>
     * i en retorna la seva posició.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text és nul.la, retorna un -1.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- La recerca es realitza desde el inici de la cadena.
     *
     * @param caract Caracter a cercar
     * @return Retorna la posició en la que ha trobat el caràcter o <b>-1</b>
     * si no hi és.
     */
    public static int charPos (String text, char caract) {
        return charPos (text, caract, FROM_BEGIN, CASE_SENSITIVE);
    }


    /**
     * Conta el número de vegades en que apareix el caracter indicat dintre de
     * la cadena <b>text</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text és nul.la, retorna un 0.
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * @param caract Caràcter que cal contar.
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @param caseSensitive <b>CASE_SENSITIVE</b> o <b>true</b> si la
     * recerca ha de tenir en compte la tipografia; <b>CASE_INSENSITIVE</b> o
     * <b>false</b> en cas contrari.
     * @return Retorna el número de vegades en que apareix el caràcter.
     */
    public static int countChar (String text,char caract, int startPos, boolean caseSensitive) {

        startPos = validaStartPos (text, startPos);

        int res = 0;
        if (text!=null) {

            if (caseSensitive) {
                for (int cont1=startPos; cont1<text.length(); cont1++) {
                    if (text.charAt(cont1)==caract) { res++; }
                }
            }
            else {
                caract = toLowerCase (caract);

                for (int cont1=startPos; cont1<text.length(); cont1++) {
                    if (toLowerCase (text.charAt(cont1))==caract) { res++; }
                }
            }
        }

        return res;
    }

    /**
     * Conta el número de vegades en que apareix el caracter indicat dintre de
     * la cadena <b>text</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text és nul.la, retorna un 0.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * @param caract Caràcter que cal contar.
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @return Retorna el número de vegades en que apareix el caràcter.
     */
    public static int countChar (String text, char caract, int startPos) {
        return countChar (text, caract, startPos, CASE_SENSITIVE);
    }

    /**
     * Conta el número de vegades en que apareix el caracter indicat dintre de
     * la cadena <b>text</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text és nul.la, retorna un 0.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- La recerca es realitza desde el inici de la cadena.
     *
     * @param caract Caràcter que cal contar.
     * @return Retorna el número de vegades en que apareix el caràcter.
     */
    public static int countChar (String text, char caract) {
        return countChar (text, caract, FROM_BEGIN, CASE_SENSITIVE);
    }

    /*
     * Retorna "true" si la cadena de text conté el caràcter "caract".
     */
    public static boolean containsChar (String text, char caract, boolean caseSensitive) {

        int pos = charPos (text, caract, FROM_BEGIN, caseSensitive);
        return (pos!=-1);
    }

    /**
     * Obté el paràmetre que està en la posició "numParam".
     *
     * NOTA: Si el separador es un espai en blanc, invoca el métode "blank"
     * abans de començar a comptar paràmetres.
     *
     * Exemples:
     *
     * La cadena "hola    que    tal" té 3 paràmetres
     * En canvi la cadena "Hola,,, que, tal" en té 5.
     *
     * NOTA: Aquesta funció recorre SEMPRE l'String desde el principi, això vol
     * dir que si es vol utilitzar de forma habitual sobre una mateixa cadena, 
     * és probable que interessi fer servir la classe "ParamsList" o 
     * "StringsList" que separen els paràmetres en objectes i tenen accés directe 
     * a cadascun d'ells.
     * 
     */
    public static String getParam (String text, char separador, int numParam) {

        String cadena = text;
        if (separador==' ') { cadena = blank (text); }
        StringBuilder res = new StringBuilder();

        char caract = '\0';
        int contParams = 0;
        int cont1 = 0;

        while (contParams<=numParam && cont1<cadena.length()) {
            caract = cadena.charAt (cont1);
            if (caract==separador) { contParams++; }
            else if (numParam == contParams) { res.append(caract); }

            cont1++;
        }
        return res.toString();
    }

    /**
     * Obté el número de paràmetres que té la cadena "text".
     *
     * NOTA: Si el separador es un espai en blanc, invoca el métode "blank"
     * abans de començar a comptar paràmetres.
     *
     * Exemples:
     *
     * La cadena "hola    que    tal" té 3 paràmetres
     * En canvi la cadena "Hola,,, que, tal" en té 5.
     * 
     * NOTA: Aquesta funció recorre SEMPRE l'String desde el principi, això vol
     * dir que si es vol utilitzar de forma habitual sobre una mateixa cadena,
     * és probable que interessi fer servir la classe "ParamsList" o
     * "StringsList" que separen els paràmetres en objectes i tenen accés directe
     * a cadascun d'ells.
     *
     */
    public static int getNumParams (String text, char separador) {

        String cadena = text;
        if (separador==' ') { cadena = blank (text); }

        int contParams = 0;
        int cont1 = 0;

        if (!isEmpty (cadena)) {
            contParams = 1;
            while (cont1<cadena.length()) {
                if (cadena.charAt(cont1)==separador) { contParams++; }
                cont1++;
            }
        }
        return contParams;
    }

    //    _____       _     _   _ _               _          _       _ _         _                   _
    //   / ____)     | |   | | (_) |             (_)        / /     | |_)       (_)                 (_)
    //  | (___  _   _| |_  | |_ _| |_ _   _  ____ _  ___   / /   ___| |_ _ _ __  _ _ __    _ _  ____ _  ___
    //   \___ \| | | | `_ \| __) | __) | | |/ ___) |/ _ \ / /   / _ \ | | ` `  \| | `_  \/ _` |/ ___) |/ _ \
    //   ____) | |_| | |_) | |_| | |_| |_| | (___| | (_) | /   |  __/ | | |\ |\ | | | | | (_| | (___| | (_) |
    //  (_____/ \__,_|_,__/ \__)_|\__)\__,_|\____)_|\___/_/     \___|_|_|_||_||_|_|_| |_|\__,_|\____)_|\___/
    //
    //       _                                    _                   _                   _
    //      | |                                  | |                 (_)                 | |
    //    __| | ___     ____  _ _ _ __  _ _  ____| |_ ___ _ __ ___    _     ____  _ _  __| | ___ _ __   ___ ___
    //   / _` |/ _ \   / ___) _` | `__) _` |/ ___) __) _ \ `__) __)  | |   / ___) _` |/ _` |/ _ \ `_  \/ _ \ __)
    //  | (_| |  __/  | (___ (_| | | | (_| | (___| |_  __/ |  \__ \  | |  | (___ (_| | (_| |  __/ | | |  __/__ \
    //   \__,_|\___|   \____)__,_|_|  \__,_|\____)\__)___|_|  (___/  |_|   \____)__,_|\__,_|\___|_| |_|\___|___/
    //
    //

    
    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han substituït les subcadenes amb el contingut de
     * <b>cercaText</b> pel la subcadena <b>nouText</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una
     * cadena en blanc.
     *
     * <p>- Si la cadena a cercar és nul.la o conté una cadena en blanc, aquest
     * métode retorna la cadena original.
     *
     * <p>- Si la cadena amb la que cal substituïr les subcadenes trobades conté el
     * valor <b>null</b> retorna la cadena original però sense les subcadenes
     * trobades.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * @param cercaText Cadena que cal cercar
     * @param nouText Cadena amb la qual cal subtituïr les subcadenes trobades.
     * @param substStyle Opció escollida per a realitzar la substitució. <b>SUBST_ONE</b>,
     * per a substituïr només la primera cadena trobada, <b>SUBST_ALL</b> (o
     * qualsevol altre valor) per a substituïr totes les cadenes trobades.
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @param caseSensitive <b>CASE_SENSITIVE</b> o <b>true</b> si la
     * recerca ha de tenir en compte la tipografia; <b>CASE_INSENSITIVE</b> o
     * <b>false</b> en cas contrari.
     * @return Retorna la cadena de text original però en la qual totes les
     * subcadenes amb el contingut de <b>cercaText</b> han estat
     * substituides per la cadena <b>nouText</b>.
     */
    public static String substString (String text, String cercaText, String nouText, byte substStyle, int startPos, boolean caseSensitive) {

        if (caseSensitive) { return substSensitiveString (text, cercaText, nouText, substStyle, startPos); }
        else { return substInsensitiveString (text, cercaText, nouText, substStyle, startPos); }
        
    }

    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han substituït les subcadenes amb el contingut de
     * <b>cercaText</b> pel la subcadena <b>nouText</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una
     * cadena en blanc.
     *
     * <p>- Si la cadena a cercar és nul.la o conté una cadena en blanc, aquest
     * métode retorna la cadena original.
     *
     * <p>- Si la cadena amb la que cal substituïr les subcadenes trobades conté el
     * valor <b>null</b> retorna la cadena original però
     * sense les subcadenes trobades.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * @param cercaText Cadena que cal cercar
     * @param nouText Cadena amb la qual cal subtituïr les subcadenes trobades.
     * @param substStyle Opció escollida per a realitzar la substitució. <b>SUBST_ONE</b>,
     * per a substituïr només la primera cadena trobada, <b>SUBST_ALL</b> (o
     * qualsevol altre valor) per a substituïr totes les cadenes trobades.
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @return Retorna la cadena de text original però en la qual totes les
     * subcadenes amb el contingut de <b>cercaText</b> han estat
     * substituides per la cadena <b>nouText</b>.
     */
     public static String substString (String text, String cercaText, String nouText, byte substStyle, int startPos) {
        return substString (text, cercaText, nouText, substStyle, startPos, CASE_SENSITIVE);
     }

    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han substituït les subcadenes amb el contingut de
     * <b>cercaText</b> pel la subcadena <b>nouText</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una
     * cadena en blanc.
     *
     * <p>- Si la cadena a cercar és nul.la o conté una cadena en blanc, aquest
     * métode retorna la cadena original.
     *
     * <p>- Si la cadena amb la que cal substituïr les subcadenes trobades conté el
     * valor <b>null</b> retorna la cadena original però
     * sense les subcadenes trobades.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- La recerca es realitza desde el inici de la cadena.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * @param cercaText Cadena que cal cercar
     * @param nouText Cadena amb la qual cal subtituïr les subcadenes trobades.
     * @param substStyle Opció escollida per a realitzar la substitució. <b>SUBST_ONE</b>,
     * per a substituïr només la primera cadena trobada, <b>SUBST_ALL</b> (o
     * qualsevol altre valor) per a substituïr totes les cadenes trobades.
     * @return Retorna la cadena de text original però en la qual totes les
     * subcadenes amb el contingut de <b>cercaText</b> han estat
     * substituides per la cadena <b>nouText</b>.
     */
    public static String substString (String text, String cercaText, String nouText, byte substStyle) {
        return substString (text, cercaText, nouText, substStyle, FROM_BEGIN, CASE_SENSITIVE);
    }

    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han substituït les subcadenes amb el contingut de
     * <b>cercaText</b> pel la subcadena <b>nouText</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una
     * cadena en blanc.
     *
     * <p>- Si la cadena a cercar és nul.la o conté una cadena en blanc, aquest
     * métode retorna la cadena original.
     *
     * <p>- Si la cadena amb la que cal substituïr les subcadenes trobades conté el
     * valor <b>null</b> retorna la cadena original però
     * sense les subcadenes trobades.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- La recerca es realitza desde el inici de la cadena.
     *
     * <p>- Substitueix la primero o totes les cadenes trobades en funció del tipus
     * de substitució asignat per defecte.
     *
     * @param cercaText Cadena que cal cercar
     * @param nouText Cadena amb la qual cal subtituïr les subcadenes trobades.
     * @return Retorna la cadena de text original però en la qual totes les
     * subcadenes amb el contingut de <b>cercaText</b> han estat
     * substituides per la cadena <b>nouText</b>.
     */
     public static String substString (String text, String cercaText, String nouText) {
        return substString (text, cercaText, nouText, SUBST_ALL, FROM_BEGIN, CASE_SENSITIVE);
    }

    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han substituït els caracters amb el valor
     * <b>cercaCaract</b> pel caracter <b>nouCaract</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una
     * cadena en blanc.
     *
     * <p>- Si el caracter a cercar conté el codi <b>\0</b>, aquest métode retorna la cadena
     * original.
     *
     * <p>- Si el caracter amb el que cal substituïr els caracters trobats conté el
     * codi <b>\0</b> retorna la cadena original però sense els caracters
     * trobats.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * @param text Text original
     * @param cercaCaract Caracter a cercar
     * @param nouCaract Caracter amb el qual cal subtituïr els caracters trobats.
     * @param substStyle Opció escollida per a realitzar la substitució. <b>SUBST_ONE</b>,
     * per a substituïr només el primer caracter trobat, <b>SUBST_ALL</b> (o
     * qualsevol altre valor) per a substituïr tots els caracters trobats.
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @param caseSensitive <b>CASE_SENSITIVE</b> o <b>true</b> si la
     * recerca ha de tenir en compte la tipografia; <b>CASE_INSENSITIVE</b> o
     * <b>false</b> en cas contrari.
     * @return Retorna la cadena de text original però en la qual tots els caracters
     * amb el contingut de <b>cercaCaract</b> han estat substituits pel
     * caracter <b>nouCaract</b>.
     */
    public static String substChar (String text, char cercaCaract, char nouCaract, byte substStyle, int startPos, boolean caseSensitive) {

        startPos = validaStartPos (text, startPos);
        substStyle = validaSubstStyle (substStyle);

        StringBuffer buffer = null;

        if (text!=null) {

            buffer = new StringBuffer (text.length());
            int cont1 = 0;
            boolean completat = false;

            if (caseSensitive) {
                while (cont1<text.length()) {
                    if (!completat && text.charAt (cont1)==cercaCaract) {
                        buffer.append (nouCaract);
                        if (substStyle==SUBST_ONE) { completat = true; }
                    }
                    else { buffer.append (text.charAt (cont1)); }
                    cont1++;
                }
            }
            else {
                cercaCaract = toLowerCase (cercaCaract);
                while (cont1<text.length()) {
                    if (!completat && toLowerCase (text.charAt (cont1))==cercaCaract) {
                        buffer.append (nouCaract);
                        if (substStyle==SUBST_ONE) { completat = true; }
                    }
                    else { buffer.append (text.charAt (cont1)); }
                    cont1++;
                }
             }
        }
        else { buffer = new StringBuffer (""); }

        return buffer.toString();
    }

    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han substituït els caracters amb el valor
     * <b>cercaCaract</b> pel caracter <b>nouCaract</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una
     * cadena en blanc.
     *
     * <p>- Si el caracter a cercar conté el codi <b>\0</b>, aquest métode retorna la cadena
     * original.
     *
     * <p>- Si el caracter amb el que cal substituïr els caracters trobats conté el
     * codi <b>\0</b> retorna la cadena original però sense els caracters
     * trobats.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * @param text Text original
     * @param cercaCaract Caracter a cercar
     * @param nouCaract Caracter amb el qual cal subtituïr els caracters trobats.
     * @param substStyle Opció escollida per a realitzar la substitució. <b>SUBST_ONE</b>,
     * per a substituïr només el primer caracter trobat, <b>SUBST_ALL</b> (o
     * qualsevol altre valor) per a substituïr tots els caracters trobats.
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @return Retorna la cadena de text original però en la qual tots els caracters
     * amb el contingut de <b>cercaCaract</b> han estat substituits pel
     * caracter <b>nouCaract</b>.
     */
    public static String substChar (String text, char cercaCaract, char nouCaract, byte substStyle, int startPos) {
        return substString (text, Character.toString (cercaCaract), Character.toString (nouCaract), substStyle, startPos, CASE_SENSITIVE);
    }

    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han substituït els caracters amb el valor
     * <b>cercaCaract</b> pel caracter <b>nouCaract</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una
     * cadena en blanc.
     *
     * <p>- Si el caracter a cercar conté el codi <b>\0</b>, aquest métode retorna la cadena
     * original.
     *
     * <p>- Si el caracter amb el que cal substituïr els caracters trobats conté el
     * codi <b>\0</b> retorna la cadena original però sense els caracters
     * trobats.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- La recerca es realitza desde el inici de la cadena
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * @param text Text original
     * @param cercaCaract Caracter a cercar
     * @param nouCaract Caracter amb el qual cal subtituïr els caracters trobats.
     * @param option Opció escollida per a realitzar la substitució. <b>SUBST_ONE<b>,
     * per a substituïr només el primer caracter trobat, <b>SUBST_ALL</b> (o
     * qualsevol altre valor) per a substituïr tots els caracters trobats.
     * @return Retorna la cadena de text original però en la qual tots els caracters
     * amb el contingut de <b>cercaCaract</b> han estat substituits pel
     * caracter <b>nouCaract</b>.
     */
    public static String substChar (String text, char cercaCaract, char nouCaract, byte substStyle) {
        return substString (text, Character.toString (cercaCaract), Character.toString (nouCaract), substStyle, FROM_BEGIN, CASE_SENSITIVE);
    }

    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han substituït els caracters amb el valor
     * <b>cercaCaract</b> pel caracter <b>nouCaract</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una
     * cadena en blanc.
     *
     * <p>- Si el caracter a cercar conté el codi <b>\0</b>, aquest métode retorna la cadena
     * original.
     *
     * <p>- Si el caracter amb el que cal substituïr els caracters trobats conté el
     * codi <b>\0</b> retorna la cadena original però sense els caracters
     * trobats.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- La recerca es realitza desde el inici de la cadena.
     *
     * <p>- Substitueix la primero o totes les cadenes trobades en funció del tipus
     * de substitució asignat per defecte.
     *
     * @param text Text original
     * @param cercaCaract Caracter a cercar
     * @param nouCaract Caracter amb el qual cal subtituïr els caracters trobats.
     * @return Retorna la cadena de text original però en la qual tots els caracters
     * amb el contingut de <b>cercaCaract</b> han estat substituits pel
     * caracter <b>nouCaract</b>.
     */
    public static String substChar (String text, char cercaCaract, char nouCaract) {
        return substString (text, Character.toString (cercaCaract), Character.toString (nouCaract), SUBST_ALL, FROM_BEGIN, CASE_SENSITIVE);
    }


    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han eliminat totes les subcadenes amb el contingut de
     * <b>cercaText</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * <p>- Si la cadena a cercar és nul.la o conté una cadena en blanc, aquest
     * métode retorna la cadena original.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * @param cercaText Cadena a eliminar.
     * @param substStyle Opció escollida per a realitzar la substitució. <b>SUBST_ONE</b>,
     * per a substituïr només el primer caracter trobat, <b>SUBST_ALL</b> (o
     * qualsevol altre valor) per a substituïr tots els caracters trobats.
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @param caseSensitive <b>CASE_SENSITIVE</b> o <b>true</b> si la
     * recerca ha de tenir en compte la tipografia; <b>CASE_INSENSITIVE</b> o
     * <b>false</b> en cas contrari.
     * @return Retorna la cadena de text original però en la qual s'han eliminat
     * totes les subcadenes iguals a <b>cercaText</b> .
     */
    public static String noString (String text, String cercaText, byte substStyle, int startPos, boolean caseSensitive) {
        return substString (text, cercaText, null, substStyle, startPos, caseSensitive);
    }

    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han eliminat totes les subcadenes amb el contingut de
     * <b>cercaText</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * <p>- Si la cadena a cercar és nul.la o conté una cadena en blanc, aquest
     * métode retorna la cadena original.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * @param cercaText Cadena a eliminar.
     * @param substStyle Opció escollida per a realitzar la substitució. <b>SUBST_ONE</b>,
     * per a substituïr només el primer caracter trobat, <b>SUBST_ALL</b> (o
     * qualsevol altre valor) per a substituïr tots els caracters trobats.
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @return Retorna la cadena de text original però en la qual s'han eliminat
     * totes les subcadenes iguals a <b>cercaText</b> .
     */
    public static String noString (String text, String cercaText, byte substStyle, int startPos) {
        return substString (text, cercaText, null, substStyle, startPos, CASE_SENSITIVE);
    }

    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han eliminat totes les subcadenes amb el contingut de
     * <b>cercaText</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * <p>- Si la cadena a cercar és nul.la o conté una cadena en blanc, aquest
     * métode retorna la cadena original.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- La recerca es realitza desde el inici de la cadena.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * @param cercaText Cadena a eliminar.
     * @param option Opció escollida per a realitzar la substitució. <b>SUBST_ONE<b>,
     * per a substituïr només el primer caracter trobat, <b>SUBST_ALL</b> (o
     * qualsevol altre valor) per a substituïr tots els caracters trobats.
     * @return Retorna la cadena de text original però en la qual s'han eliminat
     * totes les subcadenes iguals a <b>cercaText</b> .
     */
    public static String noString (String text, String cercaText, byte substStyle) {
        return substString (text, cercaText, null, substStyle, FROM_BEGIN, CASE_SENSITIVE);
    }

    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han eliminat totes les subcadenes amb el contingut de
     * <b>cercaText</b>. Aquesta recerca és sensible a la tipografia.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * <p>- Si la cadena a cercar és nul.la o conté una cadena en blanc, aquest
     * métode retorna la cadena original.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- La recerca es realitza desde el inici de la cadena.
     *
     * <p>- Substitueix la primero o totes les cadenes trobades en funció del tipus
     * de substitució asignat per defecte.
     *
     * @param cercaText Cadena a eliminar.
     * @return Retorna la cadena de text original però en la qual s'han eliminat
     * totes les subcadenes iguals a <b>cercaText</b> .
     */
    public static String noString (String text, String cercaText) {
        return substString (text, cercaText, null, SUBST_ALL, FROM_BEGIN, CASE_SENSITIVE);
    }

    public static String noChars (String text, char[] caracts, byte substStyle, int startPos, boolean caseSensitive) {

        startPos = validaStartPos (text, startPos);
        substStyle = validaSubstStyle (substStyle);

        StringBuilder res = null;

        if (text!=null) {

            res = new StringBuilder (text.length());

            int cont1 = startPos;
            int numCharsSubst = 0;
            while (cont1<text.length()) {
                if (SetUtils.charInArray (text.charAt (cont1), caseSensitive, caracts) && (substStyle==SUBST_ALL  || numCharsSubst==0)) { numCharsSubst++; }
                else { res.append (text.charAt (cont1)); }
                cont1++;
            }

        }
        else { res = new StringBuilder (""); }

        return res.toString();
    }

    public static String noChars (String text, char[] caracts, byte substStyle, int startPos) {
        return noChars (text, caracts, substStyle, startPos, CASE_SENSITIVE);
    }

    public static String noChars (String text, char[] caracts, byte substStyle) {
        return noChars (text, caracts, substStyle, FROM_BEGIN, CASE_SENSITIVE);
    }

    public static String noChars (String text, char[] caracts) {
        return noChars (text, caracts, SUBST_ALL, FROM_BEGIN, CASE_SENSITIVE);
    }

    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han eliminat tots els caracters iguals a <b>caract</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode
     * retorna una cadena en blanc.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * @param cercaText Caracter a eliminar.
     * @param substStyle Opció escollida per a realitzar la substitució. <b>SUBST_ONE</b>,
     * per a substituïr només el primer caracter trobat, <b>SUBST_ALL</b> (o
     * qualsevol altre valor) per a substituïr tots els caracters trobats.
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @param caseSensitive <b>CASE_SENSITIVE</b> o <b>true</b> si la
     * recerca ha de tenir en compte la tipografia; <b>CASE_INSENSITIVE</b> o
     * <b>false</b> en cas contrari.
     * @return Retorna la cadena de text original però en la qual s'han eliminat
     * tots els caracters <b>caract</b> .
     */
    public static String noChar (String text, char caract, byte substStyle, int startPos, boolean caseSensitive) {
        return noChars (text, new char[] {caract}, substStyle, startPos, caseSensitive);
    }


    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han eliminat tots els caracters iguals a <b>caract</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode
     * retorna una cadena en blanc.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * @param cercaText Caracter a eliminar.
     * @param substStyle Opció escollida per a realitzar la substitució. <b>SUBST_ONE</b>,
     * per a substituïr només el primer caracter trobat, <b>SUBST_ALL</b> (o
     * qualsevol altre valor) per a substituïr tots els caracters trobats.
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @return Retorna la cadena de text original però en la qual s'han eliminat
     * tots els caracters <b>caract</b> .
     */
    public static String noChar (String text, char caract, byte substStyle, int startPos) {
        return noChar (text, caract, substStyle, startPos, CASE_SENSITIVE);
    }

    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han eliminat tots els caracters iguals a <b>caract</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode
     * retorna una cadena en blanc.
     *
     * <p>- La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     *
     * <p>- La recerca es realitza desde el inici de la cadena.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * @param cercaText Caracter a eliminar.
     * @param substStyle Opció escollida per a realitzar la substitució. <b>SUBST_ONE</b>,
     * per a substituïr només el primer caracter trobat, <b>SUBST_ALL</b> (o
     * qualsevol altre valor) per a substituïr tots els caracters trobats.
     * @return Retorna la cadena de text original però en la qual s'han eliminat
     * tots els caracters <b>caract</b> .
     */
    public static String noChar (String text, char caract, byte substStyle) {
        return noChar (text, caract, substStyle, FROM_BEGIN, CASE_SENSITIVE);
    }

    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han eliminat tots els caracters iguals a <b>caract</b>.
     *
     * <b>Notes:</b>
     * - Si la cadena de text original és nul.la, aquest métode
     * retorna una cadena en blanc.
     * - La recerca de coincidències serà sensible a la tipografia en funció del
     * valor asignat per defecte.
     * - La recerca es realitza desde el inici de la cadena.
     * - Substitueix la primero o totes les cadenes trobades en funció del tipus
     * de substitució asignat per defecte.
     *
     * @param cercaText Caracter a eliminar.
     * @return Retorna la cadena de text original però en la qual s'han eliminat
     * tots els caracters <b>caract</b> .
     */
    public static String noChar (String text, char caract) {
        return noChar (text, caract, SUBST_ALL, FROM_BEGIN, CASE_SENSITIVE);
    }


    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han eliminat tots els espais en blanc.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * @param substStyle Opció escollida per a realitzar la substitució. <b>SUBST_ONE</b>,
     * per a substituïr només el primer caracter trobat, <b>SUBST_ALL</b> (o
     * qualsevol altre valor) per a substituïr tots els caracters trobats.
     * @param startPos Posició inicial a partir de la qual s'ha d'iniciar la
     * recerca.
     * @return Retorna la cadena de text original però en la qual s'han eliminat
     * tots els espais en blanc.
     */
    public static String noSpaces (String text, int startPos) {
        return noChar (text, ' ', SUBST_ALL, startPos, CASE_INSENSITIVE);
    }

    /**
     * Retorna una cadena amb el contingut de la cadena <b>text</b> però
     * en la qual s'han eliminat tots els espais en blanc.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * <p>- Si el valor assignat al paràmetre <b>substStyle</b> no és vàlid
     * asumeix el valor <b>SUBST_ALL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * @param substStyle Opció escollida per a realitzar la substitució. <b>SUBST_ONE</b>,
     * per a substituïr només el primer caracter trobat, <b>SUBST_ALL</b> (o
     * qualsevol altre valor) per a substituïr tots els caracters trobats.
     * @return Retorna la cadena de text original però en la qual s'han eliminat
     * tots els espais en blanc.
     */
    public static String noSpaces (String text) {
        return noChar (text, ' ', SUBST_ALL, FROM_BEGIN, CASE_INSENSITIVE);
    }

    public static String noEspecialChars (String text) {

        StringBuilder res = new StringBuilder (text.length());
        for (int cont1=0; cont1<text.length(); cont1++) {

            boolean validChar = false;
            char caract = StringUtils.toLowerCase (text.charAt (cont1));
            if (caract>='a' && caract<='z') { validChar = true; }
            if (caract==' ') { validChar = true; }
            if (validChar) { res.append(text.charAt (cont1)); }

        }
        return res.toString();
    }

    /**
     * Substitueix les cometes dobles pel caracter indicat
     * 
     * @param text
     * @param caract
     * @return
     */
    public static String noCometesDobles (String text, char caract) {
        return substChar (text, '"', caract, StringUtils.SUBST_ALL, StringUtils.FROM_BEGIN, StringUtils.CASE_SENSITIVE);
    }

    /**
     * Substitueix les cometes dobles per cometes simples
     *
     * @param text
     * @param caract
     * @return
     */
    public static String noCometesDobles (String text) { return noCometesDobles (text, '\''); }

    /**
     * Substitueix les cometes simples pel caracter indicat
     *
     * @param text
     * @param caract
     * @return
     */
    public static String noCometesSimples (String text, char caract) {
        return substChar (text, '"', '\'', StringUtils.SUBST_ALL, StringUtils.FROM_BEGIN, StringUtils.CASE_SENSITIVE);
    }

    /**
     * Substitueix les cometes simples per les cometes dobles
     *
     * @param text
     * @param caract
     * @return
     */
    public static String noCometesSimples (String text) { return noCometesDobles (text, '"'); }

    /*
     * Substitueix d'una cadena de text, els caràcters de salt de lines+retorn
     * de carro, pel tag HTML "<br/>".
     */
    public static String useBR (String text) {

        text = substString (text, "\r\n", "<br>");
        text = substString (text, "\n", "<br>");
        return text;
    }

    /**
     * Aquest métode retorna una cadena amb el contingut de <b>text</b>
     * però en la qual se li han tret tots els caracters espais en blanc innecessaris,
     * incloent els del inici i final de la cadena; pero conservant
     * els importants.
     *
     * <p>Per ejemple, una cadena amb el contingut següent: "  Hola   que  tal.   "
     * quedaria de la següent forma: "Hola que tal."
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     */
    public static String blank (String text) {

        StringBuilder buffer = new StringBuilder ("");

        if (text!=null) {
            char lastChar = ' ';
            char caract;
            for (int cont1=0; cont1<text.length(); cont1++) {
                caract = text.charAt(cont1);
                if (caract!=' ') {
                    if (lastChar==' ' && buffer.length()!=0) { buffer.append (lastChar); }
                    buffer.append (caract); }
                lastChar = caract;
            }
        }

        return buffer.toString();
    }

    /**
     * Aquest métode és igual al métode <b>blank</b> però amb la
     * característica de que és capaç de reconèixer els literals i deixar-los
     * intactes.
     *
     * <p>Per ejemple, una cadena amb el contingut següent:
     * "  El   meu   nom   és    'Miquel     Abadal'.   "
     * quedaria de la següent forma: "El meu nom és 'Miquel     Abadal'."
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * <p>- Si un literal no està correctament delimitat amb el caràcter de tancament,
     * no es considera com a literal.
     *
     * @return Retorna la cadena original però sense els espais en blanc
     * innecessàris.
     */
    public static String blankAdv (String text) {

        StringBuffer buffer;

        if (text!=null) {
            int textSize = text.length();
            buffer = new StringBuffer (textSize);

            char lastChar = ' ';
            char caract;
            char literalChar = '\0';
            boolean onLiteral = false;

            for (int cont1=0; cont1<textSize; cont1++) {
                caract = text.charAt(cont1);

                if (onLiteral==false) {
                    if (caract=='\'' || caract=='"') {
                        
                        // Si existeix una cometa de tancament de literal...
                        if (charPos (text, caract, cont1+1)!=-1) {
                            onLiteral= true;
                            literalChar = caract;
                        }
                    }
                    if (caract!=' ') {
                        if (lastChar==' ' && buffer.length()!=0) { buffer.append (lastChar); }
                        buffer.append (caract); }
                    lastChar = caract;
                }
                else {
                    if (caract==literalChar) { onLiteral = false; }
                    buffer.append (caract);
                }
            }

        }
        else { buffer = new StringBuffer (""); }

        return buffer.toString();
    }


    public static String trim (String text) {
        if (text!=null) { return text.trim(); }
        return text;
    }

    /**
     * Retorna la cadena <b>text</b>, però substituïnt tots els accents
     * pe'l seu caràcter sense accent, i substituint les "ñ" per "n" i les ces
     * trencades "ç" per "c".
     *
     * <p>Així doncs la cadena següent: "El caçador dóna una coça a l'àliga"
     * quedaria de la següent manera: "El cacador dona una coca a l'aliga"
     *
     * <p><b>Nota:</b>
     * 
     * <p>- Si la cadena de text original conté una valor <b>null</b> aquest métode
     * retorna una cadena en blanc.
     *
     * @return  Retorna una cadena de text amb tots els accents substituïts.
     */
    public static String senseAccents (String text) {

        if (withoutTildeTABLE==null) withoutTildeTABLE = initWithoutTilde();

        StringBuffer res;
        if (text!=null) {
            int textSize = text.length();

            res = new StringBuffer (text.length());
            for (int cont1=0; cont1<textSize; cont1++) {
                res.append (senseAccents (text.charAt(cont1)));
            }
        }
        else res = new StringBuffer ("");

        return res.toString();
    }

    /**
     * Crea una cadena amb el contingut de <b>text</b>, però duplicant el
     * caràcter <b>caract</b>, cada cop que se'l trobi.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * @param duplicateCaract Caracter que cal duplicar.
     * @param caseSensitive <b>true</b> si la recerca ha de tenir en compte
     * la tipografia <b>false</b> en cas contrari.
     * @return  Retorna la cadena de text resultat.
     */
    public static String duplicateChar (String text, char duplicateCaract, boolean caseSensitive) {

        String textOrig = Character.toString (duplicateCaract);
        String textDesti = textOrig + textOrig;
        return substString (text, textOrig, textDesti, SUBST_ALL, FROM_BEGIN, caseSensitive);
    }

    /**
     * Crea una cadena amb el contingut de <b>text</b>, però duplicant el
     * caràcter <b>caract</b>, cada cop que se'l trobi.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * @param duplicateCaract Caracter que cal duplicar.
     * @param caseSensitive <b>true</b> si la recerca ha de tenir en compte
     * la tipografia <b>false</b> en cas contrari.
     * @return  Retorna la cadena de text resultat.
     */
    public static String duplicateChar (String text, char duplicateCaract) {

        String textOrig = Character.toString (duplicateCaract);
        String textDesti = textOrig + textOrig;
        return substString (text, textOrig, textDesti, SUBST_ALL, FROM_BEGIN, CASE_SENSITIVE);
    }


    /**
     * Crea una cadena amb el contingut de <b>text</b>, però duplicant
     * totes les cometes dobles '"'.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * @return  Retorna la cadena de text resultat.
     */
    public static String duplicaCometesDobles (String text) {
         return substString (text, "\"", "\"\"", SUBST_ALL, 0);
    }

    /**
     * Crea una cadena amb el contingut de <b>text</b>, però duplicant
     * totes les cometes simples '\''.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * @return  Retorna la cadena de text resultat.
     */
    public static String duplicaCometesSimples (String text) {
        return substString (text, "'", "''", SUBST_ALL, 0);
    }

    /**
     * Evita assignar a una cadena el valor <b>null<b>.
     * 
     * En cas de que el valor a assignar sigui "null", es substitueix per la
     * cadena "defaultValue".
     * 
     * @param text
     * @return Retorna la cadena de text original, o una cadena buida si la cadena
     * original era nul.la.
     */
    public static String notNull (String text, String defaultValue) {
        if (text==null) { return defaultValue; }
        return text;
    }

    /**
     * Evita assignar a una cadena el valor <b>null<b>.
     *
     * En cas de que el valor a assignar sigui "null", es substitueix per una
     * cadena en blanc.
     *
     * @param text
     * @return Retorna la cadena de text original, o una cadena buida si la cadena
     * original era nul.la.
     */
    public static String notNull (String text) {
        if (text==null) { return ""; }
        return text;
    }
    
    /**
     * Evita que un String contingui un null o una cadena buida o una cadena amb
     * només espais en blanc, i la assigna el valor per defecte.
     */
    public static String notEmpty (String text, String defaultValue) {
        if (isEmpty(text)) return defaultValue;
        return text;
    }

    /**
     * Elimina els espais en blanc al inici i final de la cadena; i si la
     * cadena resultant és buida returna "null".
     */
    public static String notEmpty (String text) {
        if (isEmpty(text)) return null;
        return text;
    }

    /**
     * Incrementa els valors numérics inclosos a la cadena "text".
     *
     * Aquesta funció ha estat dissenyada per a modificar la part numérica d'un
     * valor utilitzat per a enmagatzemar una mida en formnat HTML; la qual pot
     * utilitzar diferents formats com incloure un valor absolut ("10") o incloure les
     * unitats ("10px", "10em"), o una valor relatiu ("10%") o simplement un text
     * ("inherit").
     *
     * Tanmateix, aquest métode, suma a tots els segments numérics de la cadena el valor
     * indicat amb el paràmetre "value".
     *
     * Per ejemple, la cadena: "10px 12px 20px 5%", executant-la amb el "value=3" quedaria
     * com: "13px 15px 23px 8%"
     *
     * @return Retorna "null" si la cadena original conté null o la cadena original, però
     * incrementant tots els segments numérics amb el valor "value".
     */
    public static String addNumericValue (String text, int value, int minValue, int maxValue) {

        String res = null;

        if (text==null) { text = "0"; }

        res = "";
        String numeric = "";
        char caract = '\0';

        for (int cont1=0; cont1<text.length(); cont1++) {
            caract = text.charAt(cont1);

            if (caract<'0' || caract>'9') {
                if (numeric.length()>0) {
                    int num = MathEx.rang (StringToNumber.intValue (numeric, 0) + value, minValue, maxValue);
                    res = res + NumberToString.toString (num);
                    numeric="";
                }
                res = res + caract;
            }
            else { numeric = numeric + caract; }
        }

        if (numeric.length()>0) {
            int num = MathEx.rang (StringToNumber.intValue (numeric, 0) + value, minValue, maxValue);
            res = res + NumberToString.toString (num);
        }


        return res;
    }

    /**
     * Incrementa els valors numérics inclosos a la cadena "text".
     *
     * Aquesta funció ha estat dissenyada per a modificar la part numérica d'un
     * valor utilitzat per a enmagatzemar una mida en formnat HTML; la qual pot
     * utilitzar diferents formats com incloure un valor absolut ("10") o incloure les
     * unitats ("10px", "10em"), o una valor relatiu ("10%") o simplement un text
     * ("inherit").
     *
     * Tanmateix, aquest métode, suma a tots els segments numérics de la cadena el valor
     * indicat amb el paràmetre "value".
     *
     * Per ejemple, la cadena: "10px 12px 20px 5%", executant-la amb el "value=3" quedaria
     * com: "13px 15px 23px 8%"
     *
     * Els valors numérics no tenen límit superior, però si inferior, ja que no poden
     * ser menors de zero.
     * @return Retorna "null" si la cadena original conté null o la cadena original, però
     * incrementant tots els segments numérics amb el valor "value".
     */
    public static String addNumericValue (String text, int value) {
        return addNumericValue (text, value, 0, -1);
    }

    //   ______                  _                                                           _ _
    //  |  ____)                (_)                                                         | |_)
    //  | |___ _   _ _ __   ____ _  ___  _ __  ___    _ __   ___ _ __     ___  _ _ __  _ __ | |_ _ __
    //  |  ___) | | | `_  \/ ___) |/ _ \| `_  \ __)  | `_ \ / _ \ `__)   / _ \| ` `  \| `_ \| | | `__)
    //  | |   | |_| | | | | (___| | (_) | | | |__ \  | |_) |  __/ |     | (_) | |\ |\ | |_) | | | |
    //  |_|    \__,_|_| |_|\____)_|\___/|_| |_|___/  |  __/ \___|_|      \___/|_||_||_|  __/|_|_|_|
    //                                               | |                              | |
    //                  _                            |_|                              |_|
    //                 | |
    //    ____  _ _  __| | ___ _ __   ___ ___
    //   / ___) _` |/ _` |/ _ \ `_  \/ _ \ __)
    //  | (___ (_| | (_| |  __/ | | |  __/__ \
    //   \____)__,_|\__,_|\___|_| |_|\___|___/
    //
    //


    /**
     * Omple la cadena <b>text</b> amb el caracter <b>caract</b>
     * fins que tingui el tamany indicat a <b>size</b>.
     *
     * <p>Notes:</p>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * <p>- Si el tamany indicat amb el paràmetre <b>size</b> és inferior al de
     * la cadena de text, retorna la cadena de text original sense cap modificació.
     *
     * <p>- Si el valor assignat al paràmetre <b>fillStyle</b> no és vàlid, assumeix
     * com a valor <b>LEFT_FILL</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * <p>- Si el tamany indicat amb el paràmetre <b>size</b> és negatiu, assumeix
     * com a valor <b>0</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * @param caract Caracter amb el que s'ha d'omplir la cadena de text.
     * @param fillOption Indica com s'ha d'omplir la cadena; <b>LEFT_FILL</b>
     * per afeguir els caracters a la l'esquerra i <b>RIGHT_FILL</b> per afeguir-los
     * a la dreta.
     * @param size Tamany final que ha de tenir la cadena de text
     * @return  Retorna la cadena de text resultant.
     */
    public static String fillString (String text, char caract, byte fillStyle, int size) {

        fillStyle = validaFillStyle (fillStyle);
        size = validaSize (size);

        text = notNull (text);
        StringBuilder buffer = new StringBuilder (text);

        if (fillStyle==LEFT_FILL) {
            while (buffer.length()<size) { buffer.insert (FROM_BEGIN, caract); }
        }
        else {
            while (buffer.length()<size) { buffer.append(caract); }
        }

        return buffer.toString();
    }

    /**
     * Omple la cadena <b>text</b> amb el caracter <b>caract</b> fins que tingui
     * el tamany indicat a <b>size</b>.
     *
     * <p>Notes:</p>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * <p>- Els caracters sempre s'afegueixen al final de la cadena.
     *
     * <p>- Si el tamany indicat amb el paràmetre <b>size</b> és inferior al de
     * la cadena de text, retorna la cadena de text original sense cap modificació.
     *
     * <p>- Si el tamany indicat amb el paràmetre <b>size</b> és negatiu, assumeix
     * com a valor <b>0</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * @param caract Caracter amb el que s'ha d'omplir la cadena de text.
     * @param size Tamany final que ha de tenir la cadena de text
     * @return  Retorna la cadena de text resultant.
     */
    public static String fillString (String text, char caract, int size) {
        return fillString (text, caract, RIGHT_FILL, size);
    }

    /**
     * Omple la cadena <b>text</b> afeguinr zeros a l'esquerra fins que
     * tingui el tamany indicat a <b>size</b>.
     *
     * <p>Notes:</p>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * <p>- Si el tamany indicat amb el paràmetre <b>size</b> és inferior al de
     * la cadena de text, retorna la cadena de text original sense cap modificació.
     *
     * <p>- Si el tamany indicat amb el paràmetre <b>size</b> és negatiu, assumeix
     * com a valor <b>0</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * @param size Tamany final que ha de tenir la cadena de text
     * @return  Retorna la cadena de text resultant.
     */
    public static String fillLeftZeros (String text, int size) {
        return fillString (text, '0', LEFT_FILL, size);
    }

    /**
     * Omple la cadena <b>text</b> afeguinr espais en blanc a l'esquerra
     * fins que tingui el tamany indicat a <b>size</b>.
     *
     * <p>Notes:</p>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * <p>- Si el tamany indicat amb el paràmetre <b>size</b> és inferior al de
     * la cadena de text, retorna la cadena de text original sense cap modificació.
     *
     * <p>- Si el tamany indicat amb el paràmetre <b>size</b> és negatiu, assumeix
     * com a valor <b>0</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * @param size Tamany final que ha de tenir la cadena de text
     * @return  Retorna la cadena de text resultant.
     */
    public static String fillLeftSpaces (String text, int size) {
        return fillString (text, ' ', LEFT_FILL, size);
    }

    /**
     * Omple la cadena <b>text</b> afeguinr espais en blanc a la dreta
     * fins que tingui el tamany indicat a <b>size</b>.
     *
     * <p>Notes:</p>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * <p>- Si el tamany indicat amb el paràmetre <b>size</b> és inferior al de
     * la cadena de text, retorna la cadena de text original sense cap modificació.
     *
     * <p>- Si el tamany indicat amb el paràmetre <b>size</b> és negatiu, assumeix
     * com a valor <b>0</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * @param size Tamany final que ha de tenir la cadena de text
     * @return  Retorna la cadena de text resultant.
     */
    public static String fillRightSpaces (String text, int size) {
        return  fillString (text, ' ', RIGHT_FILL, size);
    }


    //   ______                  _
    //  |  ____)                (_)
    //  | |___ _   _ _ __   ____ _  ___  _ __  ___    _ __   ___ _ __      _ _
    //  |  ___) | | | `_  \/ ___) |/ _ \| `_  \ __)  | `_ \ / _ \ `__)   / _` |
    //  | |   | |_| | | | | (___| | (_) | | | |__ \  | |_) |  __/ |     | (_| |
    //  |_|    \__,_|_| |_|\____)_|\___/|_| |_|___/  |  __/ \___|_|      \__,_|
    //                                               | |
    //                                               |_|
    //         _     _             _                   _                    _
    //        | |   | |           (_)                 | |                  | |
    //    ___ | |_  | |_ ___ _ __  _ _ __    ___ _   _| |_    ____  _ _  __| | ___ _ __   ___ ___
    //   / _ \| `_ \| __) _ \ `_  \ | `__)  / __) | | | `_ \ / ___) _` |/ _` |/ _ \ `_  \/ _ \ __)
    //  | (_) | |_) | |_  __/ | | | | |     \__ \ |_| | |_) | (___ (_| | (_| |  __/ | | |  __/__ \
    //   \___/|_,__/ \__)___|_| |_|_|_|     (___/\__,_|_,__/ \____)__,_|\__,_|\___|_| |_|\___|___/
    //
    //


    /**
     * Retorna la primera paraula de la cadena <b>text</b>, prescindint dels espais
     * en blanc que hi pugui haver abans.
     *
     * <p>Notes:</p>
     *
     * <p>- Si la cadena de text original és nul.la, aquest métode retorna una cadena
     * en blanc.
     *
     * <p>- Aquest métode reconeix com a caracters que delimiten el final d'una
     * paraula els caracters següents: espai en blanc, comma, punt-i-comma i punt.
     *
     * @return  Retorna un String amb la primera paraula trobada.
     */
    public static String firstWord (String text, char[] endOfWordChars) {

        if (endOfWordChars==null) { endOfWordChars = new char[] {' ', ',', '.', ';'}; }

        StringBuilder res = new StringBuilder (text.length());

        boolean trobat = false;
        boolean onSpaces = true;

        int cont1 = 0;
        while (trobat==false && cont1<text.length()) {
            if (SetUtils.charIn (text.charAt(cont1), CASE_INSENSITIVE, endOfWordChars)) {
                if (onSpaces==false) { trobat = true; }
            }
            else {
                onSpaces = false;
                res.append (text.charAt(cont1));
            }

            cont1++;
        }

        return res.toString();
    }

    public static String firstWord (String text) { return firstWord (text, null); }

    /**
     * Copia de la cadena <b>text</b> una subcadena de <b>len</b>
     * caràcters desde la posició <b>startPos</b>.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena original és <b>null</b>, retorna una cadena en blanc.
     *
     * <p>- Si la posició d'inici més la longitud solicitada superen la longitud
     * de la cadena, es copia fins al final.
     *
     * <p>- Si la longitud té el valor <code>-1</code> es considera que es vol copiar
     * desde la posició d'inici indicada fins al final.
     *
     * <p>- Si el valor assignat al paràmetre <b>startPos</b> és negatiu o superior
     * a la longitud del text, asumeix respectivament el valor <b>0</b> o la
     * longitud de la cadena i crea una instància d'error amb la corresponent
     * descripció. (veure {@link StringEx#getError})
     *
     * <p>- Si el tamany indicat amb el paràmetre <b>len</b> és negatiu, assumeix
     * com a valor <b>0</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * @param startPos Posició del caracter a partir del qual s'ha de copiar la
     * subcadena
     * @param len Longitud en caràcters de la subcadena
     * @return Retorna la subcadena indicada
     */
    public static String copy (String text, int startPos, int len) {

        startPos = validaStartPos (text, startPos);

        StringBuilder res = null;
        if (text!=null) {
            if (len==-1) { len = text.length()-startPos; }
            len = validaSize (len);

            res = new StringBuilder (text.length());
            int cont1 = startPos;
            int numChars = 0;
            while (cont1<text.length() && numChars<len) {
                res.append (text.charAt(cont1));
                cont1++;
                numChars++;
            }
        }
        else res = new StringBuilder (0);

        return res.toString ();
    }

    /**
     * Copia de la cadena <b>text</b> una subcadena dels primers <b>len</b>
     * caràcters.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena original és <b>null</b>, retorna una cadena en blanc.
     * 
     * <p>- Si el tamany indicat amb el paràmetre <b>len</b> és negatiu, assumeix
     * com a valor <b>0</b> i crea una instància d'error amb la
     * corresponent descripció. (veure {@link StringEx#getError})
     *
     * @param startPos Posició del caracter a partir del qual s'ha de copiar la
     * subcadena
     * @param len Longitud en caràcters de la subcadena
     * @return Retorna la subcadena indicada
     */
    public static String copy (String text, int len) {
        return copy (text, FROM_BEGIN, len);
    }

    /**
     * Copia la cadena text desde el primer caracter igual a "fromChar" fins
     * al següent caracter igual a "untilChar".
     *
     * Si as'activa el "inclusive" la cadena resultant contindrá els caracter
     * inicial i final; en cas contrari, aquest carac ter no s'inclouran a la
     * cadena resultant.
     */
    public static String copy (String text, char fromChar, char untilChar, boolean caseSensitive, boolean inclusive) {

        String res = null;
        int incr = 0;
        if (!inclusive) { incr = 1; }
        int startPos = charPos (text, fromChar, FROM_BEGIN, caseSensitive);
        if (startPos!=-1) {
            int endPos = charPos (text, untilChar, startPos, caseSensitive);
            if (endPos!=-1) { res = copy (text, startPos+incr, endPos-incr); }
        }
        return StringUtils.notNull (res);
    }

    /**
     * Copia la cadena "text", desde el caracter en la posició "startPos", fins que troba el
     * caracter "untilChar"; si no el troba, copia la cadena fins al final.
     */
    public static String copy (String text, int startPos, char untilChar, boolean caseSensitive, boolean inclusive) {

        String res = text;
        int incr = 0;
        if (inclusive) { incr = 1; }
        int endPos = charPos (text, untilChar, startPos, caseSensitive);
        if (endPos!=-1) { res = copy (text, startPos, endPos-startPos+incr); }
        else { res = copy (text, startPos, -1);
        
        }
        return StringUtils.notNull (res);
    }

    /**
     * Copia la cadena "text", desde el primer caràcter, fins que troba el
     * caracter "untilChar"
     */
    public static String copy (String text, char untilChar, boolean caseSensitive, boolean inclusive) {
        return copy (text, 0, untilChar, caseSensitive, inclusive);
    }

    public static String copy (String text, String fromStr, String untilStr, boolean caseSensitive, boolean inclusive) {

        String res = null;

        if (fromStr!=null && untilStr!=null) {
            int fromLen = fromStr.length();
            int untilLen = untilStr.length();

            int startPos = stringPos (text, fromStr, 0, caseSensitive);
            if (startPos!=-1) {
                
                int endPos = stringPos (text, untilStr, startPos+fromLen, caseSensitive);

                if (!inclusive) {
                    startPos = startPos + fromLen;
                    if (endPos!=-1) { endPos = endPos - untilLen; }
                }
                
                int len = -1;
                if (endPos!=-1) { len = endPos-startPos+1; }
                res = copy (text, startPos, len);
            }
        }
        return StringUtils.notNull (res);
    }

    /**
     * Concatena la cadena de <b>text</b> amb la cadena <b>concatText</b> unint-les
     * amb la cadena <b>joinText</b> com a separador.
     *
     * <p><b>Notes:</b>
     *
     * <p>- Si la cadena <b>text</B> conté el valor <b>null</b> o no conté dades
     * no es té en compte en la concatenació.
     *
     * <p>- Si la cadena <b>concatText</b> conté el valor <b>null</b> o no conté dades
     * no es té en compte en la concatenació.
     *
     * <p>- La cadena de text <b>joinText</b> només s'utilitza quan les dues cadenes
     * a concatenar contenen dades, encara que només siguin espais en blanc
     *
     * <p><b>Exemples:</b>
     *
     * <p>null + ", " + null = ""<br>
     * null + ", " + "Miquel" = "Miquel"<br>
     * "Abadal" + ", " + null = "Abadal"<br>
     * "Abadal" + null + "Miquel" = "AbadalMiquel"<br>
     * "Abadal" + ", " + "Miquel" = "Abadal, Miquel"<br>
     *
     * @param joinText
     * @param concatText
     * @return
     */
    public static String concat (String text, String joinText, String concatText) {

        int numChars = 0;
        if (text!=null) { numChars = numChars + text.length(); }
        if (joinText!=null) { numChars = numChars + joinText.length(); }
        if (concatText!=null) { numChars = numChars + concatText.length(); }

        StringBuilder res = new StringBuilder (numChars);

        if (text!=null && text.length()>0) {
             res.append (text);
             if (concatText!=null && concatText.length()>0) {
                 if (joinText!=null) { res.append (joinText); }
             }
        }
        if (concatText!=null) { res.append (concatText); }

        return res.toString();
    }

    /**
     * Añade, al final de la cadena "text" el caracter "joinChar", siempre y 
     * cuando, la cadena "text" no termine con el caracter "joinChar".
     * 
    */
    public static String concat (String text, char joinChar) {

        boolean join = true;
        if (!isEmpty (text) && text.charAt(text.length()-1)==joinChar) { join = false; }

        if (join) { text = text + joinChar; }
        return text;
    }

    public static String concat (String text, String concatText) {

        if (text!=null) {
            if (concatText!=null) { text = text + concatText; }
        }
        else { text = concatText; }

        return text;
    }

    /**
     * Obté una cadena amb els caràcters NO numérics del text
     */
    public static String extractText (String text) {

        StringBuilder res = new StringBuilder();
        if (text!=null) {
            for (int cont1=0; cont1<text.length(); cont1++) {
                if ((text.charAt(cont1)<'0' || text.charAt(cont1)>'9') && text.charAt(cont1)!='-' && text.charAt(cont1)!='+') { res.append (text.charAt(cont1)); }
            }
        }
        return res.toString();
    }

    /**
     * Obté una cadena amb els caràcters numérics del text. 
     */
    public static String extractNumber (String text) {

        StringBuilder res = new StringBuilder();
        if (text!=null) {
            for (int cont1=0; cont1<text.length(); cont1++) {
                if (text.charAt(cont1)>='0' && text.charAt(cont1)<='9') { res.append (text.charAt(cont1)); }
            }
        }
        return res.toString();
    }
    
    /**
     * Obté una cadena amb els caràcters numérics del text incloent signe. 
     */
    public static String extractSignedNumber (String text) {

        StringBuilder res = new StringBuilder();
        if (text!=null) {
            for (int cont1=0; cont1<text.length(); cont1++) {
                if (res.length()==0 && text.charAt(cont1)=='-') { res.append (text.charAt(cont1)); }
                else if (res.length()==0 && text.charAt(cont1)=='+') { }
                else if (text.charAt(cont1)>='0' && text.charAt(cont1)<='9') { res.append (text.charAt(cont1)); }
            }
        }
        return res.toString();
    }

    //   ______                  _
    //  |  ____)                (_)
    //  | |___ _   _ _ __   ____ _  ___  _ __  ___    _ __   ___ _ __      _ _
    //  |  ___) | | | `_  \/ ___) |/ _ \| `_  \ __)  | `_ \ / _ \ `__)   / _` |
    //  | |   | |_| | | | | (___| | (_) | | | |__ \  | |_) |  __/ |     | (_| |
    //  |_|    \__,_|_| |_|\____)_|\___/|_| |_|___/  |  __/ \___|_|      \__,_|
    //                                               | |
    //                                               |_|
    //                                                                   _
    //                                                                  | |
    //    ____ ___  _ _ __  _ __    _ _ _ __  _ _ _ __     ____  _ _  __| | ___ _ __   ___ ___
    //   / ___) _ \| ` `  \| `_ \ / _` | `__) _` | `__)   / ___) _` |/ _` |/ _ \ `_  \/ _ \ __)
    //  | (___ (_) | |\ |\ | |_) | (_| | | | (_| | |     | (___ (_| | (_| |  __/ | | |  __/__ \
    //   \____)___/|_||_||_|  __/ \__,_|_|  \__,_|_|      \____)__,_|\__,_|\___|_| |_|\___|___/
    //                     | |
    //                     |_|

    public static boolean equals (String text1, String text2, boolean caseSensitive) {
        text1 = lowerCase (text1, caseSensitive);
        text2 = lowerCase (text2, caseSensitive);
        return text1.equals (text2);
    }

    public static int compare (String text1, String text2, boolean caseSensitive) {
        text1 = lowerCase (text1, caseSensitive);
        text2 = lowerCase (text2, caseSensitive);
        return text1.compareTo (text2);
    }

    public static boolean equals (char caract1, char caract2, boolean caseSensitive) {
        caract1 = lowerCase (caract1, caseSensitive);
        caract2 = lowerCase (caract2, caseSensitive);
        return caract1==caract2;
    }

    public static boolean startsWidth (String text, String textInicial, boolean caseSensitive) {

        text = notNull (text);
        textInicial = notNull (textInicial);
        if (!caseSensitive) {return equals (copy (text, textInicial.length()), textInicial, caseSensitive); }
        else { return notNull(text).startsWith (notNull (textInicial)); }
    }

    public static boolean endsWidth (String text, String textInicial, boolean caseSensitive) {

        text = notNull (text);
        textInicial = notNull (textInicial);
        if (!caseSensitive) {return equals (copy (text, text.length()-textInicial.length() -1), textInicial, caseSensitive); }
        else { return notNull(text).startsWith (notNull (textInicial)); }
    }


    //   ______                  _
    //  |  ____)                (_)
    //  | |___ _   _ _ __   ____ _  ___  _ __  ___    _ __   ___ _ __      _ _
    //  |  ___) | | | `_  \/ ___) |/ _ \| `_  \ __)  | `_ \ / _ \ `__)   / _` |
    //  | |   | |_| | | | | (___| | (_) | | | |__ \  | |_) |  __/ |     | (_| |
    //  |_|    \__,_|_| |_|\____)_|\___/|_| |_|___/  |  __/ \___|_|      \__,_|
    //                                               | |
    //                                               |_|
    //                  _                                      _ _   _         _     _ _
    //                 | |                                    | | | (_)       (_)   | |_)
    //    ____  _ _  __| | ___ _ __   ___ ___    _ _ __  _   _| | |_ _ ______  _  __| |_  ___  _ _ __    _ _
    //   / ___) _` |/ _` |/ _ \ `_  \/ _ \ __)  | ` `  \| | | | | __) |______)| |/ _` | |/ _ \| ` `  \ / _` |
    //  | (___ (_| | (_| |  __/ | | |  __/__ \  | |\ |\ | |_| | | |_| |       | | (_| | | (_) | |\ |\ | (_| |
    //   \____)__,_|\__,_|\___|_| |_|\___|___/  |_||_||_|\__,_|_|\__)_|       |_|\__,_|_|\___/|_||_||_|\__,_|
    //
    //


    /**
     * Extreu de la cadena 'text' la part escrita en el idioma 'locale'
     * 
     * La cadena ha de respectar la següent sintaxi:
     * "=#IDIOMA_1:text en el idioma 1#IDIOMA_2:text en el idioma 2#"
     */
    public static String extractLanguageText (String text, String language, String alternativeLanguage) {

        String res = null;
        if (text!=null) {
            if (text.length()>0 && (text.charAt(0)=='=' || text.charAt(0)=='#')) {
                if (!isEmpty (language)) { res = StringUtils.copy (text, "#"+language+":", "#", StringUtils.CASE_INSENSITIVE, false); }
                if (isEmpty (res) && !isEmpty (alternativeLanguage)) { res = StringUtils.copy (text, "#"+alternativeLanguage+":", "#", StringUtils.CASE_INSENSITIVE, false); }
                if (isEmpty (res)) { res = StringUtils.copy(text, "#EN:", "#", StringUtils.CASE_INSENSITIVE, false); }
                if (isEmpty (res)) { res = text; }
            }
            else { res = text; }
        }
        return res;
    }

    public static String extractLanguageText (String text, Locale locale) {
        return extractLanguageText ( text, LocaleUtils.getISO1Language(locale), null);
    }

    public static String extractLanguageText (String text, Locale locale, Locale alternativeLocale) {
        return extractLanguageText ( text, LocaleUtils.getISO1Language(locale), LocaleUtils.getISO1Language(alternativeLocale));
    }
}


