/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package cat.lib.utils;

import cat.lib.errors.ErrorEx;

/**
 *
 * @author Miquel Abadal
 * @email miquel.abadal@gmail.com
 *
 */
public class ValidaUtils {

    private static String nifCharSet = "TRWAGMYFPDXBNJZSQVHLCKE";
    private static String cifCharSet = "ABCDEFGHJNPQRSUVW";
    private static String idCifCharSet = "NPQRSW";
    private static String cifLlletraTable ="ABCDEFGHIJ";
    private static String nieCharSet = "XTYZ";
    private static String dniCharSet = "0123456789";
    private static String nifNieLletraTable= "TRWAGMYFPDXBNJZSQVHLCKE";


    private static void validChars (String text, String validChars, ErrorEx error) {

        int cont1 = 0;
        while (cont1<text.length() && error.getError()==null) {

            if (!SetUtils.charIn(text.charAt(cont1), StringUtils.CASE_INSENSITIVE, validChars)) {
                error.setError("VALIDATE_ERROR", "El código tiene caràcteres no válidos");
            }
            cont1++;
        }
    }


    //  ALGORITME DE VALIDACIO DE CIF:
    //  ------------------------------
    //
    //  Validació CIF: El CIF es pot entrar utilitzant els símbols '.' i '-' per a separar
    //  les diferents seccions del codi, però a l'hora de validar-lo, només es tenen en compte
    //  els caràcters numérics i alfanumérics vàlids.
    //   ___________________________________
    //  |   |   |   |   |   |   |   |   |   |
    //  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |
    //  |___|___|___|___|___|___|___|___|___|
    //
    //  bit 0: Identificador de tipus de CIF
    //       'A' : 'Sociedades Anónimas.'
    //       'B' : 'Sociedades de responsabilidad limitada.'
    //       'C' : 'Societat col.lectiva.'
    //       'D' : 'Societat comanditaria.'
    //       'E' : 'Comunitat de bens.'
    //       'F' : 'Societat cooperativa.'
    //       'G' : 'Asociació i altres tipus no definits.'
    //       'H' : 'Comunitat de propietaris.'
    //       'K' : 'Format antic.'
    //       'L' : 'Format antic.'
    //       'M' : 'Format antic.'
    //       'N' : 'Format antic.'
    //       'P' : 'Corporació local.'
    //       'Q' : 'Organisme autónom.'
    //       'S' : 'Órgan de la administració.'
    //       'U' : ''
    //       'V' : ''
    //       'W' : ''
    //
    //  bit 1-7: sumaParells = E (bits_parells)                   --> Sumatori del valor dels dígits parells
    //           sumaSenars = E ( (valor div 10)+(valor mod 10) ) --> Sumatori del valor dels dígits senars, tenint en compte que si donen un nombre de 2 dígits
    //                                                                aquest també es sumen per donar un únic nombre d'un sol dígit
    //           digitControl = 10-últimDigit (sumaParells+sumaSenars)
    //           --> El dígit de control calculat dóna un nombre entre 1 i 10.
    //           --> Si el CIF correspon a una empresa espanyola, el dígit de control és numéric
    //               i ha de coincidir amb el valor obtingut (tenint en compte que el 10 es codifica com a 0).
    //           --> Si el CIF correspon a una empresa extrangera o un organisme públic o religios (codis 'N','P','Q','R','S','W'), el dígit de control és alfanuméric
    //               i es codifica seguint la taula següent: 0=A, 1=B, 2=C... 10=J
    //
    //  bit 8: Digit de control
    //
    //  Evaluació d'ejemple del CIF: W-67.982.658 o W67982652
    //  sumaParells = 7+8+6 = 21
    //      D1 = (6*2) = 12 = 1 + 2 = 3
    //      D3 = (9*2) = 18 = 1 + 8 = 9
    //      D5 = (2*2) = 04 = 0 + 4 = 4
    //      D7 = (5*2) = 10 = 1 + 0 = 1
    //  sumaSenars = D1 + D3 + D5 +D7 = 3 + 9 + 4 + 1 = 17
    //  digitControl = ultimDigit (sumaSenars + sumaParells) = ultimDigit (17 + 21) = ultimDigit (38) = 8
    //  digitControl = 10 - digitControl = 10 - 8 = 2
    //
    public static void validCIF (String text, ErrorEx error) {

        text = StringUtils.trim (StringUtils.toUpperCase(text));

        if (!StringUtils.isEmpty(text) && error.getError()==null) {

            // Verifica que només existeixin els caràcters permesos.
            validChars (StringUtils.copy (text, 1, -1), dniCharSet+cifCharSet+".-", error);

            if (error.getError()==null) {

                // Verifica que el primer caràcter sigui la lletra del CIF.
                if (SetUtils.charIn (text.charAt (0), StringUtils.CASE_INSENSITIVE, cifCharSet)) {

                    char idCIF = text.charAt (0);
                    text = StringUtils.copy (text, 1, -1);

                    // Elimina els caracters prescindibles "." i "-".
                    String id = "";
                    for (int cont1=0; cont1<text.length(); cont1++) {
                        if (SetUtils.charIn (text.charAt (cont1), StringUtils.CASE_INSENSITIVE, dniCharSet+nieCharSet+nifCharSet)) {id = id + text.charAt (cont1);}
                    }

                    // Verifica que hi hagui 8 caracters numérics i una lletra. }
                    if (id.length()!=8) { error.setError ("CIF_ERROR", "2"); }

                    if (error.getError()==null) {

                        // Calcula el dígit de control.
                        char digitControl = id.charAt (id.length()-1);
                        id = StringUtils.copy  (id, 0, id.length()-1);
                        int parells = 0;
                        int senars = 0;
                        int valor = 0;

                        for (int cont1=0; cont1<id.length(); cont1++) {

                            valor = ((int) id.charAt (cont1)) - ((int) '0');
                            if ((cont1 % 2)==0) {
                                valor = valor * 2;
                                valor = (valor/10)+(valor % 10);
                                senars = senars + valor;
                            }
                            else {parells = parells + valor;}
                        }

                        valor = parells + senars;
                        valor = valor - ((valor/10)*10);
                        valor = 10-valor;

                        int control = ((int) digitControl) - ((int) '0');
                        if (!SetUtils.charIn (idCIF, StringUtils.CASE_INSENSITIVE, idCifCharSet)) {

                            valor = valor - ((valor/10)*10);

                            // El dígit control acaba amb número i per tant
                            // es tracta del CIF d'una empresa espanyola.

                            if (valor!=control) { error.setError ("CIF_ERROR", "8"); }
                        }
                        else {

                            // El dígit control acaba amb lletra i per tant
                            //es tracta del CIF d'una empresa no espanyola
                            // o d'un organisme oficial.
                            if (digitControl!=cifLlletraTable.charAt (valor) && valor!=control) { error.setError ("CIF_ERROR", "9"); }
                        }
                    }
                }
                else { error.setError ("CIF_ERROR", "10"); }
            }
            else {  error.setError ("CIF_ERROR", "12"); }
        }

    }


    //  ALGORITME DE VALIDACIO DE NIF/NIE:
    //  ----------------------------------
    //
    //  Validació CIF: El CIF es pot entrar utilitzant els símbols '.' i '-' per a separar
    //  les diferents seccions del codi, però a l'hora de validar-lo, només es tenen en compte
    //  els caràcters numérics i alfanumérics vàlids.
    //   ___________________________________
    //  |   |   |   |   |   |   |   |   |   |
    //  | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |
    //  |___|___|___|___|___|___|___|___|___|
    //
    //  bit 0: Si es tracta d'un NIE (Número identificació extrangers) conté una 'X', 'Y', 'Z' o 'T'.
    //  bit 1-7 (Número NIE) o bit 0-7 (Número NIF)
    //           digitControl = (Número mod 23) +1
    //           Si es tracta d'un NIE comentçat amb 'Y': digitControl = ((Número+10.000.000) mod 23) +1
    //           Si es tracta d'un NIE comentçat amb 'Z': digitControl = ((Número+20.000.000) mod 23) +1
    //           --> EL dígit de control retorna un valor entre 1..23 que es converteix a ASCII utilitzant
    //               la següent taula de codificació: [TRWAGMYFPDXBNJZSQVHLCKE].
    //  bit 8: Lletra de control
    //
    public static void validNIF_NIE (String text, ErrorEx error) {

        text = StringUtils.trim (StringUtils.toUpperCase(text));

        if (!StringUtils.isEmpty(text) && error.getError()==null) {

            validChars (text, dniCharSet+nifCharSet+nieCharSet+".-", error);
            if (error.getError()==null) {

                // Comproba que el primer caràcter sigui el d'un NIE o el últim el d'un NIF.
                if ((SetUtils.charIn (text.charAt (0), StringUtils.CASE_INSENSITIVE, nieCharSet) ||
                     SetUtils.charIn (text.charAt (0), StringUtils.CASE_INSENSITIVE, dniCharSet))) {

                    // Elimina els caracters prescindibles "." i "-".
                     String id = "";
                     for (int cont1=0; cont1<text.length(); cont1++) {
                         if (SetUtils.charIn (text.charAt (cont1), StringUtils.CASE_INSENSITIVE, dniCharSet+nieCharSet+nifCharSet)) {id = id + text.charAt (cont1);}
                     }

                     // Si es tracta d'un NIE, cal ignorar el primer caràcter 'X' o 'T' i tractar la resta com si fos un nif.
                     char nieLletra = '\0';
                     if (SetUtils.charIn (id.charAt (0), StringUtils.CASE_INSENSITIVE, nieCharSet)) {
                          nieLletra = id.charAt (0);
                          id = StringUtils.copy (id, 2, -1);
                     }

                     // Verifica que l'últim caràcter sigui la lletra del NIF.
                     int num1 = id.length()-1;
                     if (num1>0 && (SetUtils.charIn (id.charAt (num1), StringUtils.CASE_INSENSITIVE, nifCharSet))) {

                          String dni = StringUtils.copy (id, 0, num1);
                          char lletra = id.charAt (num1);

                          // Verifica que hi hagui 8 caracters numérics i una lletra.
                          if (dni.length()<7 || dni.length()>8) { error.setError("NIF_ERROR", "2"); }
                          if (!SetUtils.charIn (lletra, StringUtils.CASE_INSENSITIVE, nifCharSet)) { error.setError("NIF_ERROR", "3"); }

                          if (error.getError()==null) {
                               // Calcula la lletra de control.
                               int value = StringToNumber.intValue (dni);
                               if (nieLletra=='Y') { value = value + 10000000; }
                               if (nieLletra=='Z') { value = value + 20000000; }
                               value = value % 23;
                               if (lletra!=nifNieLletraTable.charAt (value)) { error.setError("NIF_ERROR", "4"); }
                          }
                     }
                     else { error.setError("NIF_ERROR", "3"); }
                }
                else { error.setError("NIF_ERROR", "10"); }
            }
            else { error.setError("NIF_ERROR", "12"); }
        }

    }

    public static void validNIF_NIE_CIF (String text, ErrorEx error) {

        text = StringUtils.trim (StringUtils.toUpperCase (text));

        if (!StringUtils.isEmpty(text) && error.getError()==null) {
            if (SetUtils.charIn (text.charAt(0), StringUtils.CASE_INSENSITIVE, cifCharSet)) {validCIF (text, error);}
            else { validNIF_NIE (text, error); }
        }
    }

    public static boolean validaEmail (String text, ErrorEx error) {

        boolean res = false;

        if (text!=null) {
            int arrobaPos = StringUtils.charPos (text, '@', StringUtils.FROM_BEGIN, StringUtils.CASE_SENSITIVE);
            int serverPos = StringUtils.charPos (text, '.', arrobaPos, StringUtils.CASE_SENSITIVE);

            if (arrobaPos!=-1 && serverPos!=-1 && serverPos>(arrobaPos+1) && serverPos<(text.length()-1)) { res = true; }
            else { error.setError ("EMAIL_ERROR", "DESCRIPTION: El email indicat no és vàlid.", "EMAIL: "+text); }
        }
        
        return res;
    }

    private static int charToInt (char caract, ErrorEx error) {

        int res = -1;
        if (caract>='0' && caract<='9') { res = caract - '0'; }
        else { error.setError ("NOT_NUMERIC_ERROR", "DESCRIPTION: El caràcter no és dígit numéric", "CHAR: "+caract); }

        return res;
    }

    public static boolean validaCc (String cc, ErrorEx error) {

        boolean res = false;

        if (cc!=null) {
            if (cc.length()==20) {
                int digit1 = charToInt (cc.charAt (0), error)*7;
                digit1 += charToInt (cc.charAt (1), error)*3;
                digit1 += charToInt (cc.charAt (2), error)*6;
                digit1 += charToInt (cc.charAt (3), error)*1;
                digit1 += charToInt (cc.charAt (4), error)*2;
                digit1 += charToInt (cc.charAt (5), error)*4;
                digit1 += charToInt (cc.charAt (6), error)*8;
                digit1 += charToInt (cc.charAt (7), error)*5;

                digit1 = digit1 % 11;
                if (digit1==10) { digit1 = 1; }

                int digit2 = charToInt (cc.charAt (10), error)*10;
                digit2 += charToInt (cc.charAt (11), error)*9;
                digit2 += charToInt (cc.charAt (12), error)*7;
                digit2 += charToInt (cc.charAt (13), error)*3;
                digit2 += charToInt (cc.charAt (14), error)*6;
                digit2 += charToInt (cc.charAt (15), error)*1;
                digit2 += charToInt (cc.charAt (16), error)*2;
                digit2 += charToInt (cc.charAt (17), error)*4;
                digit2 += charToInt (cc.charAt (18), error)*8;
                digit2 += charToInt (cc.charAt (19), error)*5;

                digit2 = digit2 % 11;
                if (digit2==10) { digit2 = 1; }

                if (charToInt (cc.charAt (8), error)==digit1 && charToInt (cc.charAt (9), error)==digit2) { res = true; }
                else { error.setError ("COMPTE_BANCARIA_ERROR", "DESCRIPTION: El dígit de control de la compte bancària no és vàlid", "CC: "+cc); }
            }
            else { error.setError ("COMPTE_BANCARIA_ERROR", "DESCRIPTION: La compte bancària no és vàlida (no té 20 dígits).", "CC: "+cc); }
        }
        else { error.setError ("COMPTE_BANCARIA_ERROR", "DESCRIPTION: La compte bancària no és vàlida.", "CC: NULL"); }


        return res;
    }
}
